"use strict";

let { classes: Cc, interfaces: Ci, results: Cr, utils: Cu } = Components;
Cu.import("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative(this, "base/arAppInfo.jsm");
XPCOMUtils.importRelative(this, "base/arFileUtil.jsm");
XPCOMUtils.importRelative(this, "base/arMIMEDecoder.jsm");
XPCOMUtils.importRelative(this, "base/arUconv.jsm");
XPCOMUtils.importRelative(this, "UnMHTExtractor.jsm");
XPCOMUtils.importRelative(this, "UnMHTFormatter.jsm");
XPCOMUtils.importRelative(this, "UnMHTHTMLUtil.jsm");
XPCOMUtils.importRelative(this, "UnMHTScheme.jsm");
XPCOMUtils.importRelative(this, "UnMHTStringBundle.jsm");

let EXPORTED_SYMBOLS = ["UnMHTCachePage"];

/**
 * キャッシュページ
 */
let UnMHTCachePage = {
  /**
   * キャッシュページを作成する
   *
   * @returns {string}
   *          キャッシュページのソース
   */
  createCachePage: function() {
    let tableRow = function(header, data, id) {
      return ("  " + (id ? ("<tr id=\"" + id + "_row\">") : "<tr>")
              + (id ? ("<th id=\"" + id + "_header\">") : "<th>") + header + "</th>"
              + (id ? ("<td id=\"" + id + "_data\">") : "<td>") + data + "</td>"
              + "</tr>");
    };
    let _ee = function(text) UnMHTHTMLUtil.escapeEntity(text);
    let bundle = function(name) _ee(UnMHTStringBundle.get(name));
    
    let size = 0;
    
    let eFileInfoList = UnMHTExtractor.getFileInfoList();
    
    let bodies
      = eFileInfoList.map(function(eFileInfo, i) {
        let body = [];
        
        let _row = function(header, data, id) {
          if (id) {
            id = "entry_" + i + "_" + id;
          }
          body.push(tableRow(header, data, id));
        };
        
        size += eFileInfo.size;
        
        let param = UnMHTScheme.getUnMHTURIParam(eFileInfo.baseURI);
        let original = param.original;
        if (param.scheme == "file") {
          original = arFileUtil.getFileFromURLSpec(original).path;
        }
        
        body.push("<table class=\"file\">");
        
        _row("URI: ",
             "<a href=\"" + _ee(eFileInfo.baseURI) + "\">"
             + _ee(eFileInfo.baseURI)
             + "</a>",
             "uri");
        _row(bundle("info.original_address") + " ",
             _ee(original),
             "original_uri");
        _row(bundle("cache.opened_time") + " ",
             _ee(UnMHTFormatter.getReadableDate(eFileInfo.extractDate)),
             "opened_time");
        _row(bundle("cache.last_access_time") + " ",
             _ee(UnMHTFormatter.getReadableDate(eFileInfo.lastAccessDate)),
             "access_time");
        _row(bundle("cache.file_count") + " ",
             eFileInfo.parts.length,
             "part_count");
        
        if (eFileInfo.date) {
          let savedTime = eFileInfo.date;
          let date = arMIMEDecoder.decodeDate(savedTime);
          if (date) {
            savedTime = UnMHTFormatter.getReadableDate(date);
          }
          
          _row(bundle("info.saved_date") + " ",
               _ee(savedTime),
               "saved_date");
        }
        
        if (eFileInfo.subject) {
          _row("Subject: ",
               _ee(eFileInfo.subject),
               "subject");
        }
        
        body.push("</table>");
        
        eFileInfo.parts.forEach(function(part, j) {
          let _row2 = function(header, data, id) {
            if (id) {
              id = "part_" + j + "_" + id;
            }
            _row(header, data, id);
          };

          body.push("<table class=\"part\">");
          _row2(bundle("cache.ref_name") + " ",
                "<a href=\""
                + _ee(eFileInfo.baseURI) + _ee(part.eParam.refName)
                + "\">"
                + _ee(part.eParam.refName)
                + "</a>"
                + ((eFileInfo.startPart == part)
                   ? (" (" + bundle("cache.start_part") + ")")
                   : ""),
                "refname");
          if (part.contentLocation) {
            _row2("Content-Location: ",
                  _ee(part.contentLocation),
                  "content_location");
          }
          if (part.contentID) {
            _row2("Content-ID: ",
                  _ee(part.contentID),
                  "content_id");
          }
          if (part.contentDispositionType &&
              part.contentDispositionFilename) {
            _row2("Content-Disposition: ",
                  _ee(part.contentDispositionType)
                  + "; filename=" + _ee(part.contentDispositionFilename),
                  "content_disposition");
          }
          _row2("Content-Type: ",
                _ee(part.eParam.mimetype)
                + (part.eParam.charset
                   ? ("; charset=" + _ee(part.eParam.charset))
                   : ""),
                "content-type");
          if (part.contentTransferEncoding) {
            _row2("Content-Encoding: ",
                  _ee(part.contentTransferEncoding),
                  "content-encoding");
          }
          _row2(bundle("cache.file_size") + " ",
                UnMHTFormatter.getReadableSize(part.eParam.content.length)
                + " " + bundle("cache.bytes"),
                "file_size");
          body.push("</table>");
        });
        
        return body.join("\n");
      });
    
    let data
      = "<!DOCTYPE html>" + "\n"
      + "<html xmlns=\"http://www.w3.org/1999/xhtml\">" + "\n"
      + "<head>" + "\n"
      + "  <title>" + bundle("cache.title") + "</title>" + "\n"
      + "  <style type=\"text/css\">" + "\n"
      + "html {" + "\n"
      + (arAppInfo.isFennec
         ? ("  padding: 0" + "\n")
         : ("  background: -moz-Dialog;" + "\n"
            + "  padding: 0 1em;" + "\n"
            + "  font: message-box;" + "\n"))
      + "}" + "\n"
      + "\n"
      + "body {" + "\n"
      + (arAppInfo.isFennec
         ? ("  color: #000000;" + "\n"
            + "  background: #ffffff;" + "\n"
            + "  margin: 0;" + "\n"
            + "  padding: 0;" + "\n")
         : ("  color: -moz-FieldText;" + "\n"
            + "  min-width: 330px;" + "\n"
            + "  max-width: 80%;" + "\n"
            + "  margin: 4em auto;" + "\n"
            + "  border: 1px solid ThreeDShadow;" + "\n"
            + "  border-radius: 10px;" + "\n"
            + "  padding: 3em;" + "\n"
            + "  -moz-padding-start: 30px;" + "\n"
            + "  background: -moz-Field;" + "\n"))
      + "}" + "\n"
      + "\n"
      + (arAppInfo.isFennec
         ? ("h1 {" + "\n"
            + "  margin: 0;" + "\n"
            + "}" + "\n")
         : "")
      + "table {" + "\n"
      + "  width: 100%;" + "\n"
      + "  padding: 0.5em;" + "\n"
      + "}" + "\n"
      + "\n"
      + "th {" + "\n"
      + "  width: 12em;" + "\n"
      + "  word-wrap: break-word;" + "\n"
      + "  vertical-align: top;" + "\n"
      + "  text-align: end;" + "\n"
      + "  -moz-padding-end: 0.5em;" + "\n"
      + "}" + "\n"
      + "\n"
      + "td {" + "\n"
      + (arAppInfo.isFennec
         ? ""
         : ("  display: block;" + "\n"))
      + "  font-family: -moz-fixed;" + "\n"
      + "  white-space: pre-wrap;" + "\n"
      + "}" + "\n"
      + "\n"
      + "#entries {" + "\n"
      + "  border-radius: 10px;" + "\n"
      + "  background-color: rgba(0, 0, 0, .1);" + "\n"
      + "}" + "\n"
      + "\n"
      + "table.file {" + "\n"
      + "  border-radius: 10px 10px 0px 0px;" + "\n"
      + "  background-color: rgba(0, 0, 0, .1);" + "\n"
      + "}" + "\n"
      + "\n"
      + "table.part:nth-child(odd) {" + "\n"
      + "  background-color: rgba(0, 0, 0, .03);" + "\n"
      + "}" + "\n"
      + "\n"
      + "table.part:nth-child(even) {" + "\n"
      + "  background-color: rgba(0, 0, 0, .06);" + "\n"
      + "}" + "\n"
      + "  </style>"
      + "</head>" + "\n"
      + "<body>" + "\n"
      + "<h1>" + bundle("cache.title") + "</h1>" + "\n"
      + "<table id=\"entries\">" + "\n"
    
      + tableRow(bundle("cache.entry") + " ",
                 eFileInfoList.length,
                 "cache_count") + "\n"
    
      + tableRow(bundle("cache.size") + " ",
                 UnMHTFormatter.getReadableSize(size)
                 + " " + bundle("cache.bytes"),
                 "cache_size") + "\n"
    
      + "</table>" + "\n"
      + "<hr />" + "\n"
      + bodies.join("<hr />") + "\n"
      + "</body>" + "\n"
      + "</html>" + "\n";
    
    return arUconv.fromUnicode(data, "utf-8");
  }
};
