"use strict";

let { classes: Cc, interfaces: Ci, results: Cr, utils: Cu } = Components;
Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative(this, "base/arAlertUtil.jsm");
XPCOMUtils.importRelative(this, "base/arShutdownEvent.jsm");
XPCOMUtils.importRelative(this, "base/arTimer.jsm");
XPCOMUtils.importRelative(this, "UnMHTPref.jsm");
XPCOMUtils.importRelative(this, "UnMHTScheme.jsm");
XPCOMUtils.importRelative(this, "UnMHTStringBundle.jsm");
XPCOMUtils.defineLazyGetter(this, "UnMHTExtractor", function() {
  XPCOMUtils.importRelative(this, "UnMHTExtractor.jsm");
  return UnMHTExtractor;
});

let EXPORTED_SYMBOLS = ["UnMHTFennecPrefPage"];

/**
 * アプリケーション起動以外のスタートアップ時に
 * タブにある MHT ファイルを展開する
 *
 * @implements nsIObserver
 * @implements arIStartupEventListener
 * @implements arIShutdownEventListener
 */
let UnMHTFennecPrefPage = {
  QueryInterface:    XPCOMUtils.generateQI([Ci.nsIObserver]),
  
  /* ==== nsIObserver インターフェース ==== */
  
  /**
   * 通知を受けたイベント
   *
   * @param   {nsISupports} subject
   *          変更やアクションのあったオブジェクト
   * @param   {string} topic
   *          イベントの種類
   * @param   {string} data
   *          イベントのデータ
   */
  observe: function(subject, topic, data) {
    if (topic == "addon-options-displayed") {
      let targetDocument = subject;
      try {
        this._onAddonOptionsDisplayed(targetDocument);
      } catch (e) {
        Cu.reportError(e + "\n" + e.stack);
      }
    }
  },
  
  /* ==== arIStartupEventListener インターフェース ==== */
  
  /**
   * スタートアップ時のイベント
   *
   * @param   {object} data
   *          Bootstrap データ
   * @param   {number} reason
   *          理由
   */
  onStartup: function(data, reason) {
    Services.obs.addObserver(this, "addon-options-displayed", false);
    
    arShutdownEvent.addListener(this);
  },

  /* ==== arIShutdownEventListener ==== */
  
  /**
   * シャットダウンイベント
   *
   * @param   {object} data
   *          Bootstrap データ
   * @param   {number} reason
   *          理由
   */
  onShutdown: function(data, reason) {
    Services.obs.removeObserver(this, "addon-options-displayed");
    this._removeAllEventListener();
    this._removeAllWatch();
  },
  
  /* ==== private ==== */
  
  events: [],  /** 追加したイベントハンドラ
                *  @type {Array.<[nsIDOMDocument, nsIDOMElement, string, Function, boolean]>} */
  watches: [], /** 追加したウォッチポイント
                *  @type {Array.<[nsIDOMDocument, Object, string, Function]>} */
  
  /**
   * 設定が表示されたイベント
   *
   * @param   {nsIDOMDocument} targetDocument
   *          対象のドキュメント
   */
  _onAddonOptionsDisplayed: function(targetDocument) {
    let f = function() {
      this._removeDocumentsEventListener(targetDocument);
      this._removeDocumentsWatch(targetDocument);
    }.bind(this);
    this._addEventListener(targetDocument, targetDocument.defaultView, "beforeunload", f, true);
    
    let initMenu = function(pref) {
      let menulist = targetDocument.getElementById("ui-" + pref);
      let items = menulist.getElementsByTagName("menuitem");
      let value = UnMHTPref.getPref(pref);
      let selectedIndex = -1;
      Array.forEach(items, function(item, i) {
        if (item.getAttribute("value") == value) {
          selectedIndex = i;
        }
      });
      if (selectedIndex != -1) {
        /* このイベント中に menulist.selectedIndex を設定すると
         * 表示がおかしくなる */
        arTimer.setTimeout(function() {
          menulist.selectedIndex = selectedIndex;
        }, 100);
      }
    };
    
    let hookMenu = function(pref) {
      let menulist = targetDocument.getElementById("ui-" + pref);
      let items = menulist.getElementsByTagName("menuitem");
      this._addWatch(targetDocument, menulist, "selectedIndex", function(prop, oldval, newval) {
        if (newval >= 0 && newval < items.length) {
          UnMHTPref.setPref(pref, items[newval].getAttribute("value"));
        }
        return newval;
      });
    }.bind(this);
    
    [
      ["save.contents.video", "ask", "save"],
      ["save.contents.embed", "ask", "save"],
      ["save.quick.dup.action", "ask", "rename"],
      ["save.import.limit.exceeded.action", "ask", "ignore"]
    ].forEach(function([name, from, to]) {
      let value = UnMHTPref.getPref(name);
      if (value == from) {
        UnMHTPref.setPref(name, to);
      }
    });
    
    let menuIDs = [
      "open.multiple.action",
      "save.contents.video",
      "save.contents.embed",
      "save.insert.link.pos",
      "save.quick.state",
      "save.quick.dup.action",
      "save.filename",
      "save.filename.dup.rename.init",
      "save.import.limit.exceeded.action"
    ];
    menuIDs.forEach(function(name) {
      initMenu(name);
      hookMenu(name);
    });
    
    let node;
    
    node = targetDocument.getElementById("ui-open.cache.clear");
    this._addEventListener(targetDocument, node, "command", function() {
      let window = Services.wm.getMostRecentWindow("navigator:browser");
      if (!window.confirm(UnMHTStringBundle.get("cache.clear.confirm"))) {
        return;
      }
      UnMHTExtractor.removeOldFileInfo(0);
      
      arAlertUtil.toast(window, UnMHTStringBundle.get("cache.clear.done"), "short");
    }, true);
    
    node = targetDocument.getElementById("ui-open.cache.show");
    this._addEventListener(targetDocument, node, "command", function() {
      let window = Services.wm.getMostRecentWindow("navigator:browser");
      window.BrowserApp.addTab(UnMHTScheme.createCacheURI());
    }, true);

    node = targetDocument.getElementById("ui-reset");
    this._addEventListener(targetDocument, node, "command", function() {
      let window = Services.wm.getMostRecentWindow("navigator:browser");
      if (!window.confirm(UnMHTStringBundle.get("pref.reset.confirm"))) {
        return;
      }
      UnMHTPref.names
        .forEach(function(name) {
          UnMHTPref.clearPref(name);
          menuIDs.forEach(function(name) {
            initMenu(name);
          });
        });
    }, true);
  },

  /**
   * イベントリスナを追加する
   *
   * @param   {nsIDOMDocument} targetDocument
   *          関連付けるドキュメント
   * @param   {nsIDOMElement} node
   *          対象の要素
   * @param   {string} type
   *          イベントの種類
   * @param   {Function} listener
   *          イベントリスナ
   * @param   {boolean} useCapture
   *          キャプチャするか
   */
  _addEventListener: function(targetDocument, node, type, listener, useCapture) {
    this.events.push([targetDocument, node, type, listener, useCapture]);
    node.addEventListener(type, listener, useCapture);
  },

  /**
   * ウォッチポイントを追加する
   *
   * @param   {nsIDOMDocument} targetDocument
   *          関連付けるドキュメント
   * @param   {Object} obj
   *          対象のオブジェクト
   * @param   {string} prop
   *          プロパティ
   * @param   {Function} listener
   *          イベントリスナ
   */
  _addWatch: function(targetDocument, obj, prop, listener) {
    this.watches.push([targetDocument, obj, prop, listener]);
    obj.watch(prop, listener);
  },

  /**
   * イベントリスナを削除する
   *
   * @param   {nsIDOMDocument} targetDocument
   *          関連付けるドキュメント
   * @param   {nsIDOMElement} node
   *          対象の要素
   * @param   {string} type
   *          イベントの種類
   * @param   {Function} listener
   *          イベントリスナ
   * @param   {boolean} useCapture
   *          キャプチャするか
   */
  _removeEventListener: function(targetDocument, node, type, listener, useCapture) {
    this.events = this.events.filter(function([d, n, t, l, c]) {
      return !(targetDocument == d && node == n && type == t && listener == l && useCapture == c);
    });
    try {
      node.removeEventListener(type, listener, useCapture);
    } catch (e) {
      Cu.reportError(e);
    }
  },

  /**
   * ウォッチポイントを削除する
   *
   * @param   {nsIDOMDocument} targetDocument
   *          関連付けるドキュメント
   * @param   {Object} obj
   *          対象のオブジェクト
   * @param   {string} prop
   *          プロパティ
   * @param   {Function} listener
   *          イベントリスナ
   */
  _removeWatch: function(targetDocument, obj, prop, listener) {
    this.watches = this.watches.filter(function([d, o, p, l]) {
      return !(targetDocument == d && obj == o && prop == p && listener == l);
    });
    try {
      obj.unwatch(prop);
    } catch (e) {
      Cu.reportError(e);
    }
  },
  
  /**
   * ドキュメントに関連付けられたイベントリスナを削除する
   *
   * @param   {nsIDOMDocument} targetDocument
   *          対象のドキュメント
   */
  _removeDocumentsEventListener: function(targetDocument) {
    this.events.slice().forEach(function([d, node, type, listener, useCapture]) {
      if (d == targetDocument) {
        this._removeEventListener(d, node, type, listener, useCapture);
      }
    }.bind(this));
  },

  /**
   * ドキュメントに関連付けられたウォッチポイントを削除する
   *
   * @param   {nsIDOMDocument} targetDocument
   *          対象のドキュメント
   */
  _removeDocumentsWatch: function(targetDocument) {
    this.watches.slice().forEach(function([d, obj, prop, listener]) {
      if (d == targetDocument) {
        this._removeWatch(d, obj, prop, listener);
      }
    }.bind(this));
  },

  /**
   * すべてのイベントリスナを削除する
   */
  _removeAllEventListener: function() {
    this.events.slice().forEach(function([d, node, type, listener, useCapture]) {
      this._removeEventListener(d, node, type, listener, useCapture);
    }.bind(this));
  },

  /**
   * すべてのウォッチポイントを削除する
   */
  _removeAllWatch: function() {
    this.watches.slice().forEach(function([d, obj, prop, listener]) {
      this._removeWatch(d, obj, prop, listener);
    }.bind(this));
  }
};
