"use strict";

let { classes: Cc, interfaces: Ci, results: Cr, utils: Cu } = Components;
Cu.import("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative(this, "base/Task.jsm");
XPCOMUtils.importRelative(this, "base/arFileUtil.jsm");
XPCOMUtils.importRelative(this, "base/arMIMEEncoder.jsm");
XPCOMUtils.importRelative(this, "base/arMIMEParam.jsm");
XPCOMUtils.importRelative(this, "base/arMIMEPart.jsm");
XPCOMUtils.importRelative(this, "base/arMIMEUtil.jsm");
XPCOMUtils.importRelative(this, "base/arTimer.jsm");
XPCOMUtils.importRelative(this, "UnMHTSave.jsm");
XPCOMUtils.importRelative(this, "UnMHTVersion.jsm");

let EXPORTED_SYMBOLS = ["UnMHTFileConverter"];

/**
 * 変換コンテキスト
 *
 * @param   {Array.<nsIFile>} files
 *          対象の HTML ファイル
 * @param   {string} suffix
 *          ディレクトリ名の末尾
 * @param   {UnMHTFileConverterListener} listener
 *          イベントリスナ
 */
function UnMHTFileConvertContext(files, suffix, listener) {
  this._files
    = files
    .map(function(file) {
      if (/\.mht(ml)?$/i.test(file.path)) {
        return [null, null, null];
      }
      
      let baseDir = file.parent.clone();
      let filenameBase = file.leafName.replace(/\.[^\.]+$/, "");
      
      let findDir = function(suffix) {
        let d = baseDir.clone();
        d.append(filenameBase + suffix);
        if (d && d.exists() && d.isDirectory()) {
          return d;
        }
        return null;
      };
      
      let dir = findDir(suffix) || findDir("_files") || findDir(".files");
      if (!dir) {
        listener.onNoResDir(this, file);
        return [null, null, null];
      }
      
      return [file, filenameBase, dir];
    }.bind(this))
    .filter(function([f,,]) f);
  this._listener = listener;
}
UnMHTFileConvertContext.prototype = {
  /**
   * 破棄
   * start 後は自分で呼ぶ
   */
  destruct: function() {
    this._files = null,
    this._listener = null;
  },
  
  /**
   * 変換開始
   */
  start: function() {
    this._convert();
  },
  
  /**
   * 変換中断
   */
  abort: function() {
    this._aborted = true;
  },
  
  /* ==== private ==== */
  
  _files: null,    /** 対象の HTML ファイル
                    *  @type {Array.<[nsIFile,string,nsIFile]>}
                    *    [nsIFile 対象の HTML ファイル,
                    *     string ファイル名のベース,
                    *     nsIFile リソースのディレクトリ] */
  _listener: null, /** イベントリスナ
                    *  @type {UnMHTFileConverterListener} */
  
  _aborted: false, /** 中断したか
                    *  @type {boolean} */
  
  /**
   * ファイルを変換する
   */
  _convert: function() {
    Task.spawn(function() {
      let converted = 0;
      
      for (let i = 0; i < this._files.length; i ++) {
        if (this._aborted) {
          this._listener.onAbort(this);
          this.destruct();
          return;
        }
        
        let [file, filenameBase, resDir] = this._files[i];
        
        let baseDir = file.parent.clone();
        
        let mhtFile = UnMHTSave.getUniqueFile(baseDir, filenameBase);
        
        this._listener.onStart(this, mhtFile, file, resDir,
                               i, this._files.length);
        
        let resourceFiles
          = arFileUtil.getDirectoryEntries(resDir).filter(function(f) f.isFile());
        
        if (UnMHTFileConverter.convertFile(mhtFile, file, resDir.leafName,
                                           resourceFiles, file.leafName)) {
          this._listener.onFinish(this, mhtFile, file, resDir,
                                  i, this._files.length);
          converted ++;
        } else {
          this._listener.onFail(this, mhtFile, file, resDir,
                                i, this._files.length);
        }
        
        yield arTimer.wait(100);
      }
      
      this._listener.onComplete(this, converted);
      this.destruct();
    }.bind(this));
  }
};

/**
 * HTML ファイルを MHT ファイルに変換
 */
let UnMHTFileConverter = {
  /**
   * 変換コンテキストを作成する
   *
   * @param   {Array.<nsIFile>} files
   *          対象の HTML ファイル
   * @param   {string} suffix
   *          ディレクトリ名の末尾
   * @param   {UnMHTFileConverterListener} listener
   *          イベントリスナ
   * @returns {UnMHTFileConvertContext}
   */
  createContext: function(files, suffix, listener) {
    return new UnMHTFileConvertContext(files, suffix, listener);
  },
  
  /**
   * HTML ファイルを MHT に変換
   *
   * @param   {nsIFile} mhtFile
   *          保存先のファイル
   * @param   {nsIFile} htmlFile
   *          対象の HTML ファイル
   * @param   {string} dirname
   *          他のリソースファイルを格納したディレクトリ名
   * @param   {Array.<nsIFile>} resourceFiles
   *          他のリソースファイル
   * @param   {string} subject
   *          Subject の値
   * @returns {boolean}
   *          変換に成功したか
   */
  convertFile: function(mhtFile, htmlFile, dirname, resourceFiles, subject) {
    let fstream = arFileUtil.openOutputStream(mhtFile);
    if (!fstream) {
      return false;
    }
    
    let topPart = new arMIMEPart();
    topPart.isMultipart = true;
    topPart.boundary = arMIMEUtil.createUniqueBoundary();
    topPart.startMimetype = "text/html";
    
    topPart.addField("From",
                     "<Saved by UnMHT Ver." + UnMHTVersion + ">");
    topPart.addField("Subject", subject);
    topPart.addField("Date",
                     arMIMEEncoder.encodeDate(new Date()));
    topPart.addField("MIME-Version", "1.0");
    let param = new arMIMEParam();
    param.type = "multipart";
    param.subtype = "related";
    param.params.set("boundary", topPart.boundary);
    param.params.set("type", topPart.startMimetype);
    topPart.setContentType(param);
    topPart.parts = [];
    
    [htmlFile, ...resourceFiles].forEach(function(file) {
      let ext = "";
      let re = file.leafName.match(/\.([^\.]+)(\?.+)?(#.+)?$/i);
      if (re) {
        ext = re[1];
      }
      let mimetype = arMIMEUtil.getTypeFromExtension(ext);
      
      let location;
      if (file == htmlFile) {
        location = file.leafName;
      } else {
        location = dirname + "/" + file.leafName;
      }
      
      let part = new arMIMEPart();
      part.contentTransferEncoding = "base64";
      let param = new arMIMEParam(mimetype);
      part.setContentType(param);
      part.addField("Content-Transfer-Encoding",
                    part.contentTransferEncoding);
      part.addField("Content-Location", location);
      
      part.body = arFileUtil.readData(file);
      
      topPart.parts.push(part);
    });
    
    arMIMEEncoder.encodeMessage(topPart, fstream);
    fstream.close();
    
    topPart.destruct();
    
    try {
      mhtFile.lastModifiedTime = htmlFile.lastModifiedTime;
    } catch (e) {
    }
    
    return true;
  }
};
