"use strict";

let { classes: Cc, interfaces: Ci, results: Cr, utils: Cu } = Components;
Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative(this, "base/arPrototype.jsm");
XPCOMUtils.importRelative(this, "base/arWindowHook.jsm");
XPCOMUtils.importRelative(this, "base/arWindowUtil.jsm");
XPCOMUtils.importRelative(this, "UnMHTDLFHandledLocations.jsm");
XPCOMUtils.importRelative(this, "UnMHTPref.jsm");
XPCOMUtils.importRelative(this, "UnMHTScheme.jsm");
XPCOMUtils.defineLazyGetter(this, "UnMHTFileLoader", function() {
  XPCOMUtils.importRelative(this, "UnMHTFileLoader.jsm");
  return UnMHTFileLoader;
});

let EXPORTED_SYMBOLS = ["UnMHTFileLoadHook"];

/**
 * ファイルロード用フック
 *
 * @param   {nsIDOMWindow} window
 *          対象のウィンドウ
 * @extends    arWindowHook
 * @implements arIWindowHook
 */
function UnMHTFileLoadHook(window) {
  arWindowHook.call(this, window);
  
  this._extensions = this._getExtensionList();
  
  this.onPageLoad = this.onPageLoad.bind(this);
}
UnMHTFileLoadHook.prototype = {
  /* ---- UI イベント ---- */
  
  /**
   * ページがロードされたイベント
   *
   * @param   {Event} event
   *          対象のイベント
   */
  onPageLoad: function(event) {
    let targetWindow = event.target.defaultView;
    let targetDocument = targetWindow.document;
    let location = targetDocument.location.href;
    
    if (UnMHTDLFHandledLocations.isRegistered(location)) {
      /* ハンドル済みの mht ファイル */
      UnMHTDLFHandledLocations.unregister(location);
      
      return;
    }
    
    if (this.isExtensionForOpen(location)) {
      /* 未ハンドルの mht ファイル */
      
      if (/^chrome:\/\/ietab\//.test(location)) {
        /* IETab がハンドル済み */
        return;
      }
      
      if (this.isHTML(targetDocument)) {
        /* HTML っぽい */
        return;
      }
      
      let loadParam = {
        window:         this._window,
        uri:            Services.io.newURI(location, null, null),
        targetWindow:   targetWindow,
        hasDocument:    true,
        replaceHistory: true,
        extract:        true,
        refName:        "",
        useCache:       true
      };
      UnMHTFileLoader.loadWithParam(loadParam);
      
      return;
    }
    
    if (location.slice(0, 6) != "unmht:") {
      return;
    }
    
    if (UnMHTScheme.isUnMHTURI(location)) {
      UnMHTFileLoader.onUnMHTDocumentLoaded(this._window,
                                            targetWindow, targetDocument,
                                            location);
      return;
    }
    
    let original = UnMHTScheme.deUnMHTURI570(location);
    if (original) {
      targetWindow.QueryInterface(Ci.nsIInterfaceRequestor)
        .getInterface(Ci.nsIWebNavigation)
        .loadURI(original, 0, null, null, null);
      return;
    }
  },
  
  /**
   * ハンドルする拡張子かどうかを返す
   *
   * @param   {string} location
   *          対象のドキュメントのアドレス
   * @returns {boolean}
   *          ハンドルする拡張子か
   */
  isExtensionForOpen: function(location) {
    if (!this._extensions.length) {
      return false;
    }
    
    let re = location.match(/^[^\?#]*[^\/\?#]\.([a-z]+)(?:[\?#]|$)/);
    if (re && this._extensions.indexOf(re[1].toLowerCase()) != -1) {
      return true;
    }
    
    return false;
  },

  /**
   * HTML っぽいかどうかを返す
   *
   * @param   {nsIDOMDocument} targetDocument
   *          対象のドキュメント
   * @returns {boolean}
   *          HTML っぽいか
   */
  isHTML: function(targetDocument) {
    if ("contentType" in targetDocument) {
      if (targetDocument.contentType == "text/html") {
        return true;
      }
    }
    
    return false;
  },
  
  /* ==== arIWindowHook インターフェース ==== */
  
  /**
   * 破棄
   */
  destruct: function() {
    arWindowHook.prototype.destruct.call(this);
  },
  
  /**
   * ウィンドウにロードする
   *
   * @param   {boolean} isStartup
   *          スタートアップイベントか
   * @param   {number} reason
   *          スタートアップの理由
   */
  load: function(isStartup, reason) {
    if (!arWindowUtil.isMailWindow(this._window)) {
      arWindowUtil.addPageLoadListener(this._window, this.onPageLoad);
    }
  },
  
  /**
   * ウィンドウからアンロードする
   *
   * @param   {boolean} isShutdown
   *          シャットダウンイベントか
   * @param   {number} reason
   *          シャットダウンの理由
   */
  unload: function(isShutdown, reason) {
    if (!arWindowUtil.isMailWindow(this._window)) {
      arWindowUtil.removePageLoadListener(this._window, this.onPageLoad);
    }
  },
  
  /* ==== private ==== */
  
  _extensions: null, /** 開く拡張子
                      *  @type {Array.<string>} */
  
  /**
   * ハンドルする拡張子のリストを返す
   *
   * @returns {Array.<string>}
   *          ハンドルする拡張子のリスト
   */
  _getExtensionList: function() {
    let tmp = UnMHTPref.getPref("open.extensions");
    let list = [];
    try {
      list = JSON.parse(tmp);
      if (!list.every(function(n) typeof(n) == "string")) {
        list = [];
      }
    } catch (e) {
    }
    
    return list;
  }
};
arPrototype.extend(UnMHTFileLoadHook, arWindowHook);
