"use strict";

let { classes: Cc, interfaces: Ci, results: Cr, utils: Cu } = Components;
Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative(this, "base/Promise.jsm");
XPCOMUtils.importRelative(this, "base/Task.jsm");
XPCOMUtils.importRelative(this, "base/arAlertUtil.jsm");
XPCOMUtils.importRelative(this, "base/arAppInfo.jsm");
XPCOMUtils.importRelative(this, "base/arFileUtil.jsm");
XPCOMUtils.importRelative(this, "base/arWindowUtil.jsm");
XPCOMUtils.defineLazyGetter(this, "arMIMEDecoder", function() {
  XPCOMUtils.importRelative(this, "base/arMIMEDecoder.jsm");
  return arMIMEDecoder;
});
XPCOMUtils.defineLazyGetter(this, "arNetFileLoader", function() {
  XPCOMUtils.importRelative(this, "base/arNetFileLoader.jsm");
  return arNetFileLoader;
});
XPCOMUtils.importRelative(this, "UnMHTDOMUtil.jsm");
XPCOMUtils.importRelative(this, "UnMHTDocumentCategory.jsm");
XPCOMUtils.importRelative(this, "UnMHTFilename.jsm");
XPCOMUtils.importRelative(this, "UnMHTPref.jsm");
XPCOMUtils.importRelative(this, "UnMHTSave.jsm");
XPCOMUtils.importRelative(this, "UnMHTScheme.jsm");
XPCOMUtils.importRelative(this, "UnMHTStatusbarProgress.jsm");
XPCOMUtils.importRelative(this, "UnMHTStringBundle.jsm");
XPCOMUtils.defineLazyGetter(this, "UnMHTMail", function() {
  XPCOMUtils.importRelative(this, "UnMHTMail.jsm");
  return UnMHTMail;
});
XPCOMUtils.defineLazyGetter(this, "UnMHTFilePicker", function() {
  XPCOMUtils.importRelative(this, "UnMHTFilePicker.jsm");
  return UnMHTFilePicker;
});

let EXPORTED_SYMBOLS = ["UnMHTFileSaver"];

/**
 * MHT で保存のラッパー
 * UI からの要求に応じてデータを取得して MHT での保存を開始する
 */
let UnMHTFileSaver = {
  /** saveWithParam のパラメータ
   *  @type {object} */
  PARAM_AUTO:      {},
  PARAM_CHOOSE:    {},
  PARAM_CURRENT:   {},
  PARAM_DIR:       {},
  PARAM_MAIL:      {},
  PARAM_OVERWRITE: {},
  
  /** エラー定数
   *  @type {number} */
  RESULT_OK:               0x00000000, /* 成功 */
  RESULT_FAIL:             0x10000001, /* 一般エラー */
  RESULT_CANCEL:           0x10000002, /* ユーザによるキャンセル */
  RESULT_NO_WINDOW:        0x20000101, /* window が無い */
  RESULT_NO_DOCUMENT:      0x20000102, /* targetDocument も scrapBookData も
                                        * 無い */
  RESULT_NO_SELECTION:     0x20000103, /* 選択範囲が無い
                                        * (targetSelection が
                                        *  PARAM_CURRENT の場合) */
  RESULT_NO_BASE_DIR:      0x20000104, /* baseDir が無い
                                        * (targetFile が
                                        *  this.PARAM_DIR の場合) */
  RESULT_NO_BROWSER:       0x20000105, /* ブラウザが無い */

  RESULT_NOT_A_FILE:       0x20000201, /* targetFile や baseDir が
                                        * ファイルではない */
  RESULT_NOT_A_LOCAL_FILE: 0x20000202, /* ドキュメントがローカルファイルでは
                                        * ない
                                        * (targetFile が 
                                        *  PARAM_OVERWRITE の場合) */
  RESULT_NOT_A_LINK:       0x20000203, /* リンクに載っていない */
  RESULT_NOT_A_MAIL:       0x20000204, /* ドキュメントがメールではない */
  RESULT_NOT_SAVABLE:      0x20000205, /* 保存可能ではない */
  
  /**
   * 現在のドキュメントを MHT で保存
   *
   * @param   {nsIDOMWindow} window
   *          実行中のウィンドウ
   * @returns {Promise<[nsIFile,Array.<string>]>}
   *          resolve: 保存したファイルと保存に失敗したファイルリスト
   *          reject: {number}
   *                  UnMHTSave.RESULT_*
   *                  UnMHTFileSaver.RESULT_*
   */
  save: function(window) {
    let param = {
      window:         window,
      targetDocument: this.PARAM_CURRENT,
    };
    return this.saveWithParam(param);
  },
  
  /**
   * 対象のドキュメントを MHT で保存
   *
   * @param   {nsIDOMWindow} window
   *          実行中のウィンドウ
   * @param   {nsIDOMDocument} targetDocument
   *          対象のドキュメント
   * @returns {Promise<[nsIFile,Array.<string>]>}
   *          resolve: 保存したファイルと保存に失敗したファイルリスト
   *          reject: {number}
   *                  UnMHTSave.RESULT_*
   *                  UnMHTFileSaver.RESULT_*
   */
  saveDocument: function(window, targetDocument) {
    let param = {
      window:         window,
      targetDocument: targetDocument,
    };
    return this.saveWithParam(param);
  },
  
  /**
   * 現在のドキュメントの選択範囲を保存
   *
   * @param   {nsIDOMWindow} window
   *          実行中のウィンドウ
   * @returns {Promise<[nsIFile,Array.<string>]>}
   *          resolve: 保存したファイルと保存に失敗したファイルリスト
   *          reject: {number}
   *                  UnMHTSave.RESULT_*
   *                  UnMHTFileSaver.RESULT_*
   */
  saveSelection: function(window) {
    let param = {
      window:          window,
      targetDocument:  this.PARAM_CURRENT,
      targetSelection: this.PARAM_CURRENT,
    };
    return this.saveWithParam(param);
  },

  /**
   * 対象のドキュメントの現在の選択範囲を MHT で保存
   *
   * @param   {nsIDOMWindow} window
   *          実行中のウィンドウ
   * @param   {nsIDOMDocument} targetDocument
   *          対象のドキュメント
   * @returns {Promise<[nsIFile,Array.<string>]>}
   *          resolve: 保存したファイルと保存に失敗したファイルリスト
   *          reject: {number}
   *                  UnMHTSave.RESULT_*
   *                  UnMHTFileSaver.RESULT_*
   */
  saveSelectionInDocument: function(window, targetDocument) {
    let param = {
      window:          window,
      targetDocument:  targetDocument,
      targetSelection: this.PARAM_CURRENT,
    };
    return this.saveWithParam(param);
  },
  
  /**
   * 現在のドキュメントを MHT に上書き保存
   *
   * @param   {nsIDOMWindow} window
   *          実行中のウィンドウ
   * @returns {Promise<[nsIFile,Array.<string>]>}
   *          resolve: 保存したファイルと保存に失敗したファイルリスト
   *          reject: {number}
   *                  UnMHTSave.RESULT_*
   *                  UnMHTFileSaver.RESULT_*
   */
  overwrite: function(window) {
    let param = {
      window:          window,
      targetDocument:  this.PARAM_CURRENT,
      targetFile:      this.PARAM_OVERWRITE,
    };
    return this.saveWithParam(param);
  },
  
  /**
   * リンク先を MHT で保存
   *
   * @param   {nsIDOMWindow} window
   *          実行中のウィンドウ
   * @returns {Promise<[nsIFile,Array.<string>]>}
   *          resolve: 保存したファイルと保存に失敗したファイルリスト
   *          reject: {number}
   *                  UnMHTSave.RESULT_*
   *                  UnMHTFileSaver.RESULT_*
   */
  saveLink: function(window) {
    let deferred = Promise.defer();
    
    let href = UnMHTDOMUtil.getLinkURL(window);
    if (!href) {
      window.alert(UnMHTStringBundle.get("save.link.error.not_link"));
      deferred.reject(this.RESULT_NOT_A_LINK);
      return deferred;
    }
    
    let tabbrowser = arWindowUtil.getBrowser(window);
    if (!tabbrowser) {
      /* ==== FIXME: 警告を表示するべき? ==== */
      deferred.reject(this.RESULT_NO_BROWSER);
      return deferred;
    }
    
    let iframe = window.document.createElement("iframe");
    iframe.collapsed = true;
    iframe.setAttribute("src", href);
    iframe.setAttribute("type", "content");
    let onload = function() {
      iframe.removeEventListener("load", onload, true);
      
      let param = {
        window:         window,
        targetDocument: iframe.contentDocument,
      };
      Task.spawn(function() {
        let result = yield this.saveWithParam(param);
        iframe.parentNode.removeChild(iframe);
        deferred.resolve(result);
      }.bind(this));
    }.bind(this);
    iframe.addEventListener("load", onload, true);
    
    tabbrowser.parentNode.insertBefore(iframe, tabbrowser.nextSibling);
    
    return deferred;
  },
  
  /**
   * 複数のタブを MHT で保存
   *
   * @param   {nsIDOMWindow} window
   *          実行中のウィンドウ
   */
  savemulti: function(window) {
    let pageURL = "chrome://unmht/content/savemulti.xul";
    
    if (arWindowUtil.focus(pageURL)) {
      return;
    }
    
    let features = "chrome,titlebar,centerscreen,resizable";
    let arg = {
      targetWindow: window
    };
    arWindowUtil.openDialog(window, pageURL, "", features, arg);
  },
  
  /**
   * QuickSave
   *
   * @param   {nsIDOMWindow} window
   *          実行中のウィンドウ
   * @param   {boolean} useSelection
   *          選択範囲を使用するか
   * @returns {Promise<[nsIFile,Array.<string>]>}
   *          resolve: 保存したファイルと保存に失敗したファイルリスト
   *          reject: {number}
   *                  UnMHTSave.RESULT_*
   *                  UnMHTFileSaver.RESULT_*
   */
  quicksave: function(window, useSelection) {
    let baseDirPath = UnMHTPref.getPref("save.quick.dir");
    let baseDir = baseDirPath
      ? arFileUtil.getFile(baseDirPath)
      : (arAppInfo.isFennec
         ? arFileUtil.defaultDownloadsDirectory.clone()
         : arFileUtil.desktopDirectory.clone());
    if (baseDir && !baseDir.exists()) {
      try {
        arFileUtil.createDirectory(baseDir);
      } catch (e) {
        baseDir = null;
      }
    }
    if (!baseDir) {
      window.alert(UnMHTStringBundle.get("save.error.illegal_directory"));
      return Promise.reject(this.RESULT_NO_BASE_DIR);
    }

    let getBaseDirForLocalFile = function(param) {
      let href = param.targetDocument.location.href;
      let localFile = { value: null };
      if (UnMHTDocumentCategory.isLocalFile(href, localFile)) {
        return localFile.value.parent;
      } else {
        return baseDir;
      }
    };
    let onDup = function(param) {
      return Task.spawn(function() {
        let targetFile = param.targetFile;

        let dupAction = UnMHTPref.getPref("save.quick.dup.action");
        if (arAppInfo.isFennec && dupAction == "ask") {
          dupAction = "rename";
        }
        
        if (dupAction == "ask") {
          let result = yield UnMHTSave.showOverwriteConfirm(param.window,
                                                            param.targetFile,
                                                            param.filenameBase,
                                                            false, false);
          if (result.action == "abort" || result.action == "no") {
            throw new Task.Result(null);
          }
          
          if (result.action == "rename") {
            targetFile = result.file;
          }
        } else if (dupAction == "rename") {
          targetFile
            = UnMHTSave.getUniqueFile(param.baseDir, param.filenameBase);
        }
        
        throw new Task.Result(targetFile);
      });
    };
    
    let param = {
      window:           window,
      targetDocument:   this.PARAM_CURRENT,
      targetSelection:  ((useSelection ||
                          UnMHTPref.getPref("save.quick.use_selection"))
                         ? this.PARAM_AUTO
                         : null),
      targetFile:       this.PARAM_DIR,
      baseDir:          (UnMHTPref.getPref("save.local.same_dir")
                         ? getBaseDirForLocalFile
                         : baseDir),
      additionalURIs:   null,
      showOptionDialog: false,
      saveState:        UnMHTSave.stringToSaveState(UnMHTPref.getPref("save.quick.state")),
      onDup:            onDup
    };
    
    return this.saveWithParam(param);
  },
  
  /**
   * 現在のドキュメントを MHT としてメールで送信
   *
   * @param   {nsIDOMWindow} window
   *          実行中のウィンドウ
   * @returns {Promise<[nsIFile,Array.<string>]>}
   *          resolve: 保存したファイルと保存に失敗したファイルリスト
   *          reject: {number}
   *                  UnMHTSave.RESULT_*
   *                  UnMHTFileSaver.RESULT_*
   */
  sendmail: function(window) {
    let param = {
      window:         window,
      targetDocument: this.PARAM_CURRENT,
      targetFile:     this.PARAM_MAIL,
    };
    return this.saveWithParam(param);
  },
  
  /**
   * 対象のドキュメントを MHT としてメールで送信
   *
   * @param   {nsIDOMWindow} window
   *          実行中のウィンドウ
   * @param   {nsIDOMDocument} targetDocument
   *          対象のドキュメント
   * @returns {Promise<[nsIFile,Array.<string>]>}
   *          resolve: 保存したファイルと保存に失敗したファイルリスト
   *          reject: {number}
   *                  UnMHTSave.RESULT_*
   *                  UnMHTFileSaver.RESULT_*
   */
  sendmailDocument: function(window, targetDocument) {
    let param = {
      window:         window,
      targetDocument: targetDocument,
      targetFile:     this.PARAM_MAIL,
    };
    return this.saveWithParam(param);
  },
  
  /**
   * ScrapBook を MHT で保存する
   *
   * @param   {nsIDOMWindow} window
   *          実行中のウィンドウ
   * @param   {object} scrapBookData
   *          ScrapBook のデータ
   * @returns {Promise<[nsIFile,Array.<string>]>}
   *          resolve: 保存したファイルと保存に失敗したファイルリスト
   *          reject: {number}
   *                  UnMHTSave.RESULT_*
   *                  UnMHTFileSaver.RESULT_*
   */
  saveScrapBook: function(window, scrapBookData) {
    let param = {
      window:           window,
      scrapBookData:    scrapBookData,
      showOptionDialog: false,
    };
    return this.saveWithParam(param);
  },

  /**
   * パラメータに従ってドキュメントを保存する
   *
   * @param   {object} param
   *          保存パラメータ
   *            {
   *              window: {nsIDOMWindow}
   *                      実行中のウィンドウ (必須)
   *              targetDocument: {nsIDOMDocument}
   *                              対象のドキュメント
   *                              (scrapBookData とどちらか必須)
   *                                * UnMHTFileSaver.PARAM_CURRENT
   *                                    現在のドキュメント
   *                                * nsIDOMDocument
   *                                    指定したドキュメント
   *              scrapBookData: {object}
   *                             ScrapBook のデータ
   *                             (targetDocument とどちらか必須)
   *              date: {Date}
   *                    Dateフィールドの値 (オプショナル)
   *                      * this.PARAM_CURRENT
   *                          現在時刻
   *                      * this.PARAM_MAIL (デフォルト)
   *                          メールならばメールの Date フィールド
   *                          そうでないなら現在時刻
   *                      * Date
   *                          指定した時刻
   *              targetSelection: {nsISelection}
   *                               対象の選択範囲 (オプショナル)
   *                                 * UnMHTFileSaver.PARAM_CURRENT
   *                                     現在の選択範囲
   *                                     なければエラー
   *                                 * UnMHTFileSaver.PARAM_AUTO
   *                                     現在の選択範囲
   *                                     なければなし
   *                                 * nsISelection
   *                                     指定した選択範囲
   *                                 * null (デフォルト)
   *                                     なし
   *              showOptionDialog: {boolean}
   *                                オプションダイアログを表示するか
   *                                (オプショナル)
   *                                指定しない場合設定による
   *              targetFile: {nsIFile}
   *                          保存先 (オプショナル)
   *                            * PARAM_CHOOSE (デフォルト)
   *                                ファイル選択ダイアログを表示
   *                            * PARAM_DIR
   *                                baseDir に保存
   *                            * PARAM_OVERWRITE
   *                                上書き
   *                            * PARAM_MAIL
   *                                メールで送信
   *                            * nsIFile
   *                                指定したファイル
   *              baseDir: {nsIFile}
   *                       保存先のディレクトリ (オプショナル)
   *                         * nsIFile
   *                             指定したディレクトリ
   *                         * Function
   *                             targetDocument が決まってから関数で返す
   *                               function(param)
   *                                 *param   {object} param
   *                                 *returns {nsIFile}
   *                         * null (デフォルト)
   *                             なし
   *              filenameBase: {string}
   *                            ファイル名の基本部分 (オプショナル)
   *                              * string
   *                                  指定した文字列
   *                              * null (デフォルト)
   *                                  自動生成
   *              filenameType: {string}
   *                            ファイル名の種類 (オプショナル)
   *                              * string
   *                                  title/leafname/custom
   *                              * null (デフォルト)
   *                                  設定による
   *              additionalURIs: {Array.<string>}
   *                              追加のファイル (オプショナル)
   *                                * PARAM_CHOOSE (デフォルト)
   *                                    ダイアログを表示
   *                                * Array.<string>
   *                                    指定したファイル
   *                                * null
   *                                    なし
   *              saveState: {number}
   *                         保存状態 (オプショナル)
   *                         指定しない場合は選択する
   *              onDup: {Function}
   *                     同名のファイルがあった場合 (オプショナル)
   *                       function(param)
   *                         *param   {object} param
   *                         *returns {Promise<nsIFile>}
   *              downloadListener: {nsIDownloadProgressListener}
   *                                 ダウンロードのイベントリスナ (オプショナル)
   *            }
   * @param   {Function} f
   *          終了時コールバック
   *            function(param, result)
   *              *param   {object} param
   *              *param   {number} result
   * @returns {Promise<[nsIFile,Array.<string>]>}
   *          resolve: 保存したファイルと保存に失敗したファイルリスト
   *          reject: {number}
   *                  UnMHTSave.RESULT_*
   *                  UnMHTFileSaver.RESULT_*
   */
  saveWithParam: function(param) {
    return Task.spawn(function() {
      if (!("window" in param)) {
        throw UnMHTFileSaver.RESULT_NO_WINDOW;
      }
      
      /* デフォルト値の設定 */
      if (!("targetDocument" in param)) {
        param.targetDocument = null;
      }
      if (!("scrapBookData" in param)) {
        param.scrapBookData = null;
      }
      if (!("date" in param)) {
        param.date = this.PARAM_MAIL;
      }
      if (!("targetSelection" in param)) {
        param.targetSelection = null;
      }
      if (!("showOptionDialog" in param)) {
        param.showOptionDialog
          = (UnMHTPref.getPref("save.show_option_dialog") ||
             param.targetFile == this.PARAM_OVERWRITE);
      }
      if (!("additionalURIs" in param)) {
        param.additionalURIs
          = (param.targetFile == this.PARAM_MAIL)
          ? null
          : this.PARAM_CHOOSE;
      }
      if (!param.additionalURIs) {
        param.additionalURIs = [];
      }
      if (!("filenameType" in param)) {
        param.filenameType = null;
      }
      if (!("filenameBase" in param)) {
        param.filenameBase = null;
      }
      if (!("targetFile" in param)) {
        param.targetFile = null;
      }
      if (!("baseDir" in param)) {
        param.baseDir = null;
      }
      if (!("downloadListener" in param)) {
        param.downloadListener = null;
      }
      if (!("saveState" in param)) {
        param.saveState
          = (param.targetFile == this.PARAM_MAIL)
          ? UnMHTSave.stringToSaveState(UnMHTPref.getPref("save.mail.state"))
          : null;
      }
      
      /* 保存対象 */
      if (param.targetDocument == null &&
          param.scrapBookData == null) {
        param.targetDocument = this.PARAM_CURRENT;
      }
      if (param.targetDocument == this.PARAM_CURRENT) {
        param.targetDocument
          = arWindowUtil.getCurrentDocument(param.window);
        if (!UnMHTDocumentCategory.isSavable(param.window,
                                             param.targetDocument, false)) {
          throw this.RESULT_NOT_SAVABLE;
        }
      }
      if (!(param.targetDocument || param.scrapBookData)) {
        throw UnMHTFileSaver.RESULT_NO_DOCUMENT;
      }
      
      /* Date フィールド */
      if (param.date == this.PARAM_MAIL) {
        if (this._needMailDate(param.window)) {
          param.date = yield this._getMailDate(param);
        } else {
          param.date = new Date();
        }
      }
      
      /* 選択範囲 */
      if (param.targetSelection == this.PARAM_CURRENT ||
          param.targetSelection == this.PARAM_AUTO) {
        let targetSelection
          = param.targetDocument.defaultView.getSelection();
        
        if (targetSelection &&
            !targetSelection.isCollapsed &&
            targetSelection.rangeCount > 0) {
          param.targetSelection = targetSelection;
          
          param.targetDocument
            = targetSelection.getRangeAt(0).startContainer.ownerDocument;
        } else {
          param.targetSelection = null;
          if (param.targetSelection == this.PARAM_CURRENT) {
            throw UnMHTFileSaver.RESULT_NO_SELECTION;
          }
        }
      }
      
      /* ファイル名の基本部分 */
      if (param.filenameBase == null) {
        param.filenameBase
          = UnMHTSave.getFilenameBase(param.targetDocument,
                                      param.scrapBookData,
                                      param.filenameType,
                                      param.date);
      }
      
      /* オプションダイアログ */
      if (param.showOptionDialog) {
        param.showOptionDialog = false;
        let saveParam = yield this._getSaveOption(param);
        
        param.filenameBase = saveParam.filenameBase;
        param.saveState = saveParam.saveState;
        param.additionalURIs = saveParam.additionalURIs;
      }

      /* 保存先 */
      if (param.targetFile == null ||
          param.targetFile == this.PARAM_CHOOSE) {
        yield this._showFilePicker(param);
      }
      let isMail = false;
      if (param.targetFile == this.PARAM_DIR) {
        if (param.baseDir == null) {
          throw UnMHTFileSaver.RESULT_NO_BASE_DIR;
        }
        
        if (param.baseDir instanceof Function) {
          param.baseDir = param.baseDir(param);
        }
        
        if (!(param.baseDir instanceof Ci.nsIFile)) {
          throw UnMHTFileSaver.RESULT_NOT_A_FILE;
        }
        
        param.targetFile
          = arFileUtil.checkAndAppend(param.baseDir, param.filenameBase, "",
                                      UnMHTFilename.defaultExtension);
        
        if (param.targetFile.exists()) {
          if ("onDup" in param) {
            param.targetFile = yield param.onDup(param);
            if (!param.targetFile) {
              throw this.RESULT_CANCEL;
            }
          }
        }
      } else if (param.targetFile == this.PARAM_OVERWRITE) {
        if (!param.targetDocument) {
          throw UnMHTFileSaver.RESULT_NO_DOCUMENT;
        }
        
        param.targetFile
          = arFileUtil.getFileFromURLSpec(UnMHTScheme.deUnMHTURI(param.targetDocument.location.href));
        if (!param.targetFile) {
          throw UnMHTFileSaver.RESULT_NOT_A_LOCAL_FILE;
        }
      } else if (param.targetFile == this.PARAM_MAIL) {
        isMail = true;
        let tempDir = arFileUtil.tempDirectory.clone();
        tempDir.append(Date.now());
        arFileUtil.createUniqueDirectory(tempDir);
        
        param.targetFile
          = arFileUtil.checkAndAppend(tempDir, param.filenameBase,
                                      "", UnMHTFilename.defaultExtension);
      }
      
      if (!(param.targetFile instanceof Ci.nsIFile)) {
        throw UnMHTFileSaver.RESULT_NOT_A_FILE;
      }
      
      /* 追加ファイル */
      if (param.additionalURIs == this.PARAM_CHOOSE) {
        param.additionalURIs = yield this._getAdditionalURIs(param);
      }
      
      /* 保存開始 */
      let context = this._createContext(param);
      if (!context) {
        throw UnMHTSave.RESULT_ERROR_ADD_DOCUMENT;
      }

      /* リスナ作成 */
      let listener;
      if (arAppInfo.isFennec) {
        listener = this._getBackgroundListener(param, context);
      } else if (UnMHTPref.getPref("save.progress.statusbar")) {
        listener = this._getStatusbarListener(param, context);
      } else {
        listener = yield this._getSaveDialogListener(param, context);
      }
      
      context.setLisener(listener);
      
      let saveResult = yield context.start();
      
      if (isMail) {
        /* メール送信 */
        if (!UnMHTMail.send(param.window, param.targetFile,
                            param.targetDocument.title)) {
          param.window.alert(UnMHTStringBundle.get("save.mail.fail") + "\n"
                             + UnMHTStringBundle.get("save.mail.fail2"));
        }
      }
      
      throw new Task.Result(saveResult);
    }.bind(this)).then(null, function(result) {
      if (result == this.RESULT_CANCEL || 
          result == UnMHTSave.RESULT_CANCEL) {
        return;
      }
      if ((typeof result) == 'number') {
        Cu.reportError("onFail: " + result.toString(16));
      } else {
        Cu.reportError(result + "\n" + result.stack);
      }
    });
  },
  
  /**
   * オプションダイアログを表示する
   *
   * @param   {object} param
   *          保存パラメータ
   * @returns {Promise<object>}
   *          resolve: 保存オプション
   *          reject: {number}
   *                  失敗時の結果
   */
  _getSaveOption: function(param) {
    let saveStatePref
      = (param.targetFile == this.PARAM_MAIL)
      ? "save.mail.state"
      : "save.single.state";
    
    let saveState
      = UnMHTSave.stringToSaveState(UnMHTPref.getPref(saveStatePref));
    let saveStateMask = UnMHTSave.SAVESTATE_MASK_ALL;
    
    if ("targetFile" in param &&
        param.targetFile == this.PARAM_OVERWRITE) {
      saveStateMask = UnMHTSave.SAVESTATE_MASK_CURRENT;
      if (!(saveState & saveStateMask)) {
        saveState = UnMHTSave.SAVESTATE_CURRENT;
      }
    } else if (param.scrapBookData) {
      saveStateMask = UnMHTSave.SAVESTATE_MASK_ORIGINAL;
      if (!(saveState & saveStateMask)) {
        saveState = UnMHTSave.SAVESTATE_ORIGINAL;
      }
    } else if (param.targetSelection) {
      saveStateMask = UnMHTSave.SAVESTATE_MASK_CURRENT;
      if (!(saveState & saveStateMask)) {
        saveState = UnMHTSave.SAVESTATE_CURRENT;
      }
    }
    
    let pageURL = "chrome://unmht/content/saveoptions.xul";
    let features = "chrome,titlebar,centerscreen,resizable";
    let arg = {
      filenameBase:   param.filenameBase,
      showFilename:   param.targetFile == this.PARAM_MAIL,
      saveState:      saveState,
      saveStateMask:  saveStateMask,
      saveStatePref:  saveStatePref,
      showInsertLink: param.targetFile == this.PARAM_OVERWRITE,
    };
    return arWindowUtil.openDialog(param.window, pageURL, "", features, arg);
  },
  
  /**
   * ファイル選択ダイアログを表示する
   *
   * @param   {object} param
   *          保存パラメータ
   * @returns {Promise<number>}
   *          resolve: 成功時の結果
   *          reject: {number}
   *                  失敗時の結果
   */
  _showFilePicker: function(param) {
    return Task.spawn(function() {
      let displayDirectory = null;
      let localFile = { value: null };
      if (param.targetDocument &&
          UnMHTPref.getPref("save.local.same_dir") &&
          UnMHTDocumentCategory.isLocalFile(param.targetDocument.location.href,
                                            localFile)) {
        displayDirectory = localFile.value.parent;
      }
      
      let types = [];
      if (param.saveState != null &&
          param.additionalURIs != this.PARAM_CHOOSE) {
        types = [[param.saveState, param.additionalURIs.length > 0]];
      } else {
        let saveStateMask = UnMHTSave.SAVESTATE_MASK_ALL;
        
        if (param.scrapBookData) {
          saveStateMask = UnMHTSave.SAVESTATE_MASK_ORIGINAL;
        }
        if (param.targetSelection) {
          saveStateMask = UnMHTSave.SAVESTATE_MASK_CURRENT;
        }
        
        let stateList = UnMHTSave.getSaveStateList(saveStateMask);
        
        types
          = [...stateList.map(function(state) [state, false]),
             ...stateList.map(function(state) [state, true])];
      }
      
      /* ==== FIXME: 汎用性が無い ==== */
      let lastStateStr = UnMHTPref.getPref("save.single.state");
      let lastAdd = UnMHTPref.getPref("save.single.add");
      
      let filterIndex = -1;
      let filter = [];
      types.forEach(function([saveState, useAdd], i) {
        filter.push([UnMHTSave.makeFiletype(saveState, useAdd), "*.mht"]);
        
        if (UnMHTSave.saveStateToString(saveState) == lastStateStr &&
            useAdd == lastAdd) {
          filterIndex = i;
        }
      });
      
      let params = {
        title:            UnMHTStringBundle.get("save.title"),
        useLastDir:       true,
        displayDirectory: displayDirectory,
        defaultString:    (param.filenameBase
                           + "." + UnMHTFilename.defaultExtension),
        defaultExtension: UnMHTFilename.defaultExtension,
        filter:           filter,
        filterIndex:      filterIndex,
        uri:              (param.targetDocument
                           ? param.targetDocument.location.href
                           : null)
      };
      
      let [targetFile, filePicker] = yield UnMHTFilePicker.getSaveFile(param.window, params);
      if (!targetFile) {
        throw this.RESULT_CANCEL;
      }
      
      if (!(param.saveState != null &&
            param.additionalURIs != this.PARAM_CHOOSE)) {
        let [saveState, useAdd] = types[filePicker.filterIndex];
        
        if (UnMHTPref.getPref("save.single.state.remember")) {
          UnMHTPref.setPref("save.single.state", 
                            UnMHTSave.saveStateToString(saveState));
          UnMHTPref.setPref("save.single.add", useAdd);
        }
        
        param.saveState = saveState;
        
        if (!useAdd) {
          param.additionalURIs = [];
        }
      }
      
      targetFile = yield UnMHTSave.getValidFile(param.window, targetFile);
      if (!targetFile) {
        throw this.RESULT_CANCEL;
      }
      
      param.targetFile = targetFile;
      
      throw new Task.Result(this.RESULT_OK);
    }.bind(this));
  },

  /**
   * 追加ファイルのダイアログを表示する
   *
   * @param   {object} param
   *          保存パラメータ
   * @param   {Promise<Array.<string>>}
   *          resolve: 追加ファイル
   *          reject: {number}
   *                  失敗時の結果
   */
  _getAdditionalURIs: function(param) {
    let pageURL = "chrome://unmht/content/additional.xul";
    let features = "chrome,titlebar,centerscreen,resizable";
    return arWindowUtil.openDialog(param.window, pageURL, "", features, null);
  },
  
  /**
   * メールの日時が必要かを返す
   *
   * @param   {nsIDOMWindow} window
   *          実行中のウィンドウ
   * @returns {boolean}
   *          メールの日時が必要か
   */
  _needMailDate: function(window) {
    return (UnMHTPref.getPref("save.filename.custom.mail_date") &&
            window &&
            arWindowUtil.isMailWindow(window) &&
            "gFolderDisplay" in window &&
            "selectedMessageUris" in window.gFolderDisplay &&
            window.gFolderDisplay.selectedMessageUris.length == 1);
  },
  
  /**
   * メールの日時を取得する
   *
   * @param   {object} param
   *          保存パラメータ
   * @returns {Promise<Date>}
   *          resolve: メールの日時
   *          reject: {number}
   *                  失敗時の結果
   */
  _getMailDate: function(param) {
    return Task.spawn(function() {
      let window = param.window;
      
      let mailSession
        = Cc["@mozilla.org/messenger/services/session;1"]
        .getService(Ci.nsIMsgMailSession);
      
      let messageURL
        = mailSession.ConvertMsgURIToMsgURL(window.gFolderDisplay.selectedMessageUris[0], window.msgWindow);
      
      let uri;
      try {
        uri = Services.io.newURI(messageURL, null, null);
      } catch (e) {
      }
      
      if (!uri) {
        throw this.RESULT_NOT_A_MAIL;
      }
      
      let result = yield arNetFileLoader.loadData(uri, null, true);
      
      let date = new Date();
      if (result.status != arNetFileLoader.STATUS_NG) {
        let mailPart
          = arMIMEDecoder.decodeMessage(result.data.replace(/\r\n|\r|\n/g, "\r\n"), true);
        if (mailPart &&
            mailPart.fields.has("date")) {
          let tmpDate = arMIMEDecoder.decodeDate(mailPart.fields.get("date"));
          if (tmpDate) {
            date = tmpDate;
          }
        }
      }
      
      throw new Task.Result(date);
    }.bind(this));
  },
  
  /**
   * ステータスバーのイベントリスナを返す
   *
   * @param   {object} param
   *          保存パラメータ
   * @param   {UnMHTSaveContext} context
   *          対象の保存コンテキスト
   * @returns {IUnMHTSaveListener}
   *          イベントリスナ
   */
  _getStatusbarListener: function(param, context) {
    let prog = UnMHTStatusbarProgressMap.get(param.window);
    prog.addContext(context);
    return prog;
  },

  /**
   * 表示しないイベントリスナを返す
   *
   * @param   {object} param
   *          保存パラメータ
   * @param   {UnMHTSaveContext} context
   *          対象の保存コンテキスト
   * @returns {IUnMHTSaveListener}
   *          イベントリスナ
   */
  _getBackgroundListener: function(param, context) {
    let listener = {
      /**
       * キャンセルが完了したイベント
       *
       * @param   {UnMHTSaveContext} context
       *          対象の保存コンテキスト
       */
      onCancel: function(context) {
      },
      
      /**
       * 保存失敗イベント
       *
       * @param   {UnMHTSaveContext} context
       *          対象の保存コンテキスト
       * @param   {number} reason
       *          エラーの理由
       * @param   {Exception} e
       *          例外
       */
      onError: function(context, reason, e) {
        let message = 
          UnMHTStringBundle.get("error.title") + "\n"
          + "\n"
          +  UnMHTSave.errorReasonToMessage(reason) 
          + (e ? ("\n" + e.toString()) : "");
        param.window.alert(message);
      },
      
      /**
       * 進捗状況変化イベント
       *
       * @param   {UnMHTSaveContext} context
       *          対象の保存コンテキスト
       * @param   {string} currentFile
       *          現在取得のファイル名
       * @param   {string} currentSize
       *          取得したバイト数 / ファイルサイズ
       * @param   {number} ok
       *          取得完了した数
       * @param   {number} na
       *          取得中の数
       * @param   {number} ng
       *          取得失敗した数
       */
      onProgress: function(context, currentFile, currentSize, ok, na, ng) {
      },
      
      /**
       * 保存完了
       *
       * @param   {UnMHTSaveContext} context
       *          対象の保存コンテキスト
       * @param   {Array.<string>} failedList
       *          失敗したファイル名
       */
      onSave: function(context, failedList) {
        if (UnMHTPref.getPref("save.show_errors") && failedList.length > 0) {
          let message
            = UnMHTStringBundle.get("save.failed") + "\n"
            + failedList.join("\n");
          let buttons = [{
            label: UnMHTStringBundle.get("ok"),
            callback: function () {
            }
          }];

          arAlertUtil.doorhanger(param.window, message, "unmht", buttons);
        }
        
        arAlertUtil.toast(param.window,
                          UnMHTStringBundle.get("save.done"),
                          "short");
      },
      
      /**
       * 保存コンテキストの破棄
       *
       * @param   {UnMHTSaveContext} context
       *          対象の保存コンテキスト
       */
      onDestruct: function(context) {
      },
    };
    return listener;
  },

  /**
   * ダイアログで保存を開始する
   *
   * @param   {object} param
   *          保存パラメータ
   * @param   {UnMHTSaveContext} context
   *          対象の保存コンテキスト
   * @returns {Promise<IUnMHTSaveListener>}
   *          イベントリスナ
   */
  _getSaveDialogListener: function(param, context) {
    let pageURL = "chrome://unmht/content/save.xul";
    let features = "chrome,titlebar,centerscreen,resizable";
    let arg = {
      saveState:         param.saveState,
      saveContext:       context
    };
    return arWindowUtil.openDialog(param.window, pageURL, "", features, arg);
  },

  /**
   * 保存コンテキストを作成する
   *
   * @param   {object} param
   *          保存パラメータ
   * @returns {UnMHTSaveContext}
   *          保存コンテキスト
   *          失敗したら null
   */
  _createContext: function(param) {
    let [location, subject]
      = UnMHTSave.getLocationAndTitle(param.targetDocument, param.scrapBookData);
    
    let saveContext = UnMHTSave.createContext(param.window, param.targetFile,
                                              location, subject,
                                              param.downloadListener,
                                              param.saveState);
    
    let arg = {
      targetDocument:  param.targetDocument,
      targetSelection: param.targetSelection,
      scrapBookData:   param.scrapBookData,
      saveState:       param.saveState,
      location:        location,
      isHTML:          true,
      isStartPart:     true,
      window:          param.window
    };
    if (!saveContext.addDocument(arg)) {
      saveContext.destruct();
      return null;
    }
    
    if (param.additionalURIs) {
      param.additionalURIs.forEach(function(file) {
        saveContext.addFile(null, file, false);
      });
    }
    
    return saveContext;
  }
};
