"use strict";

let { classes: Cc, interfaces: Ci, results: Cr, utils: Cu } = Components;
Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative(this, "UnMHTScheme.jsm");
XPCOMUtils.importRelative(this, "UnMHTStringBundle.jsm");
XPCOMUtils.defineLazyGetter(this, "arUconv", function() {
  XPCOMUtils.importRelative(this, "base/arUconv.jsm");
  return arUconv;
});
XPCOMUtils.defineLazyGetter(this, "UnMHTCachePage", function() {
  XPCOMUtils.importRelative(this, "UnMHTCachePage.jsm");
  return UnMHTCachePage;
});
XPCOMUtils.defineLazyGetter(this, "UnMHTExtractor", function() {
  XPCOMUtils.importRelative(this, "UnMHTExtractor.jsm");
  return UnMHTExtractor;
});
XPCOMUtils.defineLazyGetter(this, "UnMHTHTMLUtil", function() {
  XPCOMUtils.importRelative(this, "UnMHTHTMLUtil.jsm");
  return UnMHTHTMLUtil;
});

let EXPORTED_SYMBOLS = ["UnMHTProtocolHandler"];

/**
 * unmht プロトコルからチャネルを生成する
 *
 * @implements nsIProtocolHandler
 */
function UnMHTProtocolHandler() {
}
UnMHTProtocolHandler.prototype = {
  classID:          Components.ID("{d605b959-0688-4846-887c-f6658725f7f1}"),
  contractID:       "@mozilla.org/network/protocol;1?name=unmht",
  classDescription: "UnMHT Protocol Handler JS Component",
  QueryInterface:   XPCOMUtils.generateQI([Ci.nsIProtocolHandler]),
  
  /* ==== nsIProtocolHandler インターフェース ==== */
  
  scheme: "unmht", /** プロトコルスキーム
                    *  @type {string} */
  defaultPort: -1, /** ポート番号
                    *  ネットワークベースではないので -1
                    :  @type {number} */
  
  /** プロトコルの詳細
   *  file と同様のプロトコル
   *  @type {number} */
  protocolFlags: (Ci.nsIProtocolHandler.URI_NOAUTH
                  | Ci.nsIProtocolHandler.URI_LOADABLE_BY_ANYONE),
  
  /**
   * ブラックリストのポートを上書きするか
   *
   * @param   {number} port
   *          ポート番号
   * @param   {string} scheme
   *          プロトコルスキーム
   * @returns {boolean}
   *          上書きするか
   */
  allowPort: function(port, scheme) {
    return false;
  },
  
  /**
   * URI を作成する
   *
   * @param   {string} spec
   *          対象の URI
   * @param   {string} charset
   *          対象の文字コード
   * @param   {nsIURI} baseURI
   *          読み込み元の URI
   * @returns {nsIURI}
   *          作成した URI
   */
  newURI: function(spec, charset, baseURI) {
    let uri
      = Cc["@mozilla.org/network/standard-url;1"].createInstance(Ci.nsIURI);
    
    if (!baseURI) {
      uri.spec = spec;
      return uri;
    }
    
    if (UnMHTScheme.isUnMHTURI(baseURI.spec)) {
      let resolved = UnMHTExtractor.resolveURI(baseURI.spec, spec);
      if (resolved) {
        uri.spec = resolved;
        return uri;
      }
    }
    
    uri.spec = baseURI.resolve(spec);
    return uri;
  },
  
  /**
   * チャネルを作成する
   *
   * @param   {nsIURI} uri
   *          対象の URI
   * @returns {nsIChannel}
   *          作成したチャネル
   */
  newChannel: function(uri) {
    let param = UnMHTScheme.getUnMHTURIParam(uri.spec);
    
    try {
      if (!("original" in param &&
            param.original)) {
        /* 不正な URI */
        return this._createIllegalChannel(uri);
      } else if (param.scheme == "cache") {
        return this._createCacheChannel(uri);
      } else if (param.scheme == "cid" ||
                 param.scheme == "dummy" ||
                 param.scheme == "empty") {
        return this._createEmptyChannel(uri);
      } else if (param.scheme == "loading") {
        return this._createLoadingChannel(uri);
      } else if (param.scheme == "error") {
        return this._createNotFoundChannel(uri);
      } else {
        /* mht 内のファイル */
        return this._createFileChannel(uri, param);
      }
    } catch (e) {
      Cu.reportError(e + "\n" + e.stack);
    }
    
    return this._createEmptyChannel(uri);
  },
  
  /* ==== private ==== */
  
  /**
   * mht 内のファイルのチャネルを作成する
   *
   * @param   {nsIURI} uri
   *          対象の URI (umht:// 形式)
   * @param   {UnMHTURIParam} param
   *          対象の URI の情報
   * @returns {nsIChannel}
   *          作成したチャネル
   */
  _createFileChannel: function(uri, param) {
    let [eFileInfo, part] = UnMHTExtractor.getFileInfoAndPart(uri.spec);
    if (!eFileInfo) {
      try {
        /* URI が不正でないかチェック */
        Services.io.newURI(param.original, null, null);
      } catch (e) {
        return this._createIllegalChannel(uri);
      }
      
      /* 未展開 */
      return this._createLoadingChannel(uri); 
    }
    
    if (!part) {
      /* 存在しないパート */
      return this._createEmptyChannel(uri);
    }
    
    let pipe = Cc["@mozilla.org/pipe;1"].createInstance(Ci.nsIPipe);
    pipe.init(false, false, part.eParam.content.length, 1, null);
    pipe.outputStream.write(part.eParam.content, part.eParam.content.length);
    pipe.outputStream.close();
    
    return this._createInputStreamChannel(uri,
                                          part.eParam.charset,
                                          part.eParam.mimetype,
                                          pipe.inputStream,
                                          part.eParam.content.length);
  },
  
  /**
   * キャッシュ情報のチャネルを作成する
   *
   * @param   {nsIURI} uri
   *          対象の URI
   * @returns {nsIChannel}
   *          作成したチャネル
   */
  _createCacheChannel: function(uri) {
    let data = UnMHTCachePage.createCachePage();
    
    return this._createStringInputStreamChannel(uri, "utf-8", "text/html", 
                                                data, data.length);
  },
  
  /**
   * ファイルが無い情報のチャネルを作成する
   *
   * @param   {nsIURI} uri
   *          対象の URI
   * @returns {nsIChannel}
   *          作成したチャネル
   */
  _createNotFoundChannel: function(uri) {
    let bundle = function(name) {
      return UnMHTHTMLUtil.escapeEntity(UnMHTStringBundle.get(name));
    };
    
    let message
      = UnMHTHTMLUtil.escapeEntity(UnMHTScheme.deErrorURI(uri.spec));
    return this._createErrorChannel(uri, bundle("error.file_not_found"),
                                    message, true);
  },
  
  /**
   * 不正な URI 情報のチャネルを作成する
   *
   * @param   {nsIURI} uri
   *          対象の URI
   * @returns {nsIChannel}
   *          作成したチャネル
   */
  _createIllegalChannel: function(uri) {
    let bundle = function(name) {
      return UnMHTHTMLUtil.escapeEntity(UnMHTStringBundle.get(name));
    };
    
    let message
      = UnMHTHTMLUtil.escapeEntity(uri.spec);
    return this._createErrorChannel(uri, bundle("error.illegal_address"),
                                    message, false);
  },
  
  /**
   * エラーメッセージを表示するチャネルを作成する
   *
   * @param   {nsIURI} uri
   *          対象の URI
   * @param   {string} title
   *          タイトル (HTML)
   * @param   {string} message
   *          表示するメッセージ (HTML)
   * @param   {boolean} showRetryButton
   *          リトライボタンを表示する
   * @returns {nsIChannel}
   *          作成したチャネル
   */
  _createErrorChannel: function(uri, title, message, showRetryButton) {
    let bundle = function(name) {
      return UnMHTHTMLUtil.escapeEntity(UnMHTStringBundle.get(name));
    };
    
    let data
      = "<!DOCTYPE html>" + "\n"
      + "<html xmlns=\"http://www.w3.org/1999/xhtml\">" + "\n"
      + "<head>" + "\n"
      + "  <meta charset=\"UTF-8\">" + "\n"
      + "  <title>" + bundle("error.title") + "</title>" + "\n"
      + "  <style type=\"text/css\">" + "\n"
      + "html {" + "\n"
      + "  background: -moz-Dialog;" + "\n"
      + "}" + "\n"
      + "\n"
      + "body {" + "\n"
      + "  margin: 0;" + "\n"
      + "  padding: 0 1em;" + "\n"
      + "  color: -moz-FieldText;" + "\n"
      + "  font: message-box;" + "\n"
      + "}" + "\n"
      + "\n"
      + "h1 {" + "\n"
      + "  margin: 0 0 .6em 0;" + "\n"
      + "  border-bottom: 1px solid ThreeDLightShadow;" + "\n"
      + "  font-size: 160%;" + "\n"
      + "}" + "\n"
      + "\n"
      + "h2 {" + "\n"
      + "  margin: 0 0 .6em 0;" + "\n"
      + "  border-bottom: 1px solid ThreeDLightShadow;" + "\n"
      + "  font-size: 140%;" + "\n"
      + "}" + "\n"
      + "\n"
      + "#errorPageContainer {" + "\n"
      + "  position: relative;" + "\n"
      + "  min-width: 13em;" + "\n"
      + "  max-width: 52em;" + "\n"
      + "  margin: 4em auto;" + "\n"
      + "  border: 1px solid ThreeDShadow;" + "\n"
      + "  border-radius: 10px;" + "\n"
      + "  padding: 3em;" + "\n"
      + "  -moz-padding-start: 30px;" + "\n"
      + "  background: -moz-Field;" + "\n"
      + "  background-origin: content-box;" + "\n"
      + "}" + "\n"
      + "\n"
      + "#errorDesc > p {" + "\n"
      + "  overflow: auto;" + "\n"
      + "  font-size: 130%;" + "\n"
      + "  white-space: pre-wrap;" + "\n"
      + "}" + "\n"
      + "\n"
      + "#unmht-retry-button {" + "\n"
      + "  margin-top: 2em;" + "\n"
      + "}" + "\n"
      + "  </style>" + "\n"
      + "</head>" + "\n"
      + "<body>" + "\n"
      + "  <div id=\"errorPageContainer\">" + "\n"
      + "    <h1>" + bundle("error.title") + "</h1>" + "\n"
      + "    <h2>" + title + "</h2>" + "\n"
      + "    <div>" + "\n"
      + "      <div id=\"errorDesc\">" + "\n"
      + "        <p>" + message + "</p>" + "\n"
      + "      </div>" + "\n"
      + "    </div>" + "\n"
      + (showRetryButton
         ? ("    <button id=\"unmht-retry-button\" autocomplete=\"off\" autofocus=\"true\">"
            + bundle("error.retry")
            + "</button>" + "\n")
         : "")
      + "  </div>" + "\n"
      + "</body>" + "\n"
      + "</html>" + "\n";
    
    data = arUconv.fromUnicode(data, "utf-8");
    
    return this._createStringInputStreamChannel(uri, "utf-8", "text/html", 
                                                data, data.length);
  },

  /**
   * 空のチャネルを作成する
   *
   * @param   {nsIURI} uri
   *          対象の URI
   * @returns {nsIChannel}
   *          作成したチャネル
   */
  _createEmptyChannel: function(uri) {
    return this._createStringInputStreamChannel(uri, "utf-8", "text/html", 
                                                "", 0);
  },

  /**
   * ロード中のチャネルを作成する
   *
   * @param   {nsIURI} uri
   *          対象の URI
   * @returns {nsIChannel}
   *          作成したチャネル
   */
  _createLoadingChannel: function(uri) {
    let data
      = "<!DOCTYPE html>" + "\n"
      + "<html lang=\"ja\">" + "\n"
      + "<head>" + "\n"
      + "  <meta charset=\"UTF-8\">" + "\n"
      + "  <title>Loading...</title>" + "\n"
      + "  <style type=\"text/css\">" + "\n"
      + "#unmht-loading-container {" + "\n"
      + "  position: absolute;" + "\n"
      + "  top: 50%;" + "\n"
      + "  left: 50%;" + "\n"
      + "  margin: -18px 0 0 -18px;" + "\n"
      + "}" + "\n"
      + "#unmht-loading {" + "\n"
      + "  width: 36px;" + "\n"
      + "  height: 36px;" + "\n"
      + "  background: url(data:image/png;base64,"
+ "iVBORw0KGgoAAAANSUhEUgAAACQAAAAkCAYAAADhAJiYAAAACGFjVEwAAAAYAAAAANndHFMAAAAa"
+ "ZmNUTAAAAAAAAAAkAAAAJAAAAAAAAAAAAB8D6AEB8CzIfwAABHNJREFUWIXNl19oU1ccx2+6bisO"
+ "JyvqCsXuQR3OsYfuQZ0vok+DroMJvihs6HB7EHxoUUQYsXatIPhgLZ3Dh0ClL7n3d3IHdnuS3Hsb"
+ "4qQdreXmnDSQB7Eh54YsM+F3S0K4nj3sXJeF1qZL1H7hB+GcX37nc3+/81dRmpQQ4ki1Wp2pVqsz"
+ "QogjzcZrBqTd87wxIcQz8a+eybb21wF0Wqyt068cyPO8+2vReJ53v+UDrqys9CDit4h4sVgs9gsh"
+ "Ovw+IUTA87z5FwDNCyECvn80Gm1XVfVjADim6/rBqamp9zYEUywW+xCRImLBdd0/XdfNI2IEEbtq"
+ "oCZfULJJ30/X9a2EkDOapo0AwCgAjBJCLhFCPtpIZqjruhwRn/jmuu5fiPiT/+WVSqVXCPF0FZin"
+ "lUqlV4YLqKp6AgCuAcCVGhsmhFxqKFOyTIVaGGkZRFzK5/Pdvm+1Wj0qhHgghFiR9qBarR71+8Ph"
+ "8DYAuAwAQ3VAVwBgVNf1g40AXVwDaBkRH5dKpX21/kKI9nK5vLdcLu+tX+4AsFPTtKCqqqsCAcCx"
+ "RoC+kHPmSV3Jsog4WygUtq0bRCoUCnUQQgYIIVfrgTRNG9F1ff+6QTKZzBZEjMg5k0HEZQlTQMSz"
+ "jcL4AoBDciL7UEMAcE1V1TNzc3NvNhQEEbsQ8TYiLiHiY0ScdV33u9ql3KiEEAFCyGeEkAFN04IA"
+ "cJkQckLX9a0bjaW4rttdKpX25fP5dzf85zqFQqEOANgZDocbLrmiKP98UbODNy0hRFs2m+3nnI9z"
+ "zic558FcLvfhyxovHA53AcBXAPA9AHytqmrv80QIIQKc8wHHcR5xzuc553/kcrlHjuNEM5nMp62G"
+ "0TTtA7k3jQLAMAD8KH9/rihKQOGcf+I4zqzjOA8dx/ndt1wuN885DzW8EhpQOBx+gxByVh4jz7cA"
+ "VVWHNE27GolEdinZbPZnx3HmamGkPeSczywvL+9qIVAnAPywxkY5TAj5ZvMBbbqSbbpJrSibbNnX"
+ "a1NsjGspmUxuT6fTPZlMZkuzsf730aEoimLbdidjbJBSOpVMJtVEInHHtu0vX8vhmkql3qaUjjDG"
+ "fmWMEWk6Y2yaMda/UaCmrx+MscOMsXuMMagzPZFI3FlYWHinUZiWXNAYYycppdOrAJFkMqmm0+me"
+ "Wv9gMNhmWdYOy7J2BIPBtrrsNH+FpZT2yfL8B4hSGpFzarvvG41G9xiGMWia5g3TNG8YhjEYjUb3"
+ "+P0tueQvLi6+Tym9Syn9pQ7oN8bYoD+x4/F4t2VZ103TnDBNc0zahGVZ1+PxuP8yaf4ZpCiKkkql"
+ "DlFK78pMTTPG7lFKR2zb7vR9YrHYKcMwbpumebPWZNsp36/ph2JtppaWlvoYYycZY4dt236rpjtg"
+ "muYFy7LG64Fk2wXFPwaUFjylG5FpmudkuW7W2YRpmudaPmADQAdkecZqYMZk24FXDiSEaDMM47hl"
+ "WbcsyxqXdsswjONCiLb1I7wkxWKx3YZhnDcM43wsFtvdbLy/AbWh63L0eRXnAAAAGmZjVEwAAAAB"
+ "AAAAJAAAACQAAAAAAAAAAAAfA+gBAWtfIqsAAASnZmRBVAAAAAJYhc2XT2gVRxzH90Xb2oKVVtFi"
+ "CUWxSOtBipS0Bz14sQoeeohHhWDbiwp6skJdjRqrEqG5FCQQkAZld3/jEkJmtgcj1VaRNOzMkyCx"
+ "GHAmetiEzBDoY3eWN73syvYRyXu+1zRf+MEy89vffvb3m7+W1aSSJNkZx/HtOI5vJ0mys9l4ry1j"
+ "zMo4jnu11kmapjpNU621TuI47jXGrFxyoDiOD1arVaO1rmit/86sUq1WTRzHB5ccKE3TEa11tQCT"
+ "WzVN05GWf3B2drZdKXVIKXV8dnZ239TU1Kq8zxhTStP0vtY6XQAoTdP0vjGmlPuPjo6udF13GwDs"
+ "9n2/Y3Bw8L2GYKSUe6SU40opIaUUUkqulLo5Pz+/IffRWvdnJfsXUNbWn/v5vr8aIdTled4FAOgB"
+ "gB6E0EmE0CeNZOZPpdSUlHKyYM+llD/lfx7H8XatdZSmaT6OKtlzFMfx9ixcyXXdTgC4CABnCnYO"
+ "IXSyrkxlZRI1MJNKqb+UUmEURRtz3yRJdmmt72mtZWb3kiTZlfc7jrMGAE4BwNkaoDMA0OP7fkc9"
+ "QMdfAfRESvk4iqKtRX9jzIpKpbKlUqlsMcasKPYBwHrP82zXdRcEAoDd9QDtlVLyBYCeSinvzszM"
+ "vLtokEwDAwOrEEInEELdtUCe513wff/TRYMYY96em5u7KaV8npXpiVLqqVJKzM3NddULkwsAvsgG"
+ "cg51FgAuuq7bNTY29kZdQebn5zcopfqUUqGU8rGU8q5Sqqs4leuVMaaEEPoSIXTC8zwbAE4hhDp9"
+ "31/daCwriqKNURRtNcY0/nKNBgYGVgHAesdx1jT04utkoeWybbuNc76Xc94rhLjGOf+ec/7xf/U9"
+ "x3E+AICvAeA7ADjouu5nLxNhjCk9e/bsqBDiwfT09O+c83tCiAdCiBHO+fZFYjcsz/M+ytamHgA4"
+ "BwDns+evLMsqWS9evNgmhPhNCHFHCDGaG+f8D875z3XPhDrkOM4KhNA32TbycglwXfes53ndt27d"
+ "arc4531CiLtFmMzucM5/nZmZ+bCFQO8DwA+vWCjPIYQOLT+gZVeyZTeoLWuZTftaLYuF8VWamJhY"
+ "Wy6X28fGxt5pNtZrbx2WZVmMsfcYY0cZY/2MseuU0r7x8fF9/8vmOjk5+RZj7DSlFBhjg2EYDlJK"
+ "b1BKPUrp3kaBmj5+hGHYEYahSyn9pWiMsRuU0r7p6em6y9eSAxpjrJMxthDQIGPs+sOHD9uL/rZt"
+ "tw0NDa0bGhpaZ9t2W012mj/ClsvlPVl5FgLqn5iYWJv7Dg8PbyaEHMEYn8cYnyeEHBkeHt6c97fk"
+ "kB+G4XrG2DVK6c0aKGCMHc0Htu/7Gwkh3YSQXkLIpcx6CSHdvu/nN5Pmr0FZlj7PoDxKqccYcxhj"
+ "pxljLwNgjA8QQq5ijH8sWtZ2IPdr+qJYzFS5XN7DGOsMw7Dj0aNHbxa6SxjjYxjjK7VAWdsxK98G"
+ "rBZcpesRxvjwyMhIby1Q1na45R9cTEEQ7MAYXyWEXCqU6xLG+GoQBDuWHMi27bYgCPYTQi5nZbpC"
+ "CLkcBMH+2um/pAqCYFMQBN9mtqnZeP8ApePK+wnj6gEAAAAaZmNUTAAAAAMAAAAkAAAAJAAAAAAA"
+ "AAAAAB8D6AEBhsnxQgAABHVmZEFUAAAABFiFzZdRaBRHGMf3TNpqwWpDCVpbIgQfbAnYItiiT9qm"
+ "eWwefFGaFiHkyRcpKKHlTGIuoH0oVhBp6JFIONidb277YmhNe7643O4sHGoq9O7mQuJDyGVnQMPt"
+ "zT5NHzoryzUhd7lrzB8+GJhvv/3t9818M6tpTcr3/ZOVSmW2UqnM+r5/stl4W1Ymk2kPgmBCCPFc"
+ "CPFC2fMgCCYymUz7tgMFQXBO/CsuhGDKuBBCBEFw7lUApYUQlQhMaJUgCNItf6Hnee9xzs97nndx"
+ "ZWXlCynlG+GclDImhHigylQL9EII8UBKGQv9M5lMu2EYHwLAadM0T8zMzLzdKMxnnPOHjLECY6zA"
+ "Of+bMTa1trbWGfr4vn9blawWSPi+fzv0M01zL8b4AkJoHAASAJDAGF/BGB+tOzOe5z3knD9ljD0K"
+ "jXNe9DzvRvjlQRD0CCGeqbJxZRUhxLMgCHpUuJhhGGcBYAIArkZsDGN8pa5Mcc7Pq8w8qgF6whjL"
+ "lsvlg5EsnRJCzAkhlpXN+b5/KpzXdX0fAAwDwEgN0FUASJimeaKeDF3cAOgxYyxXLpePRP2llG3V"
+ "arW7Wq12SynbonMA0IkQihuGsS4QAJyuB6hXrZlaoL8YY/fz+fxbmwZRSiaTuzHGlzDGo7VACKFx"
+ "0zQ/2DTI0tLSHsbYFGOsyDl/wjl/rGAKnPOv6oUJBQCfqIUcQo0AwIRhGBdc132triDLy8udnuf9"
+ "wBjLMsZyjLH7nPOB6FauV1LKGMb4U4zxJYRQHACGMcZnTdPc22gsrVwuHyyXy0dWV1cbf7hGyWRy"
+ "NwB06rq+r6EHt5KFlisej+/K5/O9lNKJYrH4U6FQ+Dafz3f/X+/Tdf0AAPQDwBAADBiG8dHLREgp"
+ "Y6VSaYhS+ieldK5UKv2uxphS2rNJ7IaFEOpSvSkBAGMAcE2N+zRNi2kLCwtHKaW/UUpnKaX3IvYH"
+ "pfRHKWXLrhK6rrdhjAfVMfKyBRiGMYIQGk2n0+9rxWLxhsrKvRqbpZT+uri4+G4LgToA4PsNGuUY"
+ "xvjrnQe040q24xa1pu2wbV+rHdEYN5JlWR25XO6QZVl7mo215aND0zQtl8vtJ4QM2bZ9ixByhxBy"
+ "PZvN9r6Sw3V+fv5127YvE0LuOo4zadv2pG3bvxBCpgkhnzcK1PT1w7bt447jTDmO83PUFNR113Xf"
+ "rBemJRc013W/tG17eh2gSULIHcuyDkX9pZQxXdc7dF3vqC1pS66wtm2fIYT8B0iV75ZlWR2hbyqV"
+ "OowxHsQYf6dsMJVKHQ7nW3LJd133HcdxbjqOk4wCEULuZrPZoTALqpcMI4TGDcMYUd12HACGdV0/"
+ "oMI1/xukoD52HOemWsjTjuNMEUIu53K5/ZFy9Ku0r1eK/tCv6R/FaKZs2z6j1tTx2h2huuzYeock"
+ "AAxFfZv+la5HADCgzqFaoGsAMNDyF26mdDp9DAAS0R2kxol0On1s24Hi8fguAOhDCI2qMo2pcV88"
+ "Ht+17UChEEJdCKFvlHU1G+8fXP+vVcBfEQkAAAAaZmNUTAAAAAUAAAAkAAAAJAAAAAAAAAAAAB8D"
+ "6AEBawODOAAABOxmZEFUAAAABliFzZZdaBRXFMd3jW2VVmwsWkXqR30obVqaPmmerPoi+CRUpX2w"
+ "VhB9smJfJLSsScxqpVAbBD9e8hRiZ+65OyUEhBV27kwaVFLQauauglDSkNyzCzYwd3bZmuH0ZdZu"
+ "tom7a9KQPxy4MHfO/PZ/7jl3Y7F5KgiCHb7vp3zfTwVBsGO++V5aRNTk+36n7/uTQRCoIAiU7/uT"
+ "vu93ElHTogMVCoWDWuunWutxrfWfUYxrrZ8WCoWDiw6ktb6htc5XwJQjr7W+seAfHBsb25jL5Q4p"
+ "pU4opfY8fvz4tfIzIor7vn8zCAJVDRSV7iYRxSv2rySiA2EYdk5PT58sFovbGoLJ5/O7EPFWLpe7"
+ "j4j3lVL3EPH6xMTE2vIerfWlqGTVDj3VWl+qgNlIRC7NVP7Zs2ef1e0MIt5SSv2GiLcr4sHk5GR3"
+ "+ZeXSqUWrfWjIAjy0dkZj9aPSqVSS9nJMAwNml1/EdG7NYFyudwhRPy9Cua2UuouImaUUm+X9xYK"
+ "hTbf9we11k+01k983x8sFAptFe5sCsNQzwFE09PTp2oCKaVOIOL9aiBEvIOIw4g4o/5E1FQsFrcW"
+ "i8Wt1e1ORB8RUTgXUBiG3fUA7YnOTLVDI0qpgXw+v6pmkkhTU1PNRDT+Aodqj4exsbGViHgdER8g"
+ "4l1EvKOUGokO9+f1wpRFRKfncOdXInq9riQTExNrlVJJRMwg4rBSakAp9QVVtHIDQMuI6Bsi+oOI"
+ "/iaiqTAMfyaijY3miiHiekTchohvNPzyf8HeLJVKHxLRpkZfbNiFBRcRxbPZ7G4p5VnP834YHR09"
+ "mc1mt/5f3zMMYz0A7AeA4wBw2DTNT54bQURxKeVRKeWglHLA87xfpJSDnuf1jY6Otiw0DGNsMwC0"
+ "A0ASALoA4Fy03huLxeIxKeV7nudZUkoupYSKGJBSfk9EyxcKxjCMJs75McZYNwCcLYdpmh2Msc5U"
+ "KvVOLJvNdkWuQFVwKeUNz/M2LCDQGgD4zjTNjkqgKLo4518uPaAlV7Ild6hjsSXW9tVaEoNxLrmu"
+ "2yyE2DA8PLxyvrl6e3tXAMA6wzBWN/xyOp1enclkvnIc56LjOD8KITps2971kpdrnHPexjk/zRhL"
+ "AEA75/yAZVn1/Y15+PDhq7Ztfy2EuCaE6LFtu8e27cu2bV91XffTRoEAYAcAJDnnnVFXdQDAedM0"
+ "j46MjLxSM8HQ0FCrbdtXhBA/VYZt25eFEB2NlK+3t3cF5/x0BczzYIx1W5b1Qc0kQoh9juPMBtQj"
+ "hLgkhJgxKIkobhjGGsMw1lSXFADWMcYScwzCJADsrgnkOM5O27avVgMJIXocx7noum5zeW9/f/8W"
+ "zvkxzvm3URzr7+/fUn5uGMbqaObMCmRZ1vaaQOl0+i0hxAXHcS5XAV0TQhwpuxDNknbGWLdpmh3R"
+ "tO0GgHbDMNZH6eKmaR4AgPOzXBNn+vr6ml+A8q9c1/3Ydd0Ltm1fjeKKEOJUOp1+3rLRYEvOUYr9"
+ "5X2WZa3inB+NYJPRAT/DOX+/LphKpxzH2SmE2Dc0NNRa3RHRlO2a7ZIEgOOVezOZzHLTNFsAYLdl"
+ "WdvrdqYRAcDh6B6qBjoHAIcX/IO1lEqlWgEgWdlB0TqZSqVaFx0okUgsA4C9jLHOqExd0XpvIpFY"
+ "tuhAZTHGNjPGjkSxeb75/gEIg/LDfzlMYwAAABpmY1RMAAAABwAAACQAAAAkAAAAAAAAAAAAHwPo"
+ "AQGGlVDRAAAFJWZkQVQAAAAIWIXNl19oFEccx/eMbSOtfxrEiiCmQh/qH7AvpsX+ER9aXyXYB6G2"
+ "KKIFacEnDSmXP+asBGPBJx9KKhgiu/Ob3YdzZ4+gZ/GhBlRul+Ql/gl3O2cUQ26G1uTcuWT60Fm7"
+ "Hid3Z9I0X/jBwM7+5nPf38xv9jRtnmKM7ZyamroyNTV1hTG2c775XltSygbGWDtj7CHnfJxzPs4Y"
+ "e8gYa5dSNiw6UKFQaGWMUc75PcbYGGNsTI1poVBoXXQgxtglznk2hIlAZRljlxZ8wWw2u8H3/dZ8"
+ "Pn/Y9/3dY2Njb4XPpJQxzrmlylQONM45t6SUsXB+LpdbIYRoLZVK8SAIjs/MzLxfF8yjR48+p5Qm"
+ "KaW3VPxBKb2Qz+fXhnMYY72cc1oBiDLGeiPwG0ql0vXZ2Vk5NzcnpZRSCPG4WCzuq9kZBXOTUpqO"
+ "xHAul4uHv3xycnIL5zzDGMtxzu+p/ZPjnGcmJye3hE4KIS4riOlIlIQQj2tyyvf9VuVKFCbt+/7v"
+ "lFJ7YmJiXTiXc97COQfO+YgK4Jy3hM+np6c3CiGeCiGKZUDTUkoZBMHxqkD5fP6wKlG6LG5QSq9l"
+ "s9nN0flSyoZCodBcKBSay4/78+fPtwkh/hJCzJQDzc3NSSFERy0O7a4E5Pv+TUqp8eTJk3eqJvkX"
+ "do0Q4r74Ry8Bzc7OymKxWL09jI+PN/q+f4FSOqzKdEPB3KKU7q8VJlQQBD8oNwLlVFGV6/rExMTb"
+ "NSXJ5/Nrfd/voJTalNJrlFKDUvp19CjXqng8viwIgh+FEPdLpdKfQoinQRAMPHv2bEO9ubQHDx68"
+ "l81mN4+MjNRcpldJSrlaSrlVSrmx3nfrdmHBJaWMeZ73RSaTOeV53ulMJnNsdHR003+1nq7r6wFg"
+ "HwAcBYCDhmF89GI7SCljmUzmG8/zDM/zdNd1r7iui1zX/fXOnTsfLjQMQmgTALQBQAIAugHgtBrv"
+ "1TQtpt29e/cD13UHPc8bcF33chgKriudTi9fKBhd1xswxkcQQj0A0BGGYRidCKEu0zQ3aq7rtitX"
+ "LpcBDWQymd+Gh4fXLyBQEwD8ZBhGZxRIRTfG+NulB7TkSrbkNrWmLbFjX0H/f2N8lUzTXJNMJten"
+ "0+nG+ebq7+9vBIB1uq6vrvtl27ZXOY5zwLbtLkLIGULIKULIZ69zuUopYxjjTzDGJxBCcQBowxjv"
+ "tyxrZU0JdF1/kxByjBBy3nGcs4SQnwkhvYSQPtu2P60XCAA+BoAExrhLnapOADhjGMah27dvv1E1"
+ "wdDQ0HbHcc4pkGj0EkJO6bq+olaY/v7+RozxiQjMi0AI9ViWtaVqEkLIV4SQvnIg27bPOo6TSCaT"
+ "LzVKKWVM1/UmXdebyksKAOsQQvFXNMIEAOypCnT16tVdhJDz5UCqfJ2maa4J5w4ODjZjjI9gjNtV"
+ "HBkcHGwOn+u6vlr1nIpAlmW1VISIynGcJkJIhypR1KFfHMc5ELqgekkbQqjHMIxO1W17AKBN1/XQ"
+ "xZhhGPsB4EyFa+LkwMDAu1WBNE3TUqnUNkJIh+M4fYSQPsdxztm2/b1t26si5dinbK9Uihd/Ai3L"
+ "WokxPqRgE2qDn8QY13cLOI7TlEqldqVSqS+Hhoa2X7x48aUTobpsd6VLEgCORuem0+nlhmFsBYA9"
+ "lmW11OxMPQKAg+oeKgc6DQAHF3zBajJNcwcAJKInSI0TpmnuWHSgeDy+DAD2IoS6VJm61XhvPB5f"
+ "tuhAoRBCmxBC36mY9xfC31PwzDQAtx5SAAAAGmZjVEwAAAAJAAAAJAAAACQAAAAAAAAAAAAfA+gB"
+ "AWvmYY0AAATVZmRBVAAAAApYhc2WQWgUVxjHZ42W0FaCikHBNoFC0XixIKiIl5w8NVoI9BRLQJT2"
+ "FijatbJJdNeLx0IPimuVGJz3vrdzEFMLugUvzts9qCwhuPNmoA3rYfKesWQzn15eL29kM6zsrknT"
+ "fPDBY+fNf37z/973zVrWKiMMw4NhGN40eXC1eh8cWusuKeU5pVRFSjkrpZw163Na6651B1JKDUkp"
+ "q0qp51LKZ1LKZ2ZdVUoNrTvQwsLCNaXUXAzTADW3sLBwbc0fGATBLt/3hzzPOyWEOFapVD6Kr2mt"
+ "U0qpu0qp2SZAs0qpu1rrVMP+bkQ8gYgXoig6G0VRf0cwnucd9X2feJ5XFEI8EkI89H3/6vz8/I54"
+ "TxiGWSllNQkkpayGYZiN9y0tLe1GxBlEXEYTb968Cer1+tedOEN83/9DCHG/If8UQvwUv3kYhnul"
+ "lE+UUnNKqecm56SUT8Iw3Bs7iYg3EPEtIsqG/AcRg7ac8n1/SAhRTMDc9zzvdyEEe/Hixc547+Li"
+ "4kGl1B0pZVlKWVZK3VlcXHzX+vV6fQ8i/oWIrxJAEhExiqKz7QCNmDLdT+SMEOJeEAQr3sq27a5a"
+ "rdZXq9X6bNte0e6IOICIISKqZkCIeKElkBDimBDiYROgB57n3a5UKp+2FDGhte6JoqiCiK+bAC3X"
+ "6/UTLUWCIOj2PO+qOTMzJh8IIR55nneyXZg4oij63rjx2jj1Koqit4g4o7X+uC2R+fn5HdVqNS2E"
+ "YEKIe57n3fY875vGVm43tNapKIp+QMSKKd/fy8vL+aWlpd2dalm1Wm1nEAT9L1++/KTjmxOhlOpB"
+ "xIF6vb6n03s7dmHNQ2udKpVKR0ul0pjruhc556OPHz/+/L96nm3buwDgJACcAYARQshX746D1jrF"
+ "Of+2VCrd4pzf5JznS6XSLdd1f3Fd98u1hqGU9gFAGgByAHAJAC6b9XHLslJWuVz+wnXdG5zz65zz"
+ "a3G6rvub67oXisXi5rWCsW27izF2mlKaBYDxOAkhE5TSyUKh8JnFOf+Rc55vhDF5nXP+69OnT3vX"
+ "EGg7AFwkhEw0Apm8xBg7tfGANlzJNtyhtqwN1vZN4v8fjO8L27Z7AKA3n893r1Yrn893A0Cvbds9"
+ "Hd/sOM5WxtgwAKQppRnG2Bhj7MiHflwZY0cYY2OU0gwApBljw47jbG1LoFwubyGEjALAFQCYAIBx"
+ "xtikOXiHOwUCgMMAkDMa40bzCiFktFwub2kp4DjOQLItYyjG2Fgn5cvn893G3cmkHqU06zjOQDtv"
+ "NGjcWCFgZkUGAFYMSq11yrbt7bZtb0+WFAB6KaWZ9wzCHAAMtuPQoWZAxup046Gcnp7uZ4ydZoz9"
+ "bPL09PR0f3zdNEU6Ln0SyHGcQy2BpqamtjHGzpuh1ShwhRAybJmxYGZJmlKaJYRMGAezBnqXkUsR"
+ "QobNeUx+Js5PTU1tawlkWZbFGNtnoHIAkKOUZhljo42dYQZbMydzAPDuv7fp2FEDmzMH/DxjbF9b"
+ "MI1OmfINEkL2J79pZsomXRw3v51p3FssFjcTQvYDwKDjOIfadqaTAIAR8x1KAl0GgJE1f2CrKBQK"
+ "BwAg19hBZp0rFAoH1h0ok8lsAoDjlNJJU6ZLZn08k8lsWnegOCilfZTS70z2rVbvX0O3rH7Z2Q0z"
+ "AAAAGmZjVEwAAAALAAAAJAAAACQAAAAAAAAAAAAfA+gBAYZwsmQAAATgZmRBVAAAAAxYhc2WX2gU"
+ "RxzH7/zX0GLFYFPB+qfQin9e1JeoTyVPglqwkBcpUiLqQ+lLpCKhcCZ6JxT6oA1W8eFAOQI785vb"
+ "gm2hL3u7paDkRCOXmUsgD1LDzg53kRy/lYo5pg+dLes16d2ZaPODHyw3s9/5zPf3m7lNJBYZSqm9"
+ "vu/f8H3/hlJq72L1Xjm01iullP1BEIwGQVA0OSql7Ndar3zjQL7vHwmCYCwIgntBENw1eS8IgjHf"
+ "94+8cSAp5bCU8kEM5m4QBHellA+klMNLvuCjR4/en5iYOCyEOC6EOFgqldZEY1rrpJTylpTy/jxA"
+ "96WUt7TWydj8jtnZ2aOIeA4RTz579mxLWzCTk5P7Oee3hRA/mbzDOU+XSqXOaI6UMqWUGmsEUkqN"
+ "SSlT0TxE3IiI+TAMK2EYVhFxBhH57Ozs4Zad4Zzf5pz/KISAKDnnvwghzkY7V0ptD4LAUUo9ML1z"
+ "zzw7SqntkZOI+EMYhk8R8Y8owzCUiMhbcopzfti4Ag1Aec55rlwub4jmKqX2SSmzUsrfTGaVUvui"
+ "8UqlsgkRJxBxOg5kcgYRTzYFEkIc55z/C0gIwcrlMpmamnppV5ZlraxWq5ur1epmy7JeOu61Wm0H"
+ "Ij5GxCcLAJ1rBeigEOLOPED2+Pj4zYcPH77TVMTEzMzMOkQcDcPQbwQKw7CCiM2vh8nJybc452kh"
+ "xM9CCGbSNmU82ipMFLVa7TQizhioJ4g4bXoqPz09/XZLIqVSqVMIcdb0DBkfH79ZKpU+jR/lVkNr"
+ "nQzD8DQijpryTSDidUTc2K5Wolwub5iamtrS8k7+IyqVyru1Wm1HGIab2n23bRdeNRZ0XGud9Dyv"
+ "23XdL13X/drzvM8dx/ngdYE8f/58n9Y6W6/Xi/V6/de5ubk+rfWKaDxZKBQ+KxQK113XveZ53nCh"
+ "ULjued63juN8tNQwL168+ERr/VQ3RL1ev6q1TiZc1/3Q87zvXde96rrulVheKxQKZ5fyU0JrvaZe"
+ "r482wsTiYMLzvK88zxtugLliAL/zPO+9JQT6WGv950I0c3Nzvy8/oGVXssRya2pDvqyOfWP8/xfj"
+ "QmFZ1joA6Mpmsx2LXTybzXYAQJdlWevaftm27bWMsV4AGKCUphhj/YyxA23vKPG3C4yxA4yxfkpp"
+ "CgAGGGO9tm2vbUmgWCyuJoT0AcBlABgEgAuMsSEAyADA/naBAGA/AGSMxgWjeZkQ0lcsFlc3FbBt"
+ "exelNG1e/icZY0OMsf52ypfNZjuMu0ONepTStG3bu1rZUY9x4yUBQsigsbwrPl9rnbQsq9OyrM7G"
+ "kgJAF6U0RQgZbNQza/S04lD3fEDG6oF4U46MjGxjjJ1ijH1j8tTIyMi2aNwcioGo9I1Atm13NwXK"
+ "5XLrGWPnAeBig8BlQkhvwlwLlmVtNE2fJoQMGgfTBjr6GkwSQnpNP8a1LjLGzudyufVNgRKJRIIx"
+ "ttNAZQAgQylNM8b64icDAI4t4GQGAI7FHF/LGOszsBnT4OcZYztbgok7ZcrXQwjZ7TjOqvg4AJyZ"
+ "x8UL5rcz8bmO46wihOwGgB7btrtbdqadAIATAHBpHqBLAHBiyRdsFvl8fg8AZOInyDxn8vn8njcO"
+ "lEqlVgDAIUrpkCnTRfN8KJVKLfhH+dqDUrqVUvqFya2L1fsL4pPswuTb59MAAAAaZmNUTAAAAA0A"
+ "AAAkAAAAJAAAAAAAAAAAAB8D6AEBa7rAHgAABPpmZEFUAAAADliFzZZPaFRHHMd3/de0kEoUooii"
+ "BUHUgzm0RC895GIq9OAhPWoJtR5KDuYUQ3FNNKvWRmhOIsKCuKy8md/kHfTNS4SuGGxFtuG9WQkl"
+ "FgM7s3tZQ2YIJGzekulltjyXpLtrouYHPxh25n3nM9/fb5iNRNYYnPNjnPMRk8fWqvfOYVnW5lwu"
+ "1yOEeCqEmDD5NJfL9ViWtfmDAxUKhU4hxHMhxBMhRNrkEyHE80Kh0PnBgTjnw/l8/lkIJi2ESOfz"
+ "+Wec8+F139DzvNZsNnuSMdbleV77y5cvt1XmtNZRIcQdU6Z0VU4IIe5oraOV9TMzM02zs7OnlFIX"
+ "lFJnZ2dn9zUEk81mv2KM3fF9H/u+jxljFmPsEmOspbKGc37RlKwa6Dnn/GJl3fz8/C6l1AMpJZdS"
+ "CqWUkFJOSilP1u2MgXng+/79UAJjrKdy8lwud1AI4XDO/zC988SMnVwud7DipJTyNyllQUo5XUml"
+ "1IxS6q+6nMpmsyeNM2GY+4yxJGPs7tTU1M7KWiFEmxDiNud8nHM+LoS4LYRoq8wXi8U9SilPKfVP"
+ "GMhACaXU2ZpAjLEuxhhaBejeixcv3jqV1noT53wv53yv1npTeK5YLB6SUv6tlHq1CtCFmkCe57V7"
+ "nrcSUMr3/ZF8Pv9ZTRETb968+VxKOaGUel0NJKXkSqlvaopMT09/whi7ZHom6Xle0vf9lCljbYGq"
+ "mJub6zZuvFZKvTLlK8zNzT3QWn9alwhjrIUx1sMYu8sYu+f7/sjk5OSp8FWuN7TWUaVUt5RywpTP"
+ "U0qNzM/P72pUKzI1NbUzm83uy2QydZfpf8Cai8XioWKxuGetWu8tVnVcax199OjRl5TSH1zX7aGU"
+ "fjc+Pv7eTlIqldqCILgbBMGf5XLZKZVKZ8M3NOo4zreU0luO4wxTSm+6rnvLcZzBsbGxL9YbZmFh"
+ "4esgCIrLy8s6CIJyEATLZvyr1joaefjw4X7XdX9xXfcGpfR6JV3XHaaU/rSefyW01tvK5fJEuVzW"
+ "QRAshHIxCIKlpaWl45GxsbEfKaU3wzCU0uuO49yglF59/Pjxztpb1ReLi4sHgyCQBmChKoNSqfT7"
+ "xgPacCWLbLSmNuQb6tp/tGj4KbIsazsAtCYSiaa1bp5IJJoAoNWyrO0Nf2zbdjMhpAsA+jHGMUJI"
+ "LyHkxLs+roSQE4SQXoxxDAD6CSFdtm031yWQyWS2IoS6AeAaAAwAwGVCyCAAxAHgeKNAAHAcAOJG"
+ "47LRvIYQ6s5kMltrCti2fQRjPGQ+/i8JIYOEkN5GypdIJJqMu4PVehjjIdu2j9Rzog7jxlsCCKEB"
+ "Y3lreL3WOmpZ1g7LsnZUlxQAWjHGMYTQQLWe2aOjHofaVwIyVveHmzKVSh0ghJwjhPxs8lwqlTpQ"
+ "mTeXor9S+mog27bbawIlk8kWQkgfAFypEriGEOqKRCJRs9lu0/RDCKEB4+CQgd5t5KIIoS7Tj2Gt"
+ "K4SQvmQy2bI6SSgIIYcNVBwA4hjjIUJId/hmAMDpVZyMA8DpkOPNhJBuAxs3Dd5HCDlcF0zYKVO+"
+ "DoTQ0XQ6vSU8DwDnV3DxsvntfHhtOp3eghA6CgAdtm231+1MIwEAZwDg6gpAVwHgzLpvWCtGR0fb"
+ "ACAevkFmHB8dHW2rrbDOEYvFNgFAJ8Z40JTpihl3xmKxj/dQYoz3Y4y/N7l/rXr/Ase2z/FqihZ0"
+ "AAAAGmZjVEwAAAAPAAAAJAAAACQAAAAAAAAAAAAfA+gBAYYsE/cAAATQZmRBVAAAABBYhc2XT2hU"
+ "RxzH38Y/tYVUDSX4p0WpeFBJsUWq0lus1qM55CS1JRDixYseDKFlY2I2oL1UhSJKlygh8GZ+v31e"
+ "Goy224vLvjcPltpYcPNmIfYQsnkz0C6bTE7Ty2x5XWPz1kTND34wvJn9zed9f7+Z31vLWqFxztuC"
+ "ILgaBMFVznnbSuO9tNm2vS4IgrNBENwvlUoTpVJpIgiC+0EQnLVte91rB+KcH+ec/8I5H+ec/2R8"
+ "3Dw7/iaAhjnnDyMwNX/IOR9e9Q1933/Pdd1jvu+fcl33kO/7G2pzWutEqVS6XiqVJuqBTPqua60T"
+ "kfVvzc7OfhGG4Tkp5ekwDN9vFOYTz/OuMcbuMMbueJ43whi7WCgUttTWcM4vBEGQrQcyzy7U1lUq"
+ "lVYhxIiU8qkQYkoIMSWlfBSG4eexlfE875rneWnP827VnDF2N5/P99TevFgs7uGcI+f8Z1M742aM"
+ "xWJxT03JMAyvSikDIcRvNZdS/hGG4aNYSrmue8yocqvOb7uueyOXy7VEVGqbmpr6nnN+j3N+z4z/"
+ "Pfrlcnm7ECIvpfw9CmR8Skp5Oo5Cp1zXfQ7Idd3bjLGbuVxuZ3S91rppenp6x/T09A6tdVN0rlwu"
+ "7xVCFKSUj5cCCsPwXByFDnmeN7IE0I+MsSu+77+zbBBjxWLxXSHEAynlk3ogKeXTMAxPLBtkcnJy"
+ "o+u6Fxljd02abhuYO4yxhu8XKeWXJj1PpJSPTfoCIcTIs2fP3o4VpFAobGGM9biue4MxdpMxdiWf"
+ "z5+IHuW4prVOSCnPCCEeCCEKQoh8GIbfzczMtDYay8rlci2FQmFnLpeL9yb/Y3Nzc83lcnlvuVze"
+ "vtJYr8xeqLjWOkEI+RgAzgBADwB02La97VWBVCqVj5RSPyilfl1cXMwsLi6ejp7QBACcBIAUAFwG"
+ "gEEz7qOU7lptmPn5+c+UUn8qpdTCwkJFKVU142GtdcLKZDIfUEoHCCGXAKC/5pTSIUTsXs1PCa31"
+ "RqXUQwMhIi6VUn9Vq9VPLUT8yqjSH3UD+K1t2y3LbxXPFhYWPlRKzRgAUed/V6vV8bUHtOZSZq21"
+ "ojbka+rYvzFruBXZtr0ZAFrT6fSmlW6eTqc3AUCrbdubG/6x4zjNiNhp6iiJiOcR8ejLNldEPIqI"
+ "5ymlSQDoQ8ROx3GaYwXwfX8DIaQLAIYB4BIA9CPigCn0I40CAcARAEiZGP0m5jAhpCv6x+GF5jjO"
+ "fkrpUP29hIgDiHi+kfSl0+lNRt2B+niU0iHHcfbHeaN2o8ZzF6WR/D/fMVrrhG3bLbZtt9SnFABa"
+ "KaXJ+jvOeAoA2uModHgpICN1X7Qox8bGdiNiNyJ+Y7x7bGxsd23eHIq+WurrgRzHObws0Ojo6FZE"
+ "7F2inQwTQjoty0qYzbaZoh8ihFwyCg4Z6NodliCEdJp6jMYaRMTe0dHRrcsCWZZlIeI+A5UCgJRp"
+ "I13RkwEAHS9QMgUAHRHFmxGxy8CmTIH3IuK+WDBRpUz62gkhB7LZ7ProvLnRn2vK5llPdG02m11P"
+ "CDkAAO2O4xyOrUwjZtrM5SWALgPAmVXfcDnLZDIHASAVPUFmnMpkMgdfO1AymWwCgJOU0gGTpkEz"
+ "PplMJt9co6SU7qKUfm18xZ8r/wC1EbGjrrVtJgAAABpmY1RMAAAAEQAAACQAAAAkAAAAAAAAAAAA"
+ "HwPoAQFqLaTnAAAE22ZkQVQAAAASWIXNl11oFFcUx2djtZU2pIloDVI19alqaQqF1iervgg+CbWh"
+ "fbA2EPTFVuxLkZZNYrJ+ILSGgNYXn0LsnTl3V1KhsMLOnUlDKyuYNrk3XSiUNST37tJ2Y2YM1gyn"
+ "L3fKsCbdXRM1Bw4Mc8+e+c3/nLnnrmEs0XK53HbOeQ/nvCeXy21far7HNkLIKiFEO+c8xTm/rj0l"
+ "hGgnhKx66kBCiD1CiBtCCCqEAO1U39vzLIA6hRBDEZjQh4QQncv+wHQ6vc5xnN2MsQPDw8Ot2Wx2"
+ "dbiGiDHO+QXO+fVyIF26C4gYC+NzudzzUsp9UspjhUKhLZ/Pb6oJxnXdN13XPWvb9mXtlxhjJ9Lp"
+ "dEMYI4Q4rstTrtANIcTxMG5qamq9UuqKlPKOUmq0UCiMKqVuFovF6sqaTqfXMcbOOo7Tzxi7GPFv"
+ "GWNHwjefmJho4ZwP6BJR7UOc84GJiYmWUMnp6elepdSvSqmfQpdS3lZK3axKKcdxdtu2fbkM5iJj"
+ "rM9xnPOu6zaGsePj4zuEEOeEENe0nxsfH98RrkspX1FKZaSUt6JA2n8pFAptFYEYYwccx7lUDmTb"
+ "dh9j7BvGWHM0HhHrOOfNnPNmRKyLrimltimlRpRSPy8ANCqlPFYRaHh4uFX3TDlQP2Osa2RkZG3F"
+ "JNqKxWK9UmpISpktB5JS3pFS7quYZGxsbI1t25/pnumzbbvPtu1+27Yvu677XrUwoUkpP9RqZLVS"
+ "t3RPXcnn89W9XDqdbshkMp84jnPecZyvGWNdtm3viX7K1RoixqSUH0kph3T5MlLKxNTU1Ppacxmu"
+ "6zYyxpprKdNippR6SffUxqXmemK2qOKIGDNN8y0AOAwARwHgICHkib3J7Ozsznv37l2cnZ39wfO8"
+ "a/fv32+LfqExANgPAAkA6AGA0/r6lGVZW5YbZmZmZpfneb95nveX7/vS87yi7/t/+r7fhYgxI5lM"
+ "vmpZVrdpml0A0Bm6ZVm9lNKO5TxKjI2NrfF9/3vf94ue592N+KTv+9OlUultg1L6sValM+oa8CtC"
+ "SNNyAc3NzbV4nve753mTZUB3fd+XMzMzyZUHtOJKZqy0pjaMlffZPzOreRQRQhoAYMPVq1dfWIaH"
+ "v/zgwYOdiLi55h+nUql6Sukh3UdxSulJSumuxxyudYj4OSL+gYj/IGIpCILvELG6c3U2m11tmmY7"
+ "AJwBgC4A6KSUdutGf/cxgE7iAhYEwY+I+GLFBKlUartlWb3l+xKltJtSerKW8pVKpUZEnFwICBFx"
+ "fn7+g4pJAGCvVuORjdKyrDgAbChTIEYIaSKENJWXFBHfQMRgMaAgCHqrUeidhYB0+U4RQv77KzQ4"
+ "OLiVUtpBKf1Se8fg4ODWCNDmIAi8/1HoREWggYGBRkrpFwuMkzOmaR4yDCNmGIZBCNmom77XNM0u"
+ "rWCvht4YqhcEAVmE529EfK0ikGEYBqX0dQ2VAICEHiPtqVSqPowBgIOLKJkAgIMRlTYholsGU3z4"
+ "8OH7VcFEldLl22ua5o5MJvNcdF3v6I8MZX3vaDQWEdci4qEgCLrn5+c/nZub21YTTDWmx0zPAkA9"
+ "AHB42R9YyZLJZCsAJKKnBH2dSCaTrU8dKB6P1wHAfsuyunWZTuvr/fF4/NkNSsuytliWdUT7ko8r"
+ "/wJtie+EF4iaDgAAABpmY1RMAAAAEwAAACQAAAAkAAAAAAAAAAAAHwPoAQGHu3cOAAAFKGZkQVQA"
+ "AAAUWIXNlk9oVEccx9+mta3Ufw1igyCxnvwLtof+wVbEQ+s1WHsQtEURLUgLnhpJ2bgmURFjwZMH"
+ "0UJiwrz3m7eH9c0soa7FgwoedmY3l1gbNm/WKIo7j9ZkfZNkepkH23V1Nya1/uAHj53fznze9zvz"
+ "fmNZcwzO+VrGWAdjrINzvnau8710aK2bcrncHs75ZcbYIGNskHN+OZfL7dFaN71yIMbYVs65zTnv"
+ "Z4z1Mcb6OOf9nHObMbb1lQNxzts55yiCqYBCnPP2eV+QUtqcTqe3pNPpL4eGhjadP39+QTSmtY5x"
+ "zruMVX1VOcg579Jax6L6kZGRt33f31YsFvf7vr+zUCisnBVMOp3eSAjppJT2EkJ6KaVnPM/73vO8"
+ "JVFNNps9xBhzagA52Wz2UFRXLBaXCyHOCSFuCCFumkzdu3evMVsppc2EkE5CyGlCyMkoPc/7hVK6"
+ "O3rz4eHhVsbYBWNRv0nEGLswPDzcGik5NjYWF0LcEkJkKvK6ECLVkFJXrlzZQgg5WwlDCDlJKT3l"
+ "eV7Cdd1lUW0ul1vHGEtks9lL2Wz2EmMskcvl1kXj4+PjK4QQnu/7v1cBZYQQN33f31kXiBDyFSGk"
+ "txrI87xTlNKeVCrVUlmvtW7K5/Mt+Xy+pfq4FwqFNUKI34QQ12oA3SgWi/vrAg0NDW2ilJ6pBjIW"
+ "tiOEFtadxMSDBw8WCSFs3/ev1wLyfX9b3UkQQm8RQg4RQs5SSk9VwPR6nvd5ozBRCCF2GXuuCyGu"
+ "Gftu+b5/bnR09J2GJvE8bwmldLfneQlCyAlCSDsh5IvKo9xoaK1jQohvhBC2sc/zfb+zWCwun+1c"
+ "luu6y1KpVEsmk2nsTV4Q+Xx+UaFQWHP37t335zrXfxbPVVxrHbNt+0MA2AsABwGgDSHUUrN4HuLR"
+ "o0frpZSnpZRJKeWvpVLp68oTGgOAHQDQAwBdAHDcPB91HKd1vmGklB8HQZANgkAEQTAaBEFBSimk"
+ "lB1a65jluu4qx3EStm0fA4DOKB3H6cYYH0AIvTFfMFrrBVJKkFKOSSlHogyC4E4QBH+WSqWPLIzx"
+ "t0aVzso0gD8jhJrnC6hUKq2WUuaDILhTCWSgRh8/fjz4+gG9dpZZr9umNuSv1bH/32LWrQghtBQA"
+ "Vly8eHHOrUNrvVRrvUFrvWrWf04mk4sxxrvMPopjjI9gjD97meYaj8ebwjD8USn1h1LqL6XUwzAM"
+ "+588edLYvfr27dsLbNveBwAnAOAYAHRijBNmo386W6AwDH+YmZnRSqlQKTWplCprrXUYhlfHx8ff"
+ "rTtBMplc7zhOd/V3CWOcwBgfmY19WutlRhmllJqozOnpaV0ul+tfYQFgu1HjmQ+l4zhxAFhRtWgM"
+ "IdSMEGqutvTp06cblVJ/G2X+BWRU62xEoU9qARn7jiKElka1AwMDqzHGBzDGHSYPDAwMrI7GJyYm"
+ "VimlHiqlytVAxrbDdYH6+/vfwxj/VKOdnLBte5dlWTHLsiyEUIvZ9N22bR8zCnYb6JZIPaVUn9Za"
+ "VwFNKaXuT05OflAXyLIsC2O8zkD1AECPaSP7ksnk4qgGANqeo2QPALRFdVrrlVNTU1enp6f1zMyM"
+ "NnD3y+VyW+3VX6CUsW+7bdsbMpnMm5Xj5ov+TFM2vx2srB0bG1uolNo5NTUVD8PwcMPKzCZMm+mq"
+ "AdQFAHvnfcF64bruZgDoqbwlmOce13U3v3KgeDzeBAA7HMdJGJuOm+cd8Xj8/2uUjuO0Oo7znck5"
+ "X1f+AUpr1CfNCTBqAAAAGmZjVEwAAAAVAAAAJAAAACQAAAAAAAAAAAAfA+gBAWpxBXQAAARPZmRB"
+ "VAAAABZYhc2XQWhcRRjHd6OVopZgS0MKjc2p1XiJIKiVXnrKySZCoadWAiFFb4GiJoZN0mR76VHo"
+ "odENSlycmW92D8Wo0KRH37wctrIWSWbeA7skhN33grIv70sv48F5sl027luzjfnDB495s9/+9v/N"
+ "7DeTSOxTlmWdFULcEELcsCzr7H7z/WdprTuEEFds2/5SCJERQmTM8xWtdceBAwkhztu2/bUQYl4I"
+ "cdfEvBk7f+BAtm2PCSEWamCiWLBte6ztX7i4uPhKPp9/GwAuUkrfWFlZeT56p7VOWpY1aUpVD5Sx"
+ "LGtSa52M5heLxReUUheklNccx7nkum53SzCc89c5558CQBoA0oyxOc75cD6fPxbNEUIMm/I8BWTG"
+ "hqN5pVLphOM4t5VS95VSy1LKFcdxqJTyvdjOGJibADBVE7copZcTiUTSAPVYlvWFKdu8iQUz1hM5"
+ "qZT6TCn1QCn1fRSO4/zkOA6N5ZQpU7oOZgoApgFgnBDSWePSOcuyJoQQd4QQd8zzuej92traSaUU"
+ "l1L+UAtkYsVxnEtNgQDgYiMgSuk0YywFAF2181OpVEehUOgqFApdqVTqqe3uum6vUuqeUmqpAdCy"
+ "4zhX4zjUxxibqwfinM9wzscymczRpkmMisXiy1LKb5RSPzYAuq+UutA0yerq6hFK6TAA3DJlmuKc"
+ "zxjX3okLE0lKOaSUWjZQSyYeSClvu64b78fl8/ljnPPLADDOGEtxzsc45+/WbuW40lonpZQfGKfu"
+ "KaX4+vr6eKlUOtFqrgQhpBMAulop017a3Nx8yXXd3o2NjZP7zfXMtKfjWuskpfRNALgKAKMAMEQI"
+ "ae3ftAWVy+XXPM+bq1Qq31Uqlbvb29uDtQ05CQADZtHOmj/EtFk/Z54BzFue5/3sed667/uPfN//"
+ "zfO8dc/zPtFaJxO5XK6HMTZDKZ2u3eamXYwQQp5rF4zW+kilUvnWQDyMwvf9XzzP+3Vra6s/wTm/"
+ "1qBNTBnASULI8XYB+b7/qu/7tgF4WBePyuXywuEDOnQlSxy2RW3ID9W2/9/UcitqZ+vwfb8TEfuC"
+ "IDjd8ofb3VzDMPwYEYuIWEbExzs7O5lqtXoqVoJ2Hz/CMPwI/9YfiOgj4nYYhk8QcUlr/WLTBO08"
+ "oGmtO8MwLBoYry52giAYbJqk1SOs1jpJCDlOCDleX1JE7DNl8hsAISJOxHEo9iE/m832cs5HOOef"
+ "mxjJZrO90fsgCE4j4u+IuN0IKAzD602B4l6DCCHdZtHPUUqnjYNzBro7cg8Rv0LEJ3UwfyKiG4Zh"
+ "77+x/KM4F0UAGNrDyTQADEXzqtXqKURcQsQdUybc3d11gyB4PxZMrVN7XaUN0GijpmzGRmvnaq2P"
+ "IuIgIk6EYXg9tjOtyLSZ2QZAswDQ/L7VbuVyuX4ASNeeEsxzOpfL9R84UCqV6gCAAcbYjCnTTfM8"
+ "UH+DPVAxxs4wxj40se/jyl/FjbMYnT853QAAABpmY1RMAAAAFwAAACQAAAAkAAAAAAAAAAAAHwPo"
+ "AQGH59adAAAEbmZkQVQAAAAYWIXNl0FoFFcYx2fT2IrFhopaIWgParEtHtKDbb2Ingo2hQpeFFpU"
+ "ag+ChwRFhLJJ2kQQPDSG1OIhEMllZt7bLZj2JDtvsqyWbEkMu+9tFnIozbJvlm3qLt+EXYb16+Ut"
+ "TLZZd7fZaP7wXd57881vvu97872naRtUNBo9aFnWVcuyrkaj0YMb9fe/hYgdlmWdsW37rm3bY8ru"
+ "WpZ1BhE7XjpQLBY7ZlnWPcbYKGPsR2WjauzYSwdijF1hjI37YKo2zhi70vYXTk1NvR0Ohz8mhJwy"
+ "DOPDSCTS6ZsOMMau2bY9Vgukxq5pmhaoLk4kEq8LIY4LIc4tLi6eXlhYeKclGErp+5TSG4SQEULI"
+ "iGmaw5TSi+FweGd1DWPsvErPGiA1dt4Hs4tzPiyEeCiEmBZCTHPOH6TT6U+ajoyC+Z4QMuCzW4Zh"
+ "nK1+eSwW67Zt+7ZK0aiycdu2b8disW5N0zREDAgh+jnnvwkhSNU4579wzh80FSmVppEamAFCyCAh"
+ "5Kau613VtZFI5JBlWf2MsTuMsTuWZfVHIpFD1flUKrWbcz7FOQ/5gZRNc85PNwQihJxaD8gwjEHT"
+ "NIOEkL3+9cFgsMO27T22be8JBoNrtvvS0tKBVCplCCFoLRDnfFoIca6ZCH1gmuZwLRCldIhS2jcx"
+ "MbG9oROl+fn5N5PJ5H0hRHidCD0UQhxv6CQej28zDOMiIeSWStMApXRIRa25QvRJCNGrijmsIkWF"
+ "EL9yzofT6fQbTTkJh8M7KaVnCSE3TdMMUkr7KKWfImKg8dNrhYiBRCLxRTKZvJ9KpQzO+ZQQoj+R"
+ "SOxq1Zem63oXIWRvK2mqp0wms0PV1O6N+to01Y04IgYMw+ghhHxFCPmWEPKlruv7Ngskl8u9J6UM"
+ "SiknpZRj2Wy219+QA4SQz1TR/qB+iCOqft5tN0wmk/nIcZxILpd7KqX8Q0o55zjOUyllHyIGtFAo"
+ "tN80zSHDMAb921y1i290XX+tXTDxeHyblHIil8vNOY7zxGe/O44zK6U8qlFKv16nTQwowO90XW99"
+ "N9TR8vLyfinljAJ4UmPxbDb789YD2nIp07ZaUWva1tv2r0wtt6J2to58Pv9WsVg84rpud8sPt7u5"
+ "uq57GQBmAeBPAFgEgHsA0FxJtPv4USwWLwPAiuu6WQBYBoCM67r/AEAok8nsaOignQe0lZWVLgCY"
+ "VTB/+c113TwAfN7QSatHWETsLJVKh0ul0mFE9F+VtGKxeESlabkWCABWAOB6MxFq+pDved5JRHyM"
+ "iKvKHnued7I6n8/nu1XNZOoAXWoI1Ow1qFwu9yDiM/yvnpXL5R4VvQAA/KRqxp8uCQB8dXX1QEMg"
+ "TWvuooiIk+vAVDVZXQcA+wAg5Lpu3nXdv1VkeKFQaHwFqo1Uvas0IgYqlcpcPRo1F/Ct314oFHoB"
+ "4DoAXGo6Mq2oUqk8egHQo7a/sJEQ8cILUnbhVQB1ViqVUUR87gN5rsY6G3vYPLATnufNeJ43g4gn"
+ "NurvX65q63PfRXHTAAAAGmZjVEwAAAAZAAAAJAAAACQAAAAAAAAAAAAfA+gBAWqU58EAAASlZmRB"
+ "VAAAABpYhc2XX0gcRxzH94xp04ANNiEpFikJKaHNg5RQbB+Sh7ykBvLQB/OoIGn7EgPJUxtoTq0x"
+ "JqJQXwoiCCES2d0ZFxFntg+xVNsEucrunEgwIYKzFx/u5GYQeuzOctOXOTgX5e7qaf3BD5bZ2d98"
+ "9vv7zT9N26XZtn3atu3vlJ/ebbz/bPF4vMa27WsY40cIoQGE0ADG+JFt29fi8XjNvgPZtn0BITSE"
+ "MX6IEOpHCPWr5yHbti/sOxBC6MbMzMxgAabgqu1G1QccHx+vtyyrGQBw2TCM87Ozs7VFr2MIoVsq"
+ "Vf0RH0AI3dI0LVbovLS09I7jOM2EkNZkMnnFcZyTFcFACD+FEP4AAOgDAPSZpnkfQthhWVZdoQ9C"
+ "6DrGeCgKpNquF/oRQuoJIfcIIbrruqbruiYhZCSZTH5RtjIK5mcAQFeRPzAMo7Xw55ZlNWCMezDG"
+ "gxjjh8oHMcY9lmU1aJqmSSljjuN0uq4LXNd9UuQThJCRspRSaeqLwHQBALoBAHd1XT9W6Ds9PX0G"
+ "Y3wTIdSLEOrFGN+cnp4+U3i/vLx8nBAySggZjwA9cV3XTCaTV0oCAQAubwdkGEa3aZpxAMCWv4rH"
+ "4zVTU1MnpqamTkSn+8LCQiMh5PF2QIQQgxDSWo5Cn5mmeT8KBCHsgRDeGRsbO1IyiLJUKnXUdd1h"
+ "QsjTKJDjOIbjOM0lgyQSicOGYXQAAB6oNHVBCHuUal+WC1Mw13VbVDE/dRxnXKkFCCH3VlZW3i0r"
+ "iGVZdRDCVgDAXdM04xDCOxDCr6SUsdJfbzUpZWxxcfGqUuqxqqlOQkh9pbE0XdePAQBOVpKmnSyR"
+ "SBxNJpONy8vLx3cba89sR8WllDHDMD4HALQBAL4HAHyj6/qHewVCKf2EUvqj53kjlNJBSmlL8QyN"
+ "AQC+VkXbqxbEPlU/H+8BTJPneTOe572glM6nUqk/Pc97sba21imljGmTk5ONpmn2GIbRXTzN1Xbx"
+ "ra7rh6oFk0gkDlNKf6WU/uV53myR/+553h/r6+vnNQhh+zbbRJcC/EnX9Q+qBZTJZD6ilP6mAGYj"
+ "PkcpHT54QAcuZdpBK2pNO3jT/n+zireiam4dUsq6dDp9Lp1ON1T8cbU3V855B2NsjjH2knPucM6H"
+ "Nzc3T5UVoNrHj2w228E59zjnbzjnrzjnrxljb7PZ7ISU8r2SAap5QMtkMu8zxuY4528YYysRp5zz"
+ "lpJBKj3CSikP5XK5s7lc7qyUcsualU6nz6k0vYoCKdVul6NQ2Yf8IAguCSHmhRBM+XwQBJeKgBpU"
+ "zbzeAai9JFC51yDf95uEEOkwDKUQIieEyKnntO/7TUq9GGPsF8bY2wjMKuf8742NjcaSQJpW3kVR"
+ "CDGaz+elEOKfYldto4V+m5ubpzjnE4wxyhjzOOceY2yRMVb6ChRVaqertJQyFobhcyFEGAUSQoRh"
+ "GD4vXiJWV1ePbGxsXOWc3+act5etTCUWhuGMECK/DVA+DMOZqg9Yynzfb1PpyRXB5PL5vPR9v23f"
+ "gaSUtb7vDwohgjAMRRiGQggR+L4/KKWsLR1hjywIgou+7z/zff9ZEAQXdxvvX9K4yvU3n74tAAAA"
+ "GmZjVEwAAAAbAAAAJAAAACQAAAAAAAAAAAAfA+gBAYcCNCgAAAR5ZmRBVAAAABxYhc2YX2gURxzH"
+ "9zS2tZBqQwlaWxSCD7YEbBHaok/apnk0D74oTYsQ7skXKVik5fL3AtqH0gpSFEMi4WB3f3PXF3PV"
+ "tOeLS2Zn4ajVgnc3FxIfQi47Aybc3uzT9KFzsK4Jt2fOND/4wcLO/uaz3+/c/GZP0zYZpmkeNE3z"
+ "G5UHN1vvpSORSOwAgF7TNIcBYAQARtR1byKR2LHlQOl0+igAJA3DGAKAQQAYVNfJdDp9dMuBAKAf"
+ "AEbrMIEcBYD+lk84PT39diaT+QQAThqG8WEul2sLAcWVVWGgEQCIB8c6jrMLY3zMcZzTGONTjuO8"
+ "0xQMQugIQug7AEgCQNI0zTGE0PlMJtMeAOpT98NASQDoq4/L5/N7CSGXbNueJIRMEUKmbNv+2XGc"
+ "jyMro2DCbz9uGMYZTdNimqZpuq7vA4DLpmmOGYYxZBjGkGmaYwBwWdf1fZqmaVLK2NzcXJwQctu2"
+ "7RuBnFBQjZVSNq335kNqsj31salU6hBCaAAh9L3KgVQqdah+37KsDozxNdu2b4aAbhBCpjDGpxoC"
+ "AcDJ9YCUAgkA6AyOl1LGstlsRzab7ZBSxoL3LMs6QAj5FWP8AhDGeMpxnNNRFPpASf8cEEJoGCF0"
+ "cWJi4o2GRVQ4jvMmIeQKxvhWGMi27UmM8bEoRXYZhnEeAMaVTYMIoWGl2qdRYepBCPlC2XNLKXWT"
+ "EHIbY3zp0aNHr0Uqkslk2hFCZ9SiTSCELiKEPgtbEiXUwu4hhFxR9l0jhMTz+fzeZmtpuq7vAYDO"
+ "ZmzaKCzL2p3P5w9YltWx2VqvLDZUXEoZMwzjI9UO4gDQV99LXkUUCoWuYrH4balU+oVSOl4oFHqC"
+ "DTkGAL1q0Y6qDTGp1k/LjxOU0m5KKaKU/lkul+9SSmfVdVxKGdPS6fT7pmkOB7s2AAyqdjGg6/rO"
+ "VsFIKdsopT9RSv+glN4J5Ayl9Pf5+fkjGkLo6/WapAL8Qdf1li3ChYWFdymlvymAINCdcrl8t1Qq"
+ "Xd1+QNvOMm27LWpFvq1+9v9bNN2KWtk6VlZW2iuVyuFKpbK/6Ydb3Vw55/2MsXuMsTxjbM513R+X"
+ "lpY6Gz+ttf74wTn/ijFW5Jw/5pw/5Jz/zRgrMcYmFxcXdzcs0MoDWqFQeIsxdo9z/pgx9lcwOedP"
+ "XNftaVjkJY6wO2u1WletVuuSUj63Z1UqlcOMsTzn/GEYiDFWdF33QhSFIh/yPc87IYSYFUIsqZz1"
+ "PO9EAGg/Y2xO2fQCEOf8XEOgqJ9Bvu93CyGeCiGqQgiusiqEeOr7frdSL+a67lXOeSlk1z+u6z5w"
+ "Xfe9hkCaFu1D0fO86+K/YKEUnuddr49bW1vrZIxNcs6fMMaKSpkHrut+HgkmqNRGn9JSypgQ4r4Q"
+ "YnUdoFUhxP3gFiGlfH15eflL13UvcM7PRVammfB9P60sCgNVfd9Pt3zCCEBnlWU8AMOFEML3/bNb"
+ "DpTL5dp83x8XQjxTNq0KIZ75vj8e/qdkS8PzvOPVanWmWq3OeJ53fLP1/gW7MK+p+IodfAAAABpm"
+ "Y1RMAAAAHQAAACQAAAAkAAAAAAAAAAAAHwPoAQFqyEZSAAAE8WZkQVQAAAAeWIXNl11oFFcUx2f9"
+ "aCutpKZoFakf9aG0aal9qj5Z9UXwSahK+2CtIPpkxb4UadlkY1YrhaoIfrz4FGJn5txdkcDCCjt3"
+ "Jg1VtqDWzF0FsaQhuXcX0sC9s8vWDKcvd2Xcbrq7TUzzhwMD9+y5v/mf+7FjGLOUbdvrbds+qGP9"
+ "bOv9Z8Xj8UUAsMu27QQA9AJAr37eFY/HF807UCqV2gwAScuyegCgGwC69XMylUptnncgADgAAKdq"
+ "MJE4BQAH5nzC/v7+Fel0+mMA2GFZVlcul1tSB3REt6oeqBcAjkRz8/n80qGhoc2U0t2u627LZrNv"
+ "tAVDCHmXEPINACQBIGnbdh8h5FA6nV4eAdqjx+uBkgCwp5aXzWY7KKXHHce55DjOZcdxLnued8bz"
+ "vA9bdkbD1L/9acuy9hqGETMMwzBNczUAnLRtu8+yrB7Lsnps2+4DgJOmaa42DMNAxBil9CCl9Aql"
+ "9HwtXNe9SCk905JTuk2N3rxHT9ZRyx0YGNhACDlMCPlWx+GBgYENtXHP81a4rnuWUnohCkQpPe84"
+ "zmXXdbc1BQKAHY2AtANxAFgVzUfEWCaT6cxkMp2IGIuOUUrXUErPOY7zDyDXdS9RSne34tB72vrn"
+ "gAghCULIiWvXrr3StIjW8PDwMkppj+M4Fxs4dGloaKj58ZDP55dalnUIAE7rNnUTQhLatS2twtTk"
+ "ed4nejFf1E5doJRecRznqwcPHrzUUpF0Or2cELJXL9o4IeQEIWRrfUtaESLGHMfZTintcV33R9d1"
+ "z+ZyuS+z2WxH81/XyTTNDgBY1U6bZpJu3xrP81bMttYL04yOI2LMsqyP9HVwBAD21M6SF6FCobBx"
+ "ZGTkmO/7PzDGuguFwo4oXAwAdulFe0ofiEm9fub878TIyEiX7/v9jLFB3/dvMMZuMsYGGWOHEDFm"
+ "pFKpt2zbTkRvbQDo1tfFYdM0F88VDCIuYYx9ryEgEsT3/TRj7B2DEPJFo0tSA35nmmbnXAH5vr+G"
+ "MXadMUbqgMD3/RuFQqF34QEtuJYZC21Ra/IFte3/N7UNgYjrqtXq+4j4+mwnF0K8JoTY9OTJk/ad"
+ "R8S1YRj+hIhTiPgXIv6OiF8jYtufN4gY45x/zjm/KYQYFkLkOOfJ8fHxla0WeDUMw5+xsU60C8Q5"
+ "/0wIcY9znhdC3BZC3BFC/CaEuDo6OrqsaYHp6el9M8AgIo5NTU21fFuXSqXlQoibGuaXaHDO73LO"
+ "dzYtEoZh378AhYj4QTQfERdXKpWNlUplIyI+d2YJITbpNt2uB9KuHW3FoeMz0oShQsR1tdxyubxV"
+ "SjmolHqslHospRwsl8tba+Oc8zf1mrnTAOh+sVjc3xQIEd9GxD9nADJrW7VarXYppR4GQVBSSo0p"
+ "pcb088Nqtdqla8UmJib69JqJtutXIcSt0dHRtU2BDMMwnj59+ikilup4PER8VkApdU4pNamU+qMu"
+ "JpVS52p54+PjK4UQVznnd4UQ94rF4j0hxK1SqbS9JZiaKpXKpunp6WNhGCYQcS8iPtsRiBiTUmaC"
+ "IOD1QEEQcCllBiOH3qNHj17mnO/knB8tFov7W3amHSmlriulSg0cKimlrs/5hM1ULpf36faMRWDG"
+ "lFKT5XJ537wDIeJiKWVCSjkRBAHXrZqQUibqt/+8KgiCLVLKlJQyFQRB2x+S9fobk83y/MO/0ckA"
+ "AAAaZmNUTAAAAB8AAAAkAAAAJAAAAAAAAAAAAB8D6AEBh16VuwAABRtmZEFUAAAAIFiFzZddaBRX"
+ "FMdn/WiV+lURGwQxFfpQrWBftMV+iA+tryHYh0JtUUQL0oJPVVJ2s8ZVCWrBJx+KFQwJM3Pu7MM6"
+ "d5ZQV/FBAyozQ3yJH2F37hrFkL2X1mSdu9nTl7tls2zdXRNt/nDgsvfsmd+cc+aeGU2bpUzT3GCa"
+ "5g/KNsw23isrGo0uAIDdpmnGAeA4ABxX693RaHTBGweyLGsrACQMw+gGgBgAxNQ6YVnW1jcOBAB7"
+ "AaCnAlNlPQCwd84v2NfX924ymdwOALsMw9icyWQW1QAdVKWqBToOAAerfS9cuLB4cHBwSzqd/iqd"
+ "Tu9wHGd1SzCEkA8JIb8AQAIAEqZpniCE7Esmk8urgDrUfi1QAgA6Kn62ba+wbftHx3HOUErPOo5z"
+ "llIaS6fTHzWdGQVTe/cnDcPYo2laRNM0Tdf1NgA4ZprmCcMwug3D6DZN8wQAHNN1vU3TNA0RI47j"
+ "fGvb9m+U0lNV1kspjTWVKVWmenferS62suLb39/fTgg5QAjpUnagv7+/vbJvWdYqx3HijuOcrgE6"
+ "RSk9d+XKlR0NgQBgVz0glYEoAKyt9ldZWO04zmpEjFTvpVKpNsdxErZt1wM6Syn9upkMbVKpnwFE"
+ "CIkTQo5cvHhxScMgSrquL6WUHlUlmgHkOM6ZwcHBLQ2D3L59e7FhGPsA4KQqU4wQEldZ+6RZmIps"
+ "2/5MZaNXgZymlJ6jlB7Sdf2tpoIkk8nlhJA9qmmjhJAjhJBPa0vSjBAxQin9XGXqpG3bcdXoK1qN"
+ "pem6vhIA1rZSpv9SJpNZkkql2izLWjXbWK9T9TOOiBHDMD5W4+AgAHRUzpLXoXv37m1wXfeQ7/s9"
+ "ruse9X3/y+p2iADAbtW0PepATKj+mfPXiTt37nzoed7vnueZnucN+L6v+75vuK77HSJGNMuy1pum"
+ "Ga+e2gAQU+PigK7rC+cKJpPJLPI8L+77vu553uWK+b7f53le/927dz/QCCHf1xuSCvBXXddbG4Qv"
+ "0dDQUJvrun8ogMs1NuB5Xtf8A5p3JdPmW1Nr2vx77P9PtQaBiOsRcTMirmzs/XINDw8vy2azGx8+"
+ "fPhey39+/vz5ujAM+6SUz6SUf0kpH4Rh+POrfN4gYoQx9g1jzGCM/ckYs4MgiOXz+TVNBRgbG3sn"
+ "DMOriIhSyqKUckpKGZbLZQzD8KdWgRhjexhjt4IguMEYuxYEwXXG2FAQBOdHR0cbD+1isdg5PT2N"
+ "UsrJGpNSygeI2PS0fvr06TLGmKFgMjV2MwiCnQ2DSClj5XK5HtCUlPLvFy9ezPhaQMSFhUKhvVAo"
+ "tCPijDMrm81uVGW6Vg8on8/vbwgUhuFhVa5aoKKU8tnk5OT6iq8QYjvnHIQQw0KIYbXeXtkfGxtb"
+ "q3rmeh2gW0EQdDYEmpqael9K+URKWaoGUpCXK+fF+Pj4JiGEyznPCSHuCyHuq7U7Pj6+SWUvksvl"
+ "ooyxoRqYG4yxVDabXdcQSNM0rVgsdkgpnyAilstlnJ6exlKpdBUR/w3AOe8VQjDO+Ui1qd96K375"
+ "fH4NY+w8Y+wmY+yWstTjx4+/aAqmOlNhGB4ulUpRKWVnLpdbWtlDxIgQIimEGK0DNCqESFafvCMj"
+ "I28HQbAzn8/vD4Kgs+nMtCLO+SUhRLYOUJZzfmnOL9hIhUKhk3POVO9UYO5zzlmhUGjcrHMtRFzI"
+ "Oe/inD9SZRrlnD/inHfVPv5vVJzzbRMTEwMTExMDnPNts433D6ldzINe+r/lAAAAGmZjVEwAAAAh"
+ "AAAAJAAAACQAAAAAAAAAAAAfA+gBAWm6LjMAAAS9ZmRBVAAAACJYhc2XT2gUVxzHZ/1TpK2IikHB"
+ "NoFC8c/FQiCKeMnJUzWFQE9aAmJpb4GijZU10cSLx0IPEde2xOC89/vtHEqzFuoKXpzZPVhZgph5"
+ "M9CG9TB5z1iymZ9eXg++acdp7O6axeYLP3jsvPm9z/v9fu/3Zi1rleKcd3POPzPWvVp/r618Pr8O"
+ "AI5yzscA4CIAXDTjo/l8ft0bByoWiwcAYIIxNgoAFwDgghlPFIvFA28cCABOAMClBCZllwDgRMcX"
+ "nJqa2uo4Th8A9DPG9pfL5Q0ZoNMmVVmgiwBwOj23XC5vYIztB4B+x3H6pqamtrYFg4h7EfEsAEwA"
+ "wATnfBwRhxzH2ZwCGjDPs0ATADCQzHMcZzMiDnHOxxN/iHgWEfe2HBkDk939ZcbYoGVZOcuyLNu2"
+ "dwLACOd8nDE2yhgbNYuO2La907jLMcYGAeByNoqIeLalSJk0rbTzUbPYlmTu9PR0DyKeQsRvjJ2a"
+ "np7uSZ7btr0FAEbMu/+KpOM4fU2BAKB/JSATgTwAdKXna61zpVJpW6lU2qa1zmV8dXHO8+mTmElt"
+ "fysR2mdC/5IDRBxDxOFCobCpqROjQqGwCRGHEXEs649zPu44zr6mTqrV6kbG2JDJ+2gCY3Z0sFWY"
+ "RABw0BRyAjVq6nGoWq1ubMmJORmDpmjzZpeHsilpRVrrHCIeQsRhk/IRRBxMn9iWZYqyq500vUqF"
+ "QmETAHSlD8Va1MoR11rnGGMfmevgNAAMpHpJx3X37t33Pc8bcl33fKVSGa5UKofT5ZADgKOmaC+Z"
+ "hjhh6qfjnxOu637ouu63lUrlB8/zCp7nXTfjT7XWOatYLL7HOR/L9gpzXZyybXt9p2DK5fIG13XP"
+ "ua77ved5kym76rrutWq1+oGFiCdXuiQN4Hnbtrd1Cuj+/ftdnud953ne1QzQpInWV2sPaM2lzFpr"
+ "RW1Za+/Y/59qD6LRaOwmon1KqVW3+8ePH78ThmFPvV7f0fbLS0tLu5aXlwtE9AcRRXEc1+I4/vJ1"
+ "L1ff9z/xff9HIcRPQgicm5sbmZ+f396qg7eJaCaO4+dE9ISIFBE9JSKK4/iLdoF83x8QQtwWQtwS"
+ "QswYu+P7/pUwDJtf2o1G4zgRLRORzNjTOI5rWuuW01er1d41kbklhPg5Y78KIY40dUJE5+iFskCK"
+ "iCIieukrz7bt9fV6vbter3dne1YYhj0mTTMrAN0OgqD5/7c4jj9/BdATIvq90WjsTuYuLi72KqVu"
+ "SCmrUsqqUurG4uJib/L80aNHO4QQ6Pt+aQWgchAEx1oB6iGikIj+zAA9J6JrSWFHUbRHSnlPKfVQ"
+ "KfXA2EMp5b0oivZY1ouCFkJ8LYS4k4YJguCXIAhYGIat9bpGo/Hxs2fPQvpHy0Q0s7S0tCuZE0XR"
+ "uJRyTkr5W8bmoigaT+bNz89vD4LgiqmZ277vl4MgYL7vH24JJh0pk75zRHRca/33idBa55RSN5VS"
+ "s1kgpdSsUupmukXUarW3hBBHfN8/GQTBsZYj044WFhYmTYqyQA8XFhYmO75gMymljkkp55RSD1Iw"
+ "D8xvzYu109Jar5dSnlFK1aSUs1LKWTM+o7Xu2CdL24qiqDeKouvGepu/8d/6C9KZrM8coJ2aAAAA"
+ "GmZjVEwAAAAjAAAAJAAAACQAAAAAAAAAAAAfA+gBAYQs/doAAATkZmRBVAAAACRYhc2XXWgUVxTH"
+ "Z41aabFisKlg/Si04seL+hL1qeRJUAsW8iJFSkR9KH2JKBIKa9REKPRBG6ziw4KyBGbm3J1C04a+"
+ "7M6UQiUrSWRz7ybgg9Qwdy67kSxnpGKW04felXVM2FmTSP5w4MK9c+7v/s+5s7OGsUjZtr3dtu1v"
+ "dGxfbL63VjKZXAUAR2zbvgIAVwHgqh4fSSaTq945UCaT2QcA/ZZl9QLAZQC4rMf9mUxm3zsHAoBT"
+ "AHCtBlMX1wDg1JJvmE6nNzqO0w4AHZZl7c1ms6sjQOd0qaJAVwHgXP3abDa72rKsvQDQ4ThOezqd"
+ "3tgUDGNsN2PsEgD0A0C/bdt9jLEux3HW1wGd0PNRoH4AOFFb5zjOesZYl23bfbV8jLFLjLHdsZ3R"
+ "MNHTX7csq9MwjIRhGIZpmpsBoMe27T7Lsnoty+rVm/aYprlZp0tYltUJANejLjLGLsVySpdpvpP3"
+ "6s021NYODg7uYIydYYx9r+PM4ODgjtq8aZobAKBHP/uGk47jtDcEAoCO+YC0A0kAaKtfT0SJ4eHh"
+ "1uHh4VYiSkRytdm2nay/iZHSdsRxaI+2/rUEjLErjLHuVCq1rmESrVQqtY4x1s0YuxLNZ9t2n+M4"
+ "exomyefzayzL6tJ1763B6BMdjAtTEwAc1I1cg+rV/diVz+fXxEqib0anbtqkPuWhaEniiIgSjLFD"
+ "jLFuXfIexlhn/Y2NLd2Ubc2UaSGlUql1ANBWfyli6W1cWITm34uIVs3NzXVVq9U/qtVqnohSL168"
+ "OLBcFNls9hPP8752XfeC67rfep7X/soIIkpUq9Wb9KaevXz58otlgPnM87wfcrncbc/zBlzXvZXL"
+ "5W7ncrmvDMNIGER0eB4YIiKqVqsjRLR2qWCIqCWXy513XfeW67o36uKm53k/ua77qTE3N/fXQkBE"
+ "9C8Rfb5UQJ7nfeS67o+u696MAN3wPG/A87zvVh7QiivZimtqTb5yrn1UC04sj5rbKwzDLZVKZVep"
+ "VPpwsTtPT0+///jx423FYnFT0w8j4mZEvI2Ik4j4BBFHwjA8+zbuEVGiUCh8OTExcbdYLFqc87QQ"
+ "4nyhUGiNfRJEzIRh+AwRpxHxaRiGPiLOVCqVs80CCSGOCyGGhBCOEILp+I1z3jc1NfVewwSIeCwM"
+ "wxIi/lMfGmpkZmYm9q/12NjYBxMTE3c1DETiVyHE4ThAFxFxJgqEiE8R8UmlUtlVv940zZZyuby1"
+ "XC5vNU2zpX5O94ylXXkNiHM+JIQ4GQfo9AJA04g4WSqVttTWKqUOSClTUso/daSUUq9eF8VicRPn"
+ "PM05z8zj0BDn/GhDoOfPn29DRB6GoYyU7Bki/lxrbKXUziAIskqp0SAIHgRB8ECPs0qpnYbxf0ML"
+ "Ic5zzn+PuPML5/z+o0ePPm4IZBiGMTs7exQROSLOhGFY1j2VQcTa/y1DSplUSo0HQfB3fSilxqWU"
+ "ydq6QqHQyjnv0z0zpJ25PzU11dz3uXbqNCJenJ2dPU5Erz5jiSghpbwnpXwYBZJSPpRS3qt/RRQK"
+ "hbVCiMNCiJOTk5NHYzvTjKSUA1LK0XmARqWUA0u+YSP5vn8sCIJx3T81oAdBEIz7vn/snQMRUYuU"
+ "sjsIgpEgCPI6RqSU3UTU0jjDMkkptd/3/Tu+799RSu1fbL7/AP897Rb5E8XQAAAAGmZjVEwAAAAl"
+ "AAAAJAAAACQAAAAAAAAAAAAfA+gBAWnmj6AAAAULZmRBVAAAACZYhc2XT2gUVxzHZ+OfWsGKClFE"
+ "0YIg2oMeWqKXHnIxCj14SI9aQq2HkoOeNBTXRBNrbYR6kiAslC4rM+/3MgedmURwg8FWZBtm3kqQ"
+ "KAb2vd3LGvIegYTNW/J66BsZx9idaBryhR88mN/+5vO+v9+bmTWMjxRCaA9C6Dsdez623gcrnU43"
+ "AUAbQqgHAK4AwBW9bkun000rDjQ4OHgYAPosy+oGgMsAcFmv+wYHBw+vOBAAnAKAqyFMJK4CwKll"
+ "v2E2m91i23YLALRalvVFPp9fGwM6q1sVB7oCAGejufl8fq1lWV8AQKtt2y3ZbHbLkmAwxgcwxhcA"
+ "oA8A+hBCvRjjDtu2N0WATurrcaA+ADgZ5tm2vQlj3IEQ6g3rYYwvYIwPJHZGw8R3f82yrHbDMFKG"
+ "YRimae4AgC6EUK9lWd2WZXXrm3aZprlDl0tZltUOANfiLmKMLyRySrdpsZ1365ttDnNzudxejPEZ"
+ "jPFPOs7kcrm94XXTNDcDQJf+7TtO2rbd0hAIAFoXA9IOpAGgOZqvlEp5nrfV87ytSqlUrFYzQigd"
+ "PYmx1rYmceigtv6tAhjjHozx+Uwms6FhEa1MJrMBY3weY9wTr4cQ6rVt+2DDIoVCYZ1lWR26790h"
+ "jN7RkaQwoQDgiB7kEKpbz2NHoVBYl6iIPhntemjTepdH4y1JIqVUCmN8FGN8Xre8C2PcHj2xiaWH"
+ "snkpbXqfMpnMBgBojh6KRPoQF5ZdSqmmWq12ul6vO1LKv6SUd2q12v/2PhoeHt7puu63nud1uq77"
+ "/f379798Y4RSKiWl/HVhYUFJKReklHW9rs7Ozn693DBDQ0OfO47T43neTdd1bziO0++67k3Hcb4x"
+ "DCNlzM/PH5FSzksp56SUs2HU63VVr9dHlVLrlwvGNM01ruv+6Hlev+u6P4fhed51z/N+uXfv3h6j"
+ "Vqs9rNfrMgqjY05Kyefm5vYtF9CDBw+2ua571XGc61EgHTeGhoZ+MGq12kP5r1YH0Kpr2aobasNY"
+ "Zcc+rlXxYHyfqtXqzmq1ul8ptfT3TkyFQmFjsVjcPT4+vm3JP56ZmdkuhLglhPA5588556NCiI4P"
+ "fbmOjY2dCILgFiHkd0LIHUJIJyEk2Xe1UurT6enpu5zzihDipRDihRDilRCCTU9PdywVKAiC40EQ"
+ "oCAIcr7vZwkh2SAIgBByaWJi4pOGBYQQxznnlHM+EQ0hxCvO+ejr168/SwpTLpc3amdyQRD8EQ3f"
+ "9y3f9xt/wgohzgkh2CJALzjnz6vV6v5ovlKqiVK6i1K6Syn11j/Wp0+f7tZtysaBCCEWIaQ9CdDp"
+ "9wC9FEL41Wp1Z5jLGDvMGLtNKR2mlA4zxm4zxt48LsbHx7fpmXkHKAgCVCwWjzUEmpqa2i2E+FsI"
+ "MRmDqnDOfwsHu1Qq7WOMOZTSPxljI4yxEb12SqXSPu1eihDSGQQBxGDuEkIGfN9v/m8aLc75Mc75"
+ "mHaKcc6pEOLuzMzM9jCHUnqRMfaEMZaPxRNK6cUwjxCyhRByiRBi6uFGhJCBYrH4VSKYmFOnhRDn"
+ "pqamTkxOTr75jFVKpRhjA4yx0UWARhljA9FHxLNnz9b7vt9CCGkvFovHEjuzFFFK+8vl8uM4ULlc"
+ "fkwp7V/2GzZSpVJp0y0biQCNMMaeVCqVthUHMk1zTalU6mSMPdJtGmWMPSqVSp2maa5ZcaBQlNJD"
+ "lNJbOg59bL1/AITf0EjShwB3AAAAGmZjVEwAAAAnAAAAJAAAACQAAAAAAAAAAAAfA+gBAYRwXEkA"
+ "AATIZmRBVAAAAChYhc2XT2gVRxzH90VttWDVUIJ/WpSKBxWLLVKV3mK1HvWQkzQtAYkXL3pQQssz"
+ "iUlAe2kVSqn4SCQEdub3e+ulxn99vfh4s7PwqMZCX3YexB6Cz52B9vF8m9P00FlZV0M2MQ35wg+G"
+ "2dnffOb3+81vWct6Q1FKt1JKvza29U39LVjZbLYFAI5SSvsAoB8A+s34aDabbVlyoHw+vxcABgkh"
+ "vQBwAQAumPFgPp/fu+RAANAJABcjmJhdBIDORd9wdHR0g+M4+wGgnRCyu1AorEwAdZtUJYH6AaA7"
+ "vrZQKKwkhOwGgHbHcfaPjo5umBcMIu5ExPMAMAgAg5TSAUTschxnbQzouHmeBBoEgOPROsdx1iJi"
+ "F6V0IPKHiOcRcWfqyBiY5OmHCCEdlmVlLMuybNveCAA9lNIBQkgvIaTXbNpj2/ZG4y5DCOkAgKFk"
+ "FBHxfKpImTS97uS9ZrN10dqxsbFtiHgSEb8xdnJsbGxb9Ny27XUA0GPefSWSjuPsnxMIANpfB2Qi"
+ "kAWAtvh6rXVmfHy8dXx8vFVrnUn4aqOUZuM3MZHa9jQR2mVC/5IDROxDxDO5XG71nE6McrncakQ8"
+ "g4h9SX+U0gHHcXbN6cTzvFWEkC6T994IxpzoQFqYSABwwBRyBNVr6rHL87xVqZyYm9FhijZrTnkw"
+ "mZI00lpnEPEgIp4xKe9BxI74jU0tU5Rt80nTbMrlcqsBoC1+KVJpIVFYdGmtW2ZmZk7MzMzkwzD8"
+ "LQzDH+v1+kf/136mhx033b6TEPLxi0BorTPNZnMo/E+NZrNZN+O/nj9//tliw1BKt5reNGia70Uz"
+ "PmpZVsZqNBqfhmH4dxiGKgxDGbNGGIb3tNZvLRaMbdsrEPFksq2YHteXz+c/sBqNxq0wDP9JwEgD"
+ "ON1sNj9cRKBWAPh2lkbZj4hfLUug5ZWyZVfUlrXMrn1Sy6IxzqZarbapVqvtePbs2fy/OwkVi8U1"
+ "5XJ5S7FYbJ33y9PT021BEHwnpSxJKctSyrtKqc6FflxLpdIRzvklzvlPjLGrnPPucrm8PpWDJ0+e"
+ "rJFSDkspfaXUI6XUQ6XUYynlpFLqy/kCcc4Pc85HGGPXGWPXXNe9xjm/wRg7NzExMfcNDoLgiFLq"
+ "Tynl73EzUHcrlcq7aWE8z3uHc36JMXbddd2fEzbMGNuXBui0lHLyNUAPpZTlWq22I75ea90yNTW1"
+ "eWpqarPW+qU/1mKxuMWk6VoSiDE24nnesTmBlFInZgF6JKUs1Wq1TdFaIcSeycnJ74UQN4UQN814"
+ "TwyolTF21XXdV4BMGg+lidD7QRA8UEr9kQDygyC4HBV2pVLZLoRAIcR9IcQtY/eFEFipVLab6GVK"
+ "pVI35/xGAijnuu4Pnue9NyeQgfpcKfXARGrS1NRwvV5/8cchhDjr+35BCPFL3Mzc2WhduVxezzk/"
+ "57ruMOd8hHM+YmA+SQUTj5RS6kQQBKefPn36hdb67eiZ1jpTrVavVKvVO0mgarV6p1qtXom3CM/z"
+ "VjHG9nmed4wxdih1ZOYjIcSQEOJeEsjMDS36himADgshfjW1E8HcMnOHlxzItu0Vvu+f8n3/tknT"
+ "Hd/3b/u+f8q27RVLDhRJCLHH9/3Lvu9fjl/5hepfZp2x939wIBcAAAAaZmNUTAAAACkAAAAkAAAA"
+ "JAAAAAAAAAAAAB8D6AEBaQNtFQAABQNmZEFUAAAAKliFzZddaBRXFMdnY22VVqwWrUHqR32qWmqh"
+ "YH2y0RfBJ6FW2gdrA6IvtmJfJLRsNl9+IFRF8OMlTyF2Zs7dkVQIrJC5M2mokoJWc2+6UCxRknt2"
+ "aZuYGYM1w+nLHVmnSXdXY5o/HBj2nr33x//ce+aOYTynbNtebdv2Ph2rn3e+Z1Y6na4BgB22bTcB"
+ "QDMANOvnHel0umbWgbLZ7CYAaLMsKwMAjQDQqJ/bstnsplkHAoC9ANASw5RECwDsnfEFJyYm1k1O"
+ "Tn4ZRVETEe0mooUJoAO6VEmgZgA4UJrb09PzkmVZGwBgm+M4mzs6OpZUBfP48eOPiahIT8snopUl"
+ "QLsAoG0KoDYA2BXnOY6ziDFWb9t2qx5rY4wdZYy9UxEMEb1NRH/RFIqiyCSilGEYhmmaKwCgwbbt"
+ "VsuyMpZlZfSiDaZprtDTpSzL2g0Ax5IuMsaOVuTU5OTk4algNFBARKvi3M7OzjWMsf2MsW907O/s"
+ "7FwTj5umuRgAGgAgM5WTjuNsLgsURVHrdEBEFBHRuwlHU93d3Uu7u7uXxu7FAoDltm2nS09iorTb"
+ "KnHok/8Auj86Olrxhmxvb1/AGDvCGGtKAtm23eo4zvqykxDRq1EU/TgN0JFKYWIBwId6I8dQGQA4"
+ "ZllWfX9///yKJiGilVEUfU9Eo0T0NxH9TkRfE1HVHZiIUoyxLYyxI7ZtpwGggTG223GcRdXOZRDR"
+ "qkePHm0koter/nNC7e3tCwBguWmai6uFSJXPesEiopqHDx/uCYLg8vj4eHcQBKfHx8c3vqj1dA/b"
+ "pbv9Xsuy3n9iBBGlwjDMhGH4RxAExTAMVRAEfwZB8OvY2NiWmYaxbXu17k1t+jXTop93GIaRMsIw"
+ "/CAMw5EgCO4HQXAvjjAMi2EY/nDnzp2XZwrGNM15jLH9uqM/aQG6yzdls9m3jLGxsax25V4i7gdB"
+ "8NvExMTaGQRaCgDfTtMomxljn889oNHR0blVsjm3qQ3j38f+wYMHZ/63Y5/UnGiM0+nu3bsrEHEd"
+ "Ir72vHP19fUt5JzX+r5f3fXVMAxjeHh4mVKqDRF7ELFPKdWllPrsWdwjopTrunWc84zned95nney"
+ "p6fni1wuV9k7bWhoaCEiXkLE24h4AxGvK6X6EfGWUurTaoF83//Idd0Lruuec133LOf8LOf8ouu6"
+ "X1V0gpVS25VSNxHxp9LQUF3FYrHia4MuU8Z13XOc8zOl4bru+d7e3vLfb0qpg4h4KwmEiNcRsQ8R"
+ "15XmE1GNEKJWCFGbvC9xzms556e1M08BeZ53nnO+syxQoVDYg4i/TOHQDUTsUUq9GecODAxskFKe"
+ "kFJe1nFiYGBgQzzu+/4Sz/NO6jIlHbrged7WskBDQ0MrEfGaUurnBNTtkZGR1nhjDw4OrhVCdEgp"
+ "u6SUTEeXEKJjcHBwrXYvxTnfxzm/mHDnHOf8eC6Xe6MskGEYRrFYrEPEa4VC4ZbezDcR8dLw8PCy"
+ "OEdKeUhKeVVKCYm4KqU8FOflcrnFnPPDruue15v7gu/7x33ff68imFKnCoXCHqXUQaXU9nw+/0o8"
+ "RkQpIcQpIcSVJJAQ4ooQ4lRpi+jv75/f29u7iXO+0/O8rRU7U42klI26XEmHuqSUjTO+YAVAdbo8"
+ "rASG6d/qZh3INM15Usp6IYSjy3RFCOFIKetN05w360Cx8vn8eiFEixCiJZ/Pl/8SLaN/AMUO786z"
+ "sTxgAAAAGmZjVEwAAAArAAAAJAAAACQAAAAAAAAAAAAfA+gBAYSVvvwAAAUmZmRBVAAAACxYhc2X"
+ "T2gUVxzHZ9PaRlqtDWKDILGetCrYHmqLrYiH1mtI7aGgLYpoQVrwpCFl88+oBLXgyYPEgjFhZn5v"
+ "5hDnzRI0igcVPOx7u7mk2rCZt0ZR3Blak3XeZn+9vA3rsHY3JqT5wg8G5re/9+H7+73f7mraPGWa"
+ "ZpNpmj+paJpvvTdWPB6vA4A9pml2AkAXAHSp5z3xeLxu0YEsy9oGAD2GYXQAQDsAtKvnHsuyti06"
+ "EADsB4DuEkxZdAPA/gU/cHp6+uMwDI8WCoW4lLJlYmJieQTosGpVFKgLAA6X546MjLxtGMZmANht"
+ "2/b2/v7+D+cEk8/nm6WUjxERi8UizszMYKFQuIGIa8uAmgGgpwJQDwA0l/Js215BCDlgmuZJ9a6H"
+ "EHKcELKpZmeklI+llAUp5VQpEBGllFcQMaZpmqbreiMAtJqmedIwjA7DMDrUoa26rjeqcjHDMPYC"
+ "wKmoi4SQ4zU5FYbhUXX4VCTyUsqnU1NT60q5AwMD6wkhhwghbSoODQwMrC+913X9AwBoBYCOSk7a"
+ "tr29KpCUsr1YLFYCmpZS/vPy5cst5fmIGHNdt8F13YaSeyUBwBrTNOPlNzHS2t1VgfL5fMvMzEwl"
+ "ICmlfICIq6oWUerr66snhBwjhHRGgUzTPGnb9idVi0xOTr4XhuEN1ba8ciYsFosYhuEvtcKUBABf"
+ "qEEuQXUAwCnDMA7cv39/WU1FXrx4sTYMw34p5dNCofC3lPJBGIa/vskGRsQYIeRLQsgx0zTjANBK"
+ "CNlr2/aKudbSEHEdIm5GxA/m/OGI+vr66gFgja7rc6sVHcz/RYhYl8vlvvN9/w/f923f93ufPXtW"
+ "feDeUGqHNattv98wjE9njUDEmO/7bb7viyAIMkEQjAdBIIIgSPq+//lCw5im2aR2U4/6mulWz3s0"
+ "TYtpuVzusyAI/gqC4E/f98fKYsL3fUDE2m5CDdJ1/S1CyCG10WdXgNrynZZlrdOeP38+GATBeARm"
+ "TAGmc7nc+gUEagCA316zKLsIIT8uPaAl17IlN9SatsSufVRLYjG+Tg8fPvwok8lsSKfT78+31sjI"
+ "SP3Q0FCjZVk1/1qYVTabXe15XrsQwhFCXBdCGEKI79/EPUSMUUq/ppSeoJSechyn03XdHxzHWVlT"
+ "gfHx8XrP8y4IIe55nndLCHHT87zbQoi7Qoi9cwVyHOcrSuk5SmkvpfS067pnKKXnKaVHdF1/p2oB"
+ "z/N2CSHuCCFGykNBGU+ePKm5fbquL1fO9FJKT5eH67pnh4eHt1Ytks1mD1YCEkLcFEJcz2QyG8rz"
+ "EbEunU43ptPpRkR85ffS0NBQo+u6PY7jnIkCKde+rcWhFtWeqEO3hBDO5OTkmlJuKpXaxBjrTCaT"
+ "l5PJ5GXGWGcqlZr9e2NZ1io1M5WAzl+7dm1HVaBMJrNWCDEkhLgdgbo3MTERLw326OhoE2PsEudc"
+ "55z3q9AZY5dGR0eblHsxNcC/R2B6KaXtrus2VAXSNE179OjRTgV1V8UdIcSFbDa7upSTTCaPMMZM"
+ "xtiVSJjJZPJIKc9xnJWO4/zsuu5ZSuk513XPUUrbE4nElsqn/4dTnue1ZLPZg57n7RobG3u39A4R"
+ "Y5zzbsbYYAWgQc55d/mKuHjx4rLh4eGtiUTim0QisaNmZ+YizvkJ1aJXgFQLTyz4gdXEGNvJOTc4"
+ "5/1lMP2cc4MxtnPRgRCxLpVK7eOcX1WtG+ScX02lUvui139RxTnfyBhrY4y1cc43zrfev/K81H4R"
+ "Ae1VAAAAGmZjVEwAAAAtAAAAJAAAACQAAAAAAAAAAAAfA+gBAWlfzIYAAARnZmRBVAAAAC5Yhc2Y"
+ "TWgbRxTHV+4Hpm0IdYhxIK59Slr34kIhbUouOfnU2IVATk4xmJT2ZghNnRrFX8olx0IOcSvT4Iru"
+ "zhvtIdRKIXGO3Vkf1KCGYM/sQiNsjDRrUiTvcy7Ty2yqLjaSLdX1Hx4MO483P96bfTuzhtGkCCE9"
+ "hJDPtfU0G2/fSiaTbQAwQAiZAoBpAJjW44FkMtl24EDZbLYfAFKWZU0CwA0AuKHHqWw223/gQAAw"
+ "DAAzEUyNzQDAcMsXDMOwNwzDLxDxOiIOKqXaY0BXdKniQNMAcKXWd2lp6VXLst4HgPO2bZ9ZWFh4"
+ "e08w1Wr10+3tbR//0RYiLlYqlRM1QEMAkNoBKAUAQ5GfbdtHKKUjhJBZPZeilF6jlL7XcGYQ0UfE"
+ "vxBR1tgLRPxBKZUwDMMwTbMLAMYJIbOWZU1aljWpFx03TbNLh0tYlnURAG7Gs0gpvdZQpnSZMAYj"
+ "EXETEf+sVqsnI99MJtNLKR2llH6rbTSTyfRG86ZpHgWAcQCY3CmTtm2fqQuk98xOQAEilhCxr9Zf"
+ "KZXI5XIduVyuI8peJADoJIQka9/EWGnP1wWqVquDes/EgZ6HYVhQSh2tG0QrnU63U0rHKKVTcSBC"
+ "yKxt2311gyil3kDExTAMX+gyBYj4HBExDMMvG4WJBAAf6Y0cQU0CwE3LskaWl5dfayhIpVI5sbW1"
+ "lUbEZ7pMhTAMv4qXpBEppRKU0o8ppWOEkCQAjFNKL9q2fWSvsYxqtXoSEfuCIGi4TLspnU63A0Cn"
+ "aZp7i7WfLLRcSqm2zc3NwXK5fKdcLv8spZwtlUrv/lfr6R42pLv9sGVZH7xMhFIqIaX8Wkq5GgTB"
+ "0yAInkgpV6WUv5VKpQ9bDUMI6dG9KaU/MzN6PGAYRsLY2Njol1L+EQTBYynl75EFQfC0XC7/pJRq"
+ "7E1oQKZpvkIpHdUd/WUL0F1+KpvNdhulUmleSvmkFkYDPQ6CwA2C4J0WAnUAwMQujXKaUnr58AEd"
+ "upIduk1tGIfstY/rUDTG3bS2tnbc9/3e9fX1N5uNte9Ph2EYRrFYPLa6ujouhKBCiHuc87uc88/+"
+ "l4+r7/vtnPNbQohHQohFbfeFEA8550P1I/xbTR8/hBDnhBAPhBC/xOw+5/xuoVB4q1GYlhzQPM8b"
+ "FkI83AFoUQhxz/f93lr/ZDLZls/nO/P5fGf8xtqSI6zneReEEEtxIM55TghBV1ZWjke+jLHTjuNc"
+ "Z4zdZozd1uPT0XxLDvm+73d5nmd5nvdrDOqREOKbaGMzxrodx/mOMTbPGJvTNq+fdetwzV+DDMMw"
+ "OOefeJ5ncc6XdPkeeJ53q1gsHqvJzojruj8yxu7Umn42Evk1fVGMZeoC5/yyEOJcoVB4PZpTSiUc"
+ "x5lgjKXjQIyxtOM4E7UtoumrdCNyXXdMlysONO+67ljLF6wnxthZXZ65Gpg5/ezsgQMppdoYY5dc"
+ "1/1ely6tx5eUUgf/wyqS4zinGGNXGWNXHcc51Wy8vwEPgLNswgUPqwAAAABJRU5ErkJggg=="
      + ");" + "\n"
      + "}" + "\n"
      + "  </style>" + "\n"
      + "  </head>" + "\n"
      + "  <body>" + "\n"
      + "    <div id=\"unmht-loading-container\"><div id=\"unmht-loading\"></div></div>" + "\n"
      + "  </body>" + "\n"
      + "</html>" + "\n";
    
    return this._createStringInputStreamChannel(uri, "utf-8", "text/html", 
                                                data, data.length);
  },
  
  /**
   * 文字列を返すチャネルを作成する
   *
   * @param   {nsIURI} uri
   *          対象の URI
   * @param   {string} charset
   *          文字セット
   * @param   {string} type
   *          MIME-Type
   * @param   {string} data
   *          文字列
   * @param   {number} length
   *          文字列の長さ
   * @returns {nsIChannel}
   *          作成したチャネル
   */
  _createStringInputStreamChannel: function(uri, charset, type,
                                            data, length) {
    let sstream
      = Cc["@mozilla.org/io/string-input-stream;1"]
      .createInstance(Ci.nsIStringInputStream);
    sstream.setData(data, length);
    
    return this._createInputStreamChannel(uri, charset, type,
                                          sstream, length);
  },
  
  /**
   * ストリームの内容を返すチャネルを作成する
   *
   * @param   {nsIURI} uri
   *          対象の URI
   * @param   {string} charset
   *          文字セット
   * @param   {string} type
   *          MIME-Type
   * @param   {nsIInputStream} stream
   *          ストリーム
   * @param   {number} length
   *          ストリームの長さ
   * @returns {nsIChannel}
   *          作成したチャネル
   */
  _createInputStreamChannel: function(uri, charset, type, stream, length) {
    let inputStreamChannel
      = Cc["@mozilla.org/network/input-stream-channel;1"]
      .createInstance(Ci.nsIInputStreamChannel)
      .QueryInterface(Ci.nsIChannel);
    
    inputStreamChannel.setURI(uri);
    inputStreamChannel.contentCharset = charset;
    inputStreamChannel.contentType    = type;
    inputStreamChannel.contentStream  = stream;
    inputStreamChannel.contentLength  = length;
    
    return inputStreamChannel;
  }
};
