"use strict";

let { classes: Cc, interfaces: Ci, results: Cr, utils: Cu } = Components;
Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative(this, "base/Promise.jsm");
XPCOMUtils.importRelative(this, "base/Task.jsm");
XPCOMUtils.importRelative(this, "base/arAppInfo.jsm");
XPCOMUtils.importRelative(this, "base/arArrayUtil.jsm");
XPCOMUtils.importRelative(this, "base/arFileUtil.jsm");
XPCOMUtils.importRelative(this, "base/arMIMEDecoder.jsm");
XPCOMUtils.importRelative(this, "base/arMIMEEncoder.jsm");
XPCOMUtils.importRelative(this, "base/arMIMEParam.jsm");
XPCOMUtils.importRelative(this, "base/arMIMEPart.jsm");
XPCOMUtils.importRelative(this, "base/arMIMEUtil.jsm");
XPCOMUtils.importRelative(this, "base/arNetFileLoader.jsm");
XPCOMUtils.importRelative(this, "base/arPathUtil.jsm");
XPCOMUtils.importRelative(this, "base/arPseudoID.jsm");
XPCOMUtils.importRelative(this, "base/arTimer.jsm");
XPCOMUtils.importRelative(this, "base/arUconv.jsm");
XPCOMUtils.importRelative(this, "base/arWindowUtil.jsm");
XPCOMUtils.importRelative(this, "UnMHTCSSUtil.jsm");
XPCOMUtils.importRelative(this, "UnMHTDOMConverter.jsm");
XPCOMUtils.importRelative(this, "UnMHTDOMUtil.jsm");
XPCOMUtils.importRelative(this, "UnMHTDocumentCategory.jsm");
XPCOMUtils.importRelative(this, "UnMHTFilename.jsm");
XPCOMUtils.importRelative(this, "UnMHTFormatter.jsm");
XPCOMUtils.importRelative(this, "UnMHTHTMLUtil.jsm");
XPCOMUtils.importRelative(this, "UnMHTMailPrivacy.jsm");
XPCOMUtils.importRelative(this, "UnMHTPref.jsm");
XPCOMUtils.importRelative(this, "UnMHTScheme.jsm");
XPCOMUtils.importRelative(this, "UnMHTStringBundle.jsm");
XPCOMUtils.importRelative(this, "UnMHTTreeScanner.jsm");
XPCOMUtils.importRelative(this, "UnMHTVersion.jsm");
XPCOMUtils.defineLazyGetter(this, "UnMHTCustomFilename", function() {
  XPCOMUtils.importRelative(this, "UnMHTCustomFilename.jsm");
  return UnMHTCustomFilename;
});
XPCOMUtils.defineLazyGetter(this, "UnMHTExtractor", function() {
  XPCOMUtils.importRelative(this, "UnMHTExtractor.jsm");
  return UnMHTExtractor;
});

let EXPORTED_SYMBOLS = ["UnMHTSave"];

/**
 * MHT ファイルの Content-Location を取得する
 *
 * @param   {string} loc
 *          アドレス
 * @returns {string}
 *          アドレスが MHT ファイル中のパートを指していれば
 *          Content-Location の値
 *          それ以外ならそのまま
 */
let _getMHTContentLocation = function(loc) {
  if (UnMHTScheme.isUnMHTURI(loc)) {
    /* MHT ファイルを再度保存する際には相対パスの基準を
     * 本来の Content-Location にする */
    let [eFileInfo, part]
      = UnMHTExtractor.getFileInfoAndPart(loc);
    if (eFileInfo && part && part.contentLocation) {
      return part.contentLocation;
    }
  }
  
  return loc;
};

/**
 * 保存するファイルの情報
 *
 * @param   {string} loc
 *          保存するファイルの URI
 * @param   {UnMHTSaveContext} context
 *          保存コンテキスト
 * @param   {boolean} isRequired
 *          必須か
 * @implements arINetFileLoaderListener
 */
function UnMHTSaveParam(loc, context, isRequired) {
  this.location = loc;
  this.savedFrom = loc;
  this.currentLocation = loc;
  this.saveContext = context;
  this.isRequired = isRequired;
  
  this.status = UnMHTSave.STATUS_FILE_NA;
  
  let loc = this.currentLocation;
  if (UnMHTScheme.isUnMHTURI(loc)) {
    loc = UnMHTScheme.deUnMHTURI(loc);
  }
  if (/^file:\/\//.test(loc)) {
    this.isLocal = true;
  }
  
  if (UnMHTScheme.isUnMHTURI(this.currentLocation)) {
    let [eFileInfo, part]
      = UnMHTExtractor.getFileInfoAndPart(this.currentLocation);
    if (eFileInfo && part) {
      this.eFileInfo = eFileInfo;
      if (part.contentLocation) {
        this.location = part.contentLocation;
      } else {
        this.location = "";
        this.cid = part.eParam.cid;
      }
    }
  }
}
UnMHTSaveParam.prototype = {
  eFileInfo: null,    /** 保存元のファイルの展開情報
                       *  @type {UnMHTExtractFileInfo} */
  saveContext: null,  /** 保存コンテキスト
                       *  @type {UnMHTSaveContext} */
  
  /* ファイル取得 */
  startTime: 0,        /** 開始時間
                        *  @type {number} */
  status: 0,           /** 取得状況
                        *  @type {number}
                        *    UnMHTSave.STATUS_* */
  depth: 0,            /** 参照の深さ
                        *  @type {number} */
  
  host: "",            /** 保存するファイルのホスト
                        *  ホスト当たりの接続数チェック用
                        *  @type {string} */
  
  fileSize: 0,         /** 取得前に分かっているファイルサイズ
                        *  不明の場合 0
                        *  @type {number} */
  recvSize: 0,         /** 取得したファイルサイズ
                        *  @type {number} */
  
  loadContext: null,   /** ファイルロードコンテキスト
                        *  @type {arNetFileLoadContext} */
  loaderStatus: 0,     /** ファイルローダの取得状況
                        *  @type {number}
                        *    arNetFileLoader.STATUS_* */
  
  /* ドキュメント取得 */
  isDocument: false,      /** ドキュメントか
                           *  @type {boolean} */
  targetDocument: null,   /** 現在のドキュメント
                           *  @type {nsIDOMDocument} */
  ownerDocument: null,    /** ロードしたドキュメント
                           *  @type {nsIDOMDocument} */
  targetSelection: null,  /** 対象の選択範囲
                           *  @type {nsISelection} */
  saveState: 0,           /** 保存状態
                           *  @type {number} */
  hasBaseElement: false,  /** BASE 要素を 持っているか
                           *  @type {boolean} */
  treeScanner: null,      /** ツリースキャナ
                           *  @type {UnMHTTreeScanner} */
  isRequired: false,      /** 必須か
                           *  必須のファイルが失敗した場合全体の失敗扱いとなる
                           *  @type {boolean} */
  
  frameIDs: null,         /** フレームの Content-ID
                           *  @type {Map.<nsIDOMElement,string>} */
  storeContentID: false,  /** Content-Location があっても
                           *  Content-ID を格納するか
                           *  @type {boolean} */
  
  /* ファイル情報 */
  isLocal: false,      /** ローカルファイルか
                        *  @type {boolean} */
  
  baseDir: "",         /** ベースディレクトリ
                        *  @type {string}
                        *  MHT 中のファイルの場合 Content-Location 基準 */
  isCSS: false,        /** CSS かどうか
                        *  @type {boolean}
                        *  CSS の import をチェックするか */
  isHTML: false,       /** HTML かどうか
                        *  @type {boolean}
                        *  コメント, リンクを入れるか
                        *  CSS の import をチェックするか */
  isIFrame: false,     /** インラインフレームか
                        *  @type {boolean}
                        *  コメント, リンクを入れるか */
  isText: false,       /** テキストかどうか
                        *  @type {boolean}
                        *  quoted-printable で保存するか */
  isMail: false,       /** メールかどうか
                        *  @type {boolean} */
  
  attachments: null,   /** 添付ファイル
                        *  @type {Array}
                        *    [[string ファイル名, string URL], ...] */
  
  location: "",        /** Content-Location に格納する値
                        *  @type {string} */
  savedFrom: "",       /** 保存元のアドレス
                        *  @type {string} */
  currentLocation: "", /** 保存対象の URI
                        *  @type {string} */
  cid: "",             /** Content-ID に格納する値
                        *  @type {string} */
  referrer: "",        /** リファラ
                        *  @type {string} */
  
  content: "",         /** ファイルの中身
                        *  @type {string} */
  
  mimetype: "",        /** MIME-Type
                        *  @type {string}
                        *  データ取得途中に変化する可能性がある */
  charset: "",         /** キャラクタセット
                        *  @type {string}
                        *  データ取得途中に変化する可能性がある */
  
  importFiles: null,   /** @import しているファイル
                        *  @type {Array}
                        *    [[string  URL,
                        *      number nsIContentPolicy の対象の種類], ...] */
  
  
  messageURI: "",          /** メールの URI
                            *  @type {string} */
  mailHeaderDisplay: null, /** メールに表示するヘッダ
                            *  @type {Array}
                            *   [[string 名前, string 値], ...] */
  
  faviconData: null,   /** favicon の data: スキーム表現
                        *  @type {string} */
  faviconType: null,   /** favicon の MIME-Type
                        *  @type {string} */
  
  /**
   * 破棄
   */
  destruct: function() {
    this.saveContext = null;
    
    this.eFileInfo = null;
    this.importFiles = null;
    
    this.targetDocument = null;
    this.ownerDocument = null;
    this.targetSelection = null;
    this.frameIDs = null;
    if (this.treeScanner) {
      try {
        this.treeScanner.destruct();
      } catch (e) {
      }
      this.treeScanner = null;
    }
    
    this.content = "";
  },
  
  /* ==== arINetFileLoaderListener インターフェース ==== */
  
  /**
   * 状態変化のイベント
   *
   * @param   {arNetFileLoadContext} context
   *          ファイルロードコンテキスト
   * @param   {number} status
   *          状態
   *            arNetFileLoader.STATUS_*
   */
  onStatusChange: function(context, status) {
    this.loaderStatus = status;
  },
  
  /**
   * 取得開始のイベント
   *
   * @param   {arNetFileLoadContext} context
   *          ファイルロードコンテキスト
   * @param   {number} status
   *          状態
   *            arNetFileLoader.STATUS_*
   * @param   {nsIURI} location
   *          取得中の URI
   * @param   {number} size
   *          ファイルサイズ
   *          不明ならば 0
   */
  onStart: function(context, status, location, size) {
    this.loaderStatus = status;
    this.currentLocation = location.spec;
    this.fileSize = size;    
    
    this.startTime = Date.now();
    this.content = "";
  },
  
  /**
   * 取得終了のイベント
   *
   * @param   {arNetFileLoadContext} context
   *          ファイルロードコンテキスト
   * @param   {number} status
   *          状態
   *            arNetFileLoader.STATUS_*
   * @param   {nsIChannel} channel
   *          ネットワークから取得した場合のチャネル
   */
  onStop: function(context, status, channel) {
    if (this.status == UnMHTSave.STATUS_NG) {
      return;
    }
    
    if (status == arNetFileLoader.STATUS_NG) {
      this.status = UnMHTSave.STATUS_NG;
      return;
    }
    
    if (this.mimetype == "" && channel) {
      let contentType = "";
      try {
        contentType
          = channel.QueryInterface(Ci.nsIHttpChannel)
          .getResponseHeader("Content-Type");
      } catch (e) {
      }
      if (contentType) {
        let param
          = arMIMEDecoder.decodeParam(contentType);
        if (param.type && param.subtype) {
          this.mimetype = param.type + "/" + param.subtype;
        }
        if (param.params.has("charset")) {
          this.charset  = param.params.get("charset");
        }
      }
    }
    
    if (this.mimetype == "") {
      let re = this.currentLocation.match(/\.([^\.]+)(?:\?.+)?(?:#.+)?$/i);
      if (re) {
        this.mimetype = arMIMEUtil.getTypeFromExtension(re[1]);
      }
    }
    
    if (this.isHTML || this.isCSS) {
      let origLoc = this.currentLocation;
      let isBase = false;
      if (this.isHTML && this.baseDir) {
        origLoc = this.baseDir;
        isBase = true;
      }
      
      let loc = _getMHTContentLocation(origLoc);
      let isMHT = false;
      if (loc != origLoc) {
        isMHT = true;
      }
      
      if (!isBase) {
        loc = arPathUtil.getBaseDir(loc);
      }
      
      this.importFiles
        = UnMHTCSSUtil.gatherImport(this.content, loc)
        .map(function(url) {
          if (isMHT) {
            try {
              let MHTloc
                = UnMHTExtractor.getMHTFileURI(Services.io.newURI(url, null, null),
                                               Services.io.newURI(origLoc, null, null));
              if (MHTloc) {
                return MHTloc.spec;
              }
            } catch (e) {
            }
          }
          return url;
        })
        .map(function(url) [url, Ci.nsIContentPolicy.TYPE_STYLESHEET]);
    }
    
    this.status = UnMHTSave.STATUS_OK;
    
    UnMHTSave.modifyContent(this, this.currentLocation,
                            this.saveContext._saveDate,
                            "");
  },
  
  /**
   * データ到着のイベント
   *
   * @param   {arNetFileLoadContext} context
   *          ファイルロードコンテキスト
   * @param   {number} status
   *          状態
   *            arNetFileLoader.STATUS_*
   * @param   {string} data
   *          データ
   * @param   {number} count 
   *          データの長さ
   */
  onData: function(context, status, data, count) {
    this.content += data;
    this.recvSize += count;
  }
};

/**
 * 保存コンテキスト
 *
 * @param   {nsIDOMWindow} window
 *          実行中のウィンドウ
 * @param   {nsIFile} targetFile
 *          対象のファイル,
 * @param   {string} location
 *          メールを保存する際のルートドキュメントの URI
 * @param   {string} subject
 *          ファイルの subject
 * @param   {nsIDownloadProgressListener} downloadListener
 *          ダウンロードのイベントリスナ
 * @param   {number} saveState
 *          保存状態の数値表現
 * @returns {UnMHTSaveContext}
 *          保存コンテキスト
 */
function UnMHTSaveContext(window, targetFile, location, subject,
                          downloadListener, saveState) {
  this._window = window;
  this._targetFile = targetFile;
  this._location = location;
  if (downloadListener) {
    this._hasDownload = true;
  }
  this._saveState = saveState;
  
  this._deferred = Promise.defer();
  
  this._saveDate = new Date();
  
  this._urls = new Set();
  this._saveParams = [];
  
  this._deepImportAction = UnMHTPref.getPref("save.import.limit.exceeded.action");
  
  let name = "network.http.max-connections";
  if (Services.prefs.getPrefType(name) != 0) {
    this._maxConns = Math.ceil(Services.prefs.getIntPref(name) / 2);
  }
  name = "network.http.max-connections-per-server";
  if (Services.prefs.getPrefType(name) != 0) {
    this._maxConnsPerServer
      = Math.ceil(Services.prefs.getIntPref(name) / 2);
  }
  
  this._saveContentsAction
    = new Map(["video", "embed"]
              .map(function(type)
                   [type, UnMHTPref.getPref("save.contents." + type)]));
  
  this._prepareTopPart(subject);
}
UnMHTSaveContext.prototype = {
  progress: 0,           /** 進捗状況 (0-1)
                          *  @type {number} */
  
  /**
   * 破棄処理
   * start 後は自分で呼ぶ
   */
  destruct: function() {
    if (this._listener) {
      this._listener.onDestruct();
      this._listener = null;
    }
    
    if (this._saveParams) {
      this._saveParams.forEach(function(sParam) sParam.destruct());
      this._saveParams = null;
    }
    
    if (this._topPart) {
      this._topPart.destruct();
      this._topPart = null;
    }
    
    if (this._mailPart) {
      this._mailPart.destruct();
      this._mailPart = null;
    }
    
    this._targetFile = null;
    this._urls = null;
    this._startPart = null;
    
    this._window = null;
  },

  /**
   * イベントリスナを設定する
   *
   * @param   {IUnMHTSaveListener} listener
   *          イベントリスナ
   */
  setLisener: function(listener) {
    this._listener = listener;
  },
  
  /**
   * フィールドを追加する
   *
   * @param   {string} name
   *          フィールド名
   * @param   {string} vlaue
   *          フィールド値
   * @param   {boolean} storeAll
   *          UnMHT が使用するフィールドも無視しない
   */
  addField: function(name, value, storeAll) {
    if (!this._isFreeField(name) ||
        name.toLowerCase() == "content-type") {
      if (storeAll) {
        name = "X-UnMHT-Original-" + name;
      } else {
        return;
      }
    }
    this._topPart.addField(name, value);
  },
  
  /**
   * 複数のタブを含むかどうかを指定する
   *
   * @param   {boolean} isTabs
   *          複数のタブを含むかどうか
   */
  setTabs: function(isTabs) {
    this._isTabs = isTabs;
  },

  /**
   * タブのリストを指定する
   *
   * @param   {Array.<string>} tabs
   *          タブの Content-ID のリスト
   */
  setTabList: function(tabs) {
    this._tabs = tabs;
  },
  
  /**
   * ドキュメントを保存対象に追加する
   *
   * @param   {object} arg
   *            {
   *              targetDocument:  {nsIDOMDocument}
   *                               対象のドキュメント
   *              targetSelection: {nsISelection}
   *                               対象の選択範囲もしくは null
   *              scrapBookData:   {object}
   *                               ScrapBook のデータ
   *              saveState:       {number}
   *                               保存状態
   *              location:        {string}
   *                               ドキュメントの URL
   *              isHTML:          {boolean}
   *                               HTML かどうか
   *              isStartPart:     {boolean}
   *                               開始パートか
   *              savedFrom:       {string}
   *                               保存元 (オプショナル)
   *              doNotStop:       {boolean}
   *                               エラーが起きても中断しない (オプショナル)
   *              window:          {nsIDOMWindow}
   *                               ドキュメントを持つブラウザを
   *                               実行中のウィンドウ
   *                               メールの添付ファイル取得に使う
   *                               (オプショナル)
   *            }
   * @returns {arMIMEPart}
   *          対応するパート
   *          失敗したら null
   */
  addDocument: function(arg) {
    if (this._isStopped || this._isCanceled) {
      return null;
    }
    
    let getArg = function(name, defValue) {
      return (name in arg) ? arg[name] : defValue;
    };
    
    let targetDocument  = getArg("targetDocument", null);
    let targetSelection = getArg("targetSelection", null);
    let scrapBookData   = getArg("scrapBookData", null);
    let saveState       = getArg("saveState", UnMHTSave.SAVESTATE_CURRENT);
    let location        = getArg("location", "");
    let isHTML          = getArg("isHTML", false);
    let isStartPart     = getArg("isStartPart", false);
    let window          = getArg("window", null);
    let savedFrom       = getArg("savedFrom", location);
    let needContentID   = getArg("needContentID", false);
    
    
    if (UnMHTScheme.isUnMHTURI(location)) {
      let [eFileInfo, currentPart]
        = UnMHTExtractor.getFileInfoAndPart(location);
      if (!(eFileInfo && currentPart)) {
        return null;
      }
    }
    
    let part = null;
    if (scrapBookData) {
      part = this._addScrapBook(scrapBookData, isStartPart);
    } else if ((saveState & UnMHTSave.SAVESTATE_MASK_ORIGINAL) &&
               UnMHTScheme.isUnMHTURI(location)) {
      part = this._addUnMHTDocument(location, isStartPart, savedFrom,
                                    needContentID);
      
      this._addNormalDocument(targetDocument, targetSelection,
                              saveState,
                              location, false,
                              0, false, window, true);
    } else if (isHTML) {
      part
        = this._addNormalDocument(targetDocument, targetSelection,
                                  saveState,
                                  location, isStartPart,
                                  0, false, window, false);
    } else {
      part = this.addFile(targetDocument, location, isStartPart);
    }
    
    if (!part) {
      return null;
    }
    
    if (part.sParam) {
      part.sParam.savedFrom = savedFrom;
    }
    
    return part;
  },
  
  /**
   * データドキュメントを保存対象に追加する
   *
   * @param   {string} content
   *          内容
   * @param   {string} location
   *          Content-Location の値
   * @param   {string} charset
   *          Content-Type のうち、文字コード
   * @param   {string} mimetype
   *          Content-Type のうち、MIME タイプ
   * @param   {boolean} isStartPart
   *          開始パートか
   * @returns {arMIMEPart}
   *          対応するパート
   *          失敗したら null
   */
  addDataDocument: function(content, location, charset, mimetype,
                            isStartPart) {
    let part = new arMIMEPart();
    let sParam = new UnMHTSaveParam(location, this, true);
    part.sParam = sParam;
    sParam.depth = 0;
    
    sParam.charset = charset;
    sParam.mimetype = mimetype;
    switch (sParam.mimetype) {
      case "text/html": {
        sParam.isHTML = true;
        sParam.isText = true;
        break;
      }
      case "text/css": {
        sParam.isCSS = true;
        sParam.isText = true;
        break;
      }
      case "text/javascript": {
        sParam.isText = true;
        break;
      }
    }
    
    sParam.content = content;
    
    sParam.status = UnMHTSave.STATUS_OK;
    
    this._saveParams.push(sParam);
    if (isStartPart) {
      this._startPart = part;
      this._topPart.parts.unshift(part);
    } else {
      this._topPart.parts.push(part);
    }
    
    return part;
  },
  
  /**
   * ファイルを保存対象に追加する
   *
   * @param   {nsIDOMDocument} targetDocument
   *          対象のファイルを開いたドキュメント
   *          なければ null
   * @param   {string} location
   *          対象のファイルの URL
   * @param   {boolean} isStartPart
   *          開始パートか
   * @returns {arMIMEPart}
   *          対応するパート
   *          失敗したら null
   */
  addFile: function(targetDocument, location, isStartPart) {
    if (!UnMHTSave.checkPolicy(Ci.nsIContentPolicy.TYPE_DOCUMENT,
                               location, targetDocument)) {
      return null;
    }
    
    let urlForCheck = UnMHTMailPrivacy.hideMailURL(location, this._location);
    this._urls.add(urlForCheck);
    
    let part = new arMIMEPart();
    let sParam = new UnMHTSaveParam(location, this, isStartPart);
    sParam.ownerDocument = targetDocument;
    part.sParam = sParam;
    if (targetDocument) {
      try {
        if ("referrer" in targetDocument &&
            targetDocument.referrer) {
          sParam.referrer = targetDocument.referrer;
        }
        
        sParam.mimetype = targetDocument.contentType;
        sParam.charset = targetDocument.characterSet || "utf-8";
      } catch (e) {
        /* ドキュメントが生きていない */
      }
    }
    
    this._saveParams.push(sParam);
    if (isStartPart) {
      this._startPart = part;
      this._topPart.parts.unshift(part);
    } else {
      this._topPart.parts.push(part);
    }
    
    return part;
  },
  
  /**
   * 保存開始
   *
   * @returns {Promise<[nsIFile,Array.<string>]>}
   *          resolve: 保存したファイルと保存に失敗したファイルリスト
   *          reject: {number}
   *                  UnMHTSave.RESULT_*
   */
  start: function() {
    if (this._saveParams.length > 0) {
      this._checkProgress();
    } else {
      this.progress = 1;
      if (this._listener) {
        this._listener.onProgress(this, "", "0 / 0", 0, 0, 0);
      }
      
      try {
        this._storeToFile();
      } catch (e) {
        Cu.reportError(e + "\n" + e.stack);
      }
    }

    return this._deferred.promise;
  },
  
  /**
   * 保存をキャンセル
   */
  cancel: function() {
    if (this._isCanceling) {
      return;
    }
    
    this._isCanceling = true;
    
    if (this._isStopped) {
      if (this.listener) {
        this._listener.onCancel(this);
      }
      this._deferred.reject(UnMHTSave.RESULT_CANCEL);
      return;
    }
    
    this._isCanceled = true;
    
    if (this._saveParams) {
      this._saveParams.forEach(function(sParam) {
        try {
          sParam.loadContext.cancel();
        } catch (e) {
        }
      });
    }
  },
  
  /**
   * 対象のファイル
   *  @type {nsIFile}
   */
  get targetFile () {
    if (!this._targetFile) {
      return null;
    }
    return this._targetFile.clone();
  },

  /* ==== private ==== */
  
  _deferred: null,        /** 結果を表す Deferred
                           *  @type {Deferred<[nsIFile,Array.<string>]>} */
  _window: null,          /** 実行中のウィンドウ
                           *  @type {nsIDOMWindow} */
  _targetFile: null,      /** 対象のファイル
                           *  @type {nsIFile} */
  _location: "",          /** ルートドキュメントの URI
                           *  @type {string} */
  _listener: null,        /** イベントリスナ
                           *  @type {IUnMHTSaveListener} */
  
  _maxConns: 30,          /** 最大リクエスト数
                           *  @type {number} */
  _maxConnsPerServer: 8,  /** サーバあたりの最大リクエスト数
                           *  @type {number} */
  
  _saveDate: null,        /** 保存開始時刻
                           *  @type {Date} */
  
  _topPart: null,         /** トップレベルのパート
                           *  @type {arMIMEPart} */
  _startPart: null,       /** 開始パート
                           *  @type {arMIMEPart} */
  _isMixed: false,        /** トップレベルが multipart/mixed か
                           *  @type {boolean} */
  _isAlternative: false,  /** トップレベルが multipart/alternative か
                           *  @type {boolean} */
  _isTabs: false,         /** 複数のタブを含むかどうか
                           *  @type {boolean} */
  _tabs: null,            /** タブの Content-ID のリスト
                           *  @type {Array.<string>} */
  
  _urls: null,            /** 収集した URL
                           *  @type {Set.<string>} */
  _saveParams: null,      /** 作業中の保存情報
                           *  @type {Array.<UnMHTSaveParam>} */
  
  _isCanceled: false,     /** キャンセルされたか
                           *  @type {boolean} */
  _isCanceling: false,     /** キャンセル中
                            *  @type {boolean} */
  _isStopped: false,      /** 保存が停止したか
                           *  @type {boolean} */
  
  _deepImportAction: "",  /** 現在の値
                           *  @type {string}
                           *    "save": 保存する
                           *    "ignore": 無視する
                           *    "ask": ダイアログを表示する */
  
  _saveContentsAction: "", /** コンテンツの保存の選択
                            *  @type {Map.<string,string>}
                            *    <string 種類, string 保存するか>
                            *      "save": 保存する
                            *      "ignore": 無視する
                            *      "ask": ダイアログを表示する */
  
  _isMailOriginal: false,  /** メールを元のファイルで保存するか
                            *  @type {boolean} */
  _hasDownload: false,     /** ダウンロード履歴に登録済みか
                            *  @type {boolean} */
  
  /**
   * UnMHT が使用しないフィールドかどうか
   *
   * @param   {string} name
   *          フィールド名
   * @returns {boolean}
   *          UnMHT が使用しないフィールドかどうか
   */
  _isFreeField: function(name) {
    name = name.toLowerCase();
    
    return !(name == "from" ||
             name == "subject" ||
             name == "date" ||
             name == "mime-version" ||
             name == "x-unmht-tabs");
  },
  
  /**
   * ファイルを追加可能かどうかチェックして追加する
   *
   * @param   {string} url
   *          対象の URL
   * @param   {number} type
   *          nsIContentPolicy の対象の種類
   * @param   {nsIDOMDocument} targetDocument
   *          対象のドキュメント
   * @param   {number} depth
   *          現在の深さ
   * @param   {string} referrer
   *          リファラ
   * @param   {string} mimetype
   *          ファイルの MIME-Type
   *          オプショナル
   * @param   {boolean} isRemoteGather
   *          リモートファイルのみ収集するか
   */
  _checkAndPush: function(url, type,
                          targetDocument,
                          depth, referrer,
                          mimetype, isRemoteGather) {
    if (!url) {
      return;
    }
    
    if (/^data:/.test(url)) {
      /* data スキームは別途保存する必要が無い */
      return;
    }
    
    if (isRemoteGather && /^unmht:/.test(url)) {
      return;
    }
    
    if (this._isMailOriginal) {
      if (UnMHTMailPrivacy.isMailURL(url)) {
        return;
      }
      if (/^chrome:/.test(url)) {
        return;
      }
    }
    
    if (!mimetype &&
        UnMHTMailPrivacy.isMailURL(url)) {
      let re = url.match(/(^.+\?.*)&type=([^&]+)(.*)/);
      if (re) {
        mimetype = re[2];
      }
    }
    
    let urlForCheck = UnMHTMailPrivacy.hideMailURL(url, this._location);
    if (!this._urls.has(urlForCheck) &&
        UnMHTSave.checkPolicy(type, url, targetDocument)) {
      this._urls.add(urlForCheck);
      
      let part = new arMIMEPart();
      let sParam = new UnMHTSaveParam(url, this, false);
      sParam.ownerDocument = targetDocument;
      part.sParam = sParam;
      
      sParam.depth = depth + 1;
      sParam.referrer = referrer;
      
      switch (type) {
        case Ci.nsIContentPolicy.TYPE_STYLESHEET: {
          sParam.isCSS = true;
          sParam.isText = true;
          sParam.mimetype = "text/css";
          break;
        }
        case Ci.nsIContentPolicy.TYPE_SCRIPT: {
          sParam.isText = true;
          sParam.mimetype = "text/javascript";
          break;
        }
      }
      
      if (mimetype) {
        sParam.mimetype = mimetype;
      }
      
      this._saveParams.push(sParam);
      this._topPart.parts.push(part);
    }
  },
  
  /**
   * 対象の要素から保存対象のファイルの URL を収集する
   *
   * @param   {nsIDOMDocument} targetDocument
   *          対象のドキュメント
   * @param   {string} location
   *          ドキュメントの URL
   * @param   {string} savedFrom
   *          ドキュメントの URL
   * @param   {number} saveState
   *          保存状態
   * @param   {nsIDOMElement} node
   *          対象の要素
   * @param   {string} baseDir
   *          ベースディレクトリ
   * @param   {number} depth
   *          現在の深さ
   * @param   {nsISelection} targetSelection
   *          対象の選択範囲もしくは null
   * @param   {string} referrer
   *          リファラ
   * @param   {boolean} isMail
   *          メールか
   * @param   {boolean} isRemoteGather
   *          リモートファイルのみ収集するか
   * @param   {boolean} saveVideo
   *          VIDEO/AUDIO を保存するか
   * @param   {boolean} saveEmbed
   *          EMBED/OBJECT を保存するか
   */
  _gatherFilesFromNode: function(targetDocument, location, savedFrom,
                                 saveState,
                                 node, baseDir, depth,
                                 targetSelection,
                                 referrer, isMail, isRemoteGather,
                                 frameIDs, saveVideo, saveEmbed) {
    if (targetSelection) {
      if (!UnMHTDOMUtil.isNodeInSelection(node, targetSelection)) {
        return;
      }
    }
    
    let checkCSS = function(value) {
      if (!value) {
        return;
      }
      if (value.cssValueType == Ci.nsIDOMCSSValue.CSS_PRIMITIVE_VALUE) {
        if (value.primitiveType == Ci.nsIDOMCSSPrimitiveValue.CSS_URI) {
          let url = arPathUtil.resolve(baseDir, value.getStringValue());
          this._checkAndPush(url, Ci.nsIContentPolicy.TYPE_IMAGE,
                             targetDocument, depth, referrer,
                             null, isRemoteGather);
        }
      } else if (value.cssValueType == Ci.nsIDOMCSSValue.CSS_VALUE_LIST) {
        Array.forEach(value, function(item) checkCSS(item));
      }
    }.bind(this);
    
    [
      "",
      ":before",
      ":after",
      "::first-letter",
      "::first-line"
      /*
       * ::selection は色のみなので無視
       */
    ].forEach(function(pseudo) {
      let style = null;
      try {
        style = targetDocument.defaultView.getComputedStyle(node, pseudo);
      } catch (e) {
        return;
      }
      
      if (!style) {
        return;
      }
      
      [
        "background-image",
        "list-style-image",
        "border-image-source",
        "cursor",
        "content"
      ].forEach(function(name) {
        let value = null;
        try {
          value = style.getPropertyCSSValue(name);
        } catch (e) {
          return;
        }
        checkCSS(value);
      }.bind(this));
    }.bind(this));
    
    if ("background" in node) {
      this._checkAndPush(node.background, Ci.nsIContentPolicy.TYPE_IMAGE,
                         targetDocument, depth, referrer,
                         null, isRemoteGather);
    }
    
    let window = this._window;
    
    if (node instanceof Ci.nsIDOMHTMLStyleElement) {
      let isMHT = UnMHTScheme.isUnMHTURI(savedFrom);
      
      let params = {
        noScript: false,
        isMail: isMail,
        saveCanvas: false,
      };
      UnMHTCSSUtil.gatherImport(UnMHTDOMConverter.toPlainText(this._window,
                                                              location,
                                                              node, null,
                                                              params),
                                baseDir)
        .map(function(url) {
          if (isMHT) {
            try {
              let MHTloc
                = UnMHTExtractor.getMHTFileURI(Services.io.newURI(url, null, null),
                                               Services.io.newURI(savedFrom, null, null));
              if (MHTloc) {
                return MHTloc.spec;
              }
            } catch (e) {
            }
          }
          return url;
        })
        .forEach(function(url) {
          this._checkAndPush(url, Ci.nsIContentPolicy.TYPE_STYLESHEET,
                             targetDocument, depth, referrer,
                             null, isRemoteGather);
        }.bind(this));
    } else if (node instanceof Ci.nsIDOMHTMLImageElement) {
      this._checkAndPush(node.src, Ci.nsIContentPolicy.TYPE_IMAGE,
                         targetDocument, depth, referrer,
                         null, isRemoteGather);
    } else if (node instanceof Ci.nsIDOMHTMLVideoElement) {
      if (saveVideo) {
        this._checkAndPush(node.src, Ci.nsIContentPolicy.TYPE_OBJECT,
                           targetDocument, depth, referrer,
                           null, isRemoteGather);
        this._checkAndPush(node.poster, Ci.nsIContentPolicy.TYPE_OBJECT,
                           targetDocument, depth, referrer,
                           null, isRemoteGather);
      }
    } else if (node instanceof Ci.nsIDOMHTMLAudioElement) {
      if (saveVideo) {
        this._checkAndPush(node.src, Ci.nsIContentPolicy.TYPE_OBJECT,
                           targetDocument, depth, referrer,
                           null, isRemoteGather);
      }
    } else if (node instanceof Ci.nsIDOMHTMLSourceElement) {
      if (saveVideo) {
        this._checkAndPush(node.src, Ci.nsIContentPolicy.TYPE_OBJECT,
                           targetDocument, depth, referrer,
                           null, isRemoteGather);
      }
    } else if (node instanceof Ci.nsIDOMHTMLLinkElement &&
               node.rel) {
      if (/\bicon\b/i.test(node.rel)) {
        this._checkAndPush(node.href, Ci.nsIContentPolicy.TYPE_OBJECT,
                           targetDocument, depth, referrer,
                           null, isRemoteGather);
      } else if (/\bstylesheet\b/i.test(node.rel)) {
        this._checkAndPush(node.href, Ci.nsIContentPolicy.TYPE_STYLESHEET,
                           targetDocument, depth, referrer,
                           null, isRemoteGather);
      }
    } else if (node instanceof Ci.nsIDOMHTMLInputElement &&
               node.type.toLowerCase() == "image") {
      this._checkAndPush(node.src, Ci.nsIContentPolicy.TYPE_IMAGE,
                         targetDocument, depth, referrer,
                         null, isRemoteGather);
    } else if (node instanceof Ci.nsIDOMHTMLScriptElement) {
      if (saveState != UnMHTSave.SAVESTATE_CURRENT_NOJS) {
        this._checkAndPush(node.src, Ci.nsIContentPolicy.TYPE_SCRIPT,
                           targetDocument, depth, referrer,
                           null, isRemoteGather);
      }
    } else if (node instanceof Ci.nsIDOMHTMLEmbedElement) {
      if (saveEmbed) {
        this._checkAndPush(arPathUtil.resolve(baseDir, node.src),
                           Ci.nsIContentPolicy.TYPE_OBJECT,
                           targetDocument, depth, referrer,
                           node.getAttribute("type"), isRemoteGather);
      }
    } else if (node instanceof Ci.nsIDOMHTMLObjectElement) {
      if (saveEmbed) {
        this._checkAndPush(arPathUtil.resolve(baseDir, node.data),
                           Ci.nsIContentPolicy.TYPE_OBJECT,
                           targetDocument, depth, referrer,
                           node.getAttribute("type"), isRemoteGather);
      }
    } else if (node instanceof Ci.nsIDOMHTMLFrameElement) {
      let cid = frameIDs.has(node) ? frameIDs.get(node) : null;
      
      this._addNormalDocument(node.contentDocument, targetSelection,
                              saveState,
                              node.contentDocument.location.href, false,
                              depth, false, null, isRemoteGather, cid);
    } else if (node instanceof Ci.nsIDOMHTMLIFrameElement) {
      let cid = frameIDs.has(node) ? frameIDs.get(node) : null;
      
      this._addNormalDocument(node.contentDocument, targetSelection,
                              saveState,
                              node.contentDocument.location.href, false,
                              depth, true, null, isRemoteGather, cid);
    }
  },
  
  /**
   * コンテンツを保存するかどうか調べる
   *
   * @param   {string} type
   *          種類
   * @returns {Promise<boolean>}
   *          保存するか
   */
  _checkSaveContents: function(type) {
    return Task.spawn(function() {
      let action = this._saveContentsAction.get(type);
      if (arAppInfo.isFennec && action == "ask") {
        action = "save";
      }
      if (action == "ask") {
        action = yield UnMHTSave.showSaveContentsConfirm(this._window, type);
        this._saveContentsAction.set(type, action);
      }
      
      throw new Task.Result(action == "save");
    }.bind(this));
  },

  /**
   * CSS の階層が深い場合に保存するかどうか調べる
   *
   * @returns {Promise<boolean>}
   *          保存するか
   */
  _checkDeepImportAction: function() {
    return Task.spawn(function() {
      let action = this._deepImportAction;
      if (arAppInfo.isFennec && action == "ask") {
        action = "ignore";
      }
      if (this._deepImportAction == "ask") {
        action = yield UnMHTSave.showDeepCSSConfirm(this._window);
        this._deepImportAction = action;
      }
      
      throw new Task.Result(action == "save");
    }.bind(this));
  },
  
  /**
   * 通常の HTML ドキュメントを保存対象に追加する
   *
   * @param   {nsIDOMDocument} targetDocument
   *          対象のドキュメント
   * @param   {nsISelection} targetSelection
   *          対象の選択範囲もしくは null
   * @param   {number} saveState
   *          保存状態
   * @param   {string} location
   *          ドキュメントの URL
   * @param   {boolean} isStartPart
   *          開始パートか
   * @param   {number} depth
   *          現在の深さ
   * @param   {boolean} isIFrame
   *          インラインフレームか
   * @param   {nsIDOMWindow} window
   *          ドキュメントを持つブラウザを実行中のウィンドウ
   *          メールの添付ファイル取得に使う (オプショナル)
   * @param   {boolean} isRemoteGather
   *          リモートファイルのみ収集するか
   * @returns {arMIMEPart}
   *          メインのパート
   *          失敗したら null
   */
  _addNormalDocument: function(targetDocument, targetSelection,
                               saveState,
                               location, isStartPart,
                               depth, isIFrame,
                               window, isRemoteGather,
                               cid) {
    if (!isRemoteGather) {
      let urlForCheck = UnMHTMailPrivacy.hideMailURL(location, this._location);
      this._urls.add(urlForCheck);
    }
    
    let part = null;
    if (!isRemoteGather) {
      part = new arMIMEPart();
    }
    let sParam = new UnMHTSaveParam(location, this, true);
    sParam.ownerDocument = targetDocument;
    if (part) {
      part.sParam = sParam;
    }
    
    sParam.depth = depth + 1;
    try {
      if ("referrer" in targetDocument &&
          targetDocument.referrer) {
        sParam.referrer = targetDocument.referrer;
      }
    } catch (e) {
      if (part) {
        part.destruct();
      }
      sParam.destruct();
      return null;
    }
    
    sParam.isIFrame = isIFrame;
    if (cid) {
      sParam.cid = cid;
      sParam.storeContentID = true;
    }
    sParam.isHTML = true;
    sParam.isText = true;
    sParam.isMail = UnMHTMailPrivacy.isMailURL(location);
    sParam.mimetype = "text/html";
    try {
      sParam.charset = targetDocument.characterSet || "utf-8";
    } catch (e) {
      sParam.charset = "utf-8";
    }
    
    if (isStartPart && sParam.eFileInfo) {
      /* UnMHT の展開した結果であり
       * 開始パートならば展開結果のヘッダを継承する */
      this._addFieldList(sParam.eFileInfo.topPart.fieldList, false);
    }
    
    let ok;
    [ok, sParam.baseDir, sParam.hasBaseElement]
      = this._getBaseDir(targetDocument, location);
    if (!ok) {
      if (part) {
        part.destruct();
      }
      sParam.destruct();
      return null;
    }
    
    sParam.isDocument = true;
    sParam.targetDocument = targetDocument;
    sParam.targetSelection = targetSelection;
    sParam.saveState = saveState;
    sParam.frameIDs = new Map();
    
    if ((saveState & UnMHTSave.SAVESTATE_MASK_CURRENT) ||
        targetSelection) {
      sParam.status = UnMHTSave.STATUS_DOC_CUR_NA;
    } else if (isRemoteGather) {
      sParam.status = UnMHTSave.STATUS_REMOTE_NA;
    } else {
      sParam.status = UnMHTSave.STATUS_DOC_ORI_NA;
    }
    this._saveParams.push(sParam);
    
    if (isRemoteGather) {
      return part;
    }
    
    if (isStartPart) {
      this._startPart = part;
      this._topPart.parts.unshift(part);
    } else {
      this._topPart.parts.push(part);
    }
    
    if (sParam.isMail && window) {
      if (sParam.status == UnMHTSave.STATUS_DOC_ORI_NA) {
        this._isMailOriginal = true;
      } else if (sParam.status == UnMHTSave.STATUS_DOC_CUR_NA) {
        if ("currentAttachments" in window &&
            window.currentAttachments) {
          sParam.attachments
            = window.currentAttachments.map(function(attachment) {
              if ((("hasFile" in attachment && attachment.hasFile) ||
                   !("hasFile" in attachment)) &&
                  !("isDeleted" in attachment && attachment.isDeleted) &&
                  !("isExternalAttachment" in attachment && attachment.isExternalAttachment) &&
                  attachment.url) {
                let filePart = this.addFile(null, attachment.url, false);
                filePart.sParam.mimetype = attachment.contentType;
                let name = "attachment";
                if ("displayName" in attachment && attachment.displayName) {
                  name = attachment.displayName;
                }
                if ("name" in attachment && attachment.name) {
                  name = attachment.name;
                }
                return [name, attachment.url];
              }
              return null;
            }.bind(this))
            .filter(function(attachment) attachment);
        }
      }
      
      if ("gFolderDisplay" in window &&
          "selectedMessageUris" in window.gFolderDisplay &&
          window.gFolderDisplay.selectedMessageUris.length == 1) {
        let mailSession
          = Cc["@mozilla.org/messenger/services/session;1"]
          .getService(Ci.nsIMsgMailSession);
        
        let messageURL
          = mailSession.ConvertMsgURIToMsgURL(window.gFolderDisplay.selectedMessageUris[0], window.msgWindow);
        
        let uri;
        try {
          uri = Services.io.newURI(messageURL, null, null);
        } catch (e) {
        }
        
        if (uri) {
          if (sParam.status == UnMHTSave.STATUS_DOC_CUR_NA) {
            sParam.status = UnMHTSave.STATUS_MAIL_CUR_MAIL_START;
          } else if (sParam.status == UnMHTSave.STATUS_DOC_ORI_NA) {
            sParam.status = UnMHTSave.STATUS_MAIL_ORI_MAIL_START;
          }
          
          sParam.messageURI = uri;
        }
        
        if ("gExpandedHeaderList" in window &&
            "gExpandedHeaderView" in window &&
            "currentHeaderData" in window &&
            UnMHTPref.getPref("save.header.mail.show")) {
          this._getMailHeader(window, sParam);
        }
      }
    }
    
    return part;
  },
  
  /**
   * メールヘッダーの情報を取得する
   *
   * @param   {nsIDOMWindow} window
   *          ドキュメントを持つブラウザを実行中のウィンドウ
   *          メールの添付ファイル取得に使う (オプショナル)
   * @param   {UnMHTSaveParam} sParam
   *          保存情報
   */
  _getMailHeader: function(window, sParam) {
    try {
      let display = [];
      
      let context = {};
      Cu.import("resource:///modules/mailServices.js", context);
      
      let used = new Set();
      
      let emailFormatter = function(value) {
        if (!value) {
          return "";
        }
        
        let addresses = {};
        let fullNames = {};
        let names = {};
        
        let count = context.MailServices.headerParser
          .parseHeadersWithArray(value, addresses, names, fullNames);
        
        let values = [];
        for (let i = 0; i < count; i ++) {
          values.push(fullNames.value[i] || names.value[i]);
        }
        
        return values.join(", ");
      };
      let newsgroupsFormatter = function(value) {
        return value.split(",").join(", ");
      };
      let messageIdFormatter = function(value) {
        return value.split(/\s+/).join(", ");
      };
      
      let addToDisplay = function(lowerHeaderName) {
        if (used.has(lowerHeaderName)) {
          return false;
        }
        
        if (!(lowerHeaderName in window.gExpandedHeaderView)) {
          return false;
        }
        let view = window.gExpandedHeaderView[lowerHeaderName];
        if (!view || !view.valid) {
          return false;
        }
        
        if (!(lowerHeaderName in window.currentHeaderData)) {
          return false;
        }
        let data = window.currentHeaderData[lowerHeaderName];
        if (!data) {
          return false;
        }
        
        let formatters = {
          "from": emailFormatter,
          "reply-to": emailFormatter,
          "to": emailFormatter,
          "cc": emailFormatter,
          "bcc": emailFormatter,
          
          "newsgroups": newsgroupsFormatter,
          "references": messageIdFormatter,
          "followup-to": newsgroupsFormatter,
          "message-id": messageIdFormatter,
          "in-reply-to": messageIdFormatter,
          "sender": messageIdFormatter
        };
        
        let value = "";
        
        if (lowerHeaderName in formatters) {
          value = formatters[lowerHeaderName](data.headerValue);
        } else {
          value = data.headerValue;
        }
        
        display.push([data.headerName, value]);
        
        used.add(lowerHeaderName);
        
        return true;
      };
      
      for (let i = 0; i < window.gExpandedHeaderList.length; i ++) {
        addToDisplay(window.gExpandedHeaderList[i].name);
      }
      
      if (addToDisplay("x-mozilla-localizeddate")) {
        used.add("date");
      }
      
      for (let lowerHeaderName of Object.keys(window.currentHeaderData)) {
        addToDisplay(lowerHeaderName);
      }
      
      sParam.mailHeaderDisplay = display;
    } catch (e) {
      Cu.reportError(e + "\n" + e.stack);
    }
  },
  
  /**
   * ベースディレクトリを取得する
   *
   * @param   {nsIDOMDocument} targetDocument
   *          対象のドキュメント
   * @param   {string} location
   *          ドキュメントの URL
   * @returns {Array}
   *            [boolean 成功したか, string ベースディレクトリ,
   *             boolean BASE 要素を持っているか]
   *         
   */
  _getBaseDir: function(targetDocument, location) {
    let hasBaseElement = false;
    let baseDir
      = arPathUtil.getBaseDir(_getMHTContentLocation(location));
    try {
      Array.slice(targetDocument.getElementsByTagName("base"))
        .forEach(function(node) {
          hasBaseElement = true;
          if (node.hasAttribute("href") &&
              node.getAttribute("href")) {
            let baseURI;
            try {
              let href = _getMHTContentLocation(node.getAttribute("href"));
              baseURI = Services.io.newURI(href, null, null);
            } catch (e) {
              return;
            }
            baseDir = arPathUtil.resolve(baseDir, baseURI.resolve("./"));
          }
        });
    } catch (e) {
      return [false, null, false];
    }
    
    return [true, baseDir, hasBaseElement];
  },
  
  /**
   * UnMHT で展開したドキュメントを保存対象に追加する
   *
   * @param   {string} location
   *          対象の URI
   * @param   {boolean} isStartPart
   *          開始パートか
   * @param   {string} savedFrom
   *          保存元のアドレス
   * @param   {boolean} needContentID
   *          Content-ID が必要か
   * @returns {arMIMEPart}
   *          メインのパート
   *          失敗したら null
   */
  _addUnMHTDocument: function(location, isStartPart, savedFrom,
                              needContentID) {
    let [eFileInfo, currentPart]
      = UnMHTExtractor.getFileInfoAndPart(location);
    if (!(eFileInfo && currentPart)) {
      return null;
    }
    
    if (isStartPart && currentPart == eFileInfo.startPart) {
      this._addFieldList(eFileInfo.topPart.fieldList, false);
    }
    
    let createSaveParam = function(part) {
      if (part.isMultipart) {
        part.parts.forEach(function(p) createSaveParam(p));
        return;
      }
      
      if (part.mimetype == "text/html") {
        let sParam = new UnMHTSaveParam(location, this, true);
        
        sParam.isHTML = true;
        sParam.content = part.body;
        
        UnMHTSave.modifyContent(sParam, savedFrom, this._saveDate,
                                "");
        
        part.body = sParam.content;
        
        sParam.destruct();
      }
    }.bind(this);
    
    let findPart = {
      from: currentPart,
      to: null
    };
    let part = eFileInfo.topPart.clone(findPart);
    let mainPart = findPart.to;
    
    createSaveParam(part);
    
    if (needContentID && !mainPart.contentID) {
      mainPart.contentID = currentPart.eParam.cid;
      mainPart.addField("Content-ID", mainPart.contentID);
    }
    
    if (isStartPart) {
      if (part.isMultipart) {
        this._startPart = part.findStartPart();
        if (this._startPart.mimetype == "multipart/mixed") {
          this._startPart = null;
          this._isMixed = true;
        } else if (this._startPart.mimetype == "multipart/alternative") {
          this._startPart = null;
          this._isAlternative = true;
        }
        
        part.parts.forEach(function(p) {
          this._topPart.parts.push(p);
        }.bind(this));
      } else {
        this._startPart = part;
        this._topPart.parts.unshift(part);
      }
    } else {
      part.fieldList
        = part.fieldList
        .filter((function([name, value]) this._isFreeField(name)).bind(this));
      part.createFieldsFromList();
      
      this._topPart.parts.push(part);
    }
    
    return mainPart;
  },
  
  /**
   * ScrapBook のデータを保存対象に追加する
   *
   * @param   {object} scrapBookData
   *          ScrapBook のデータ
   * @param   {boolean} isStartPart
   *          開始パートか
   * @returns {arMIMEPart}
   *          メインのパート
   *          失敗したら null
   */
  _addScrapBook: function(scrapBookData, isStartPart) {
    let mainPart = null;
    
    arFileUtil.getDirectoryEntries(scrapBookData["dir"])
      .forEach(function(targetFile) {
        if (targetFile.leafName == "index.dat") {
          return;
        }
        
        let isIndex = false;
        if (targetFile.leafName == "index.html") {
          isIndex = true;
        }
        
        let part = new arMIMEPart();
        let sParam = new UnMHTSaveParam(targetFile.leafName, this, true);
        part.sParam = sParam;
        
        sParam.depth = 0;
        sParam.referrer = "";
        
        if (isIndex) {
          sParam.location = scrapBookData["source"];
          sParam.mimetype = "text/html";
          sParam.charset = scrapBookData["charset"];
        } else {
          sParam.location
            = arPathUtil.resolve(arPathUtil.getBaseDir(scrapBookData["source"]),
                                 sParam.location);
          let re = sParam.location.match(/\.([^\.]+)(?:\?.+)?(?:#.+)?$/i);
          if (re) {
            sParam.mimetype = arMIMEUtil.getTypeFromExtension(re[1]);
          }
        }
        sParam.savedFrom = sParam.location;
        sParam.currentLocation = sParam.location;
        
        sParam.content = arFileUtil.readData(targetFile);

        UnMHTSave.modifyContent(sParam, sParam.savedFrom, this._saveDate,
                                "");
        
        sParam.status = UnMHTSave.STATUS_OK;
        
        this._saveParams.push(sParam);
        if (isIndex) {
          mainPart = part;
          this._topPart.parts.unshift(part);
          if (isStartPart) {
            this._startPart = part;
          }
        } else {
          this._topPart.parts.push(part);
        }
      }.bind(this));
    
    return mainPart;
  },
  
  /**
   * インポートされたファイルを追加する
   *
   * @param   {UnMHTSaveParam} sParam
   *          対象の保存情報
   */
  _addImportFiles: function(sParam) {
    return Task.spawn(function() {
      if (!(sParam.status == UnMHTSave.STATUS_OK ||
            sParam.status == UnMHTSave.STATUS_NG)) {
        return;
      }
      let importFiles = sParam.importFiles;
      sParam.importFiles = null;
      
      if (!importFiles || !importFiles.length) {
        return;
      }
      
      if (sParam.depth + 1 >= UnMHTSave.MAX_IMPORT_DEPTH) {
        let save = yield this._checkDeepImportAction();
        if (!save) {
          return;
        }
      }
      
      importFiles.forEach(function([url, type]) {
        let urlForCheck = UnMHTMailPrivacy.hideMailURL(url, this._location);
        if (!this._urls.has(urlForCheck) &&
            UnMHTSave.checkPolicy(type, url, sParam.ownerDocument)) {
          this._urls.add(urlForCheck);
          
          let part = new arMIMEPart();
          let newSParam = new UnMHTSaveParam(url, this, false);
          newSParam.ownerDocument = sParam.ownerDocument;
          part.sParam = newSParam;
          
          newSParam.depth = sParam.depth + 1;
          newSParam.referrer = sParam.currentLocation;
          
          if (type == Ci.nsIContentPolicy.TYPE_STYLESHEET) {
            newSParam.isText = true;
            newSParam.isCSS = true;
            newSParam.mimetype = "text/css";
          }
          
          this._saveParams.push(newSParam);
          this._topPart.parts.push(part);
        }
      }.bind(this));
    }.bind(this));
  },
  
  /**
   * mht で保存 - ファイル収集のチェックと状態の遷移
   */
  _checkProgress: function() {
    Task.spawn(function() {
      for (;;) {
        if (this._isCanceled) {
          this._isStopped = true;
          if (this._listener) {
            this._listener.onCancel(this);
          }
          this._deferred.reject(UnMHTSave.RESULT_CANCEL);
          this.destruct();
          return;
        }
        
        let ok = 0;
        let na_progress = 0;
        let ng = 0;
        
        let conns = 0;
        let connsPerServer = new Map();
        
        let currentFile = "";
        let currentSize = "";
        
        let now = Date.now();
        
        for (let i = 0, len = this._saveParams.length; i < len; i ++) {
          let sParam = this._saveParams[i];
          
          if (currentFile == "") {
            switch (sParam.status) {
              case UnMHTSave.STATUS_DOC_CUR_GATHER_WAIT:
              case UnMHTSave.STATUS_DOC_ORI_GATHER_WAIT:
              case UnMHTSave.STATUS_MAIL_ORI_GATHER_WAIT:
              case UnMHTSave.STATUS_REMOTE_GATHER_WAIT: {
                currentFile = sParam.currentLocation;
                currentSize
                  = "scanning DOM nodes: "
                  + Math.floor(100 * sParam.treeScanner.progress) + "%";
                break;
              }
              case UnMHTSave.STATUS_OK:
              case UnMHTSave.STATUS_NG: {
                break;
              }

              case UnMHTSave.STATUS_FILE_NET: {
                if (sParam.fileSize) {
                  currentFile = sParam.currentLocation;
                  currentSize
                    = sParam.recvSize + " bytes / " + sParam.fileSize + " bytes";
                  break;
                }
                /* FALLTHROUGH */
              }
              default: {
                currentFile = sParam.currentLocation;
                currentSize
                  = sParam.recvSize + " bytes / "
                  + (sParam.fileSize || "unknown") + " bytes";
              }
            }
          }
          
          yield this._addImportFiles(sParam);
          
          switch (sParam.status) {
            case UnMHTSave.STATUS_FILE_NET: {
              if (sParam.fileSize) {
                let progress = sParam.recvSize / sParam.fileSize;
                if (sParam.isDocument) {
                  progress = 0.5 + progress * 0.5;
                }
                na_progress += progress;
              }
              
              conns ++;
              if (connsPerServer.has(sParam.host)) {
                connsPerServer.set(sParam.host,
                                   connsPerServer.get(sParam.host) + 1);
              } else {
                connsPerServer.set(sParam.host, 1);
              }
              
              if (sParam.recvSize == 0) {
                if (sParam.loaderStatus == arNetFileLoader.STATUS_NA ||
                    sParam.loaderStatus == arNetFileLoader.STATUS_CACHE) {
                  if (now > sParam.startTime + UnMHTSave.CACHE_TIMEOUT * 1000) {
                    sParam.startTime = now;
                    sParam.loaderStatus = arNetFileLoader.STATUS_NET;
                    
                    sParam.loadContext.skipCache();
                  }
                } else if (sParam.loaderStatus == arNetFileLoader.STATUS_NET) {
                  if (now > sParam.startTime + UnMHTSave.RECEIVE_TIMEOUT * 1000) {
                    try {
                      sParam.loadContext.cancel();
                    } catch (e) {
                    }
                    
                    sParam.status = UnMHTSave.STATUS_NG;
                  }
                }
              }
              break;
            }
            
            case UnMHTSave.STATUS_DOC_CUR_NA: {
              sParam.status = UnMHTSave.STATUS_DOC_CUR_FAVICON_START;
              break;
            }
            case UnMHTSave.STATUS_DOC_CUR_FAVICON_START: {
              sParam.status = UnMHTSave.STATUS_DOC_CUR_FAVICON_WAIT;
              this._startGetFavicon(sParam);
            }
            case UnMHTSave.STATUS_DOC_CUR_FAVICON_WAIT: {
              break;
            }
            case UnMHTSave.STATUS_DOC_CUR_SERIAL_START: {
              sParam.status = UnMHTSave.STATUS_DOC_CUR_SERIAL_WAIT;
              this._startSerializeDocument(sParam);
              break;
            }
            case UnMHTSave.STATUS_DOC_CUR_SERIAL_WAIT: {
              break;
            }
            case UnMHTSave.STATUS_DOC_CUR_GATHER_START: {
              sParam.status = UnMHTSave.STATUS_DOC_CUR_GATHER_WAIT;
              yield this._startGatherDocument(sParam);
              break;
            }
            case UnMHTSave.STATUS_DOC_CUR_GATHER_WAIT: {
              let progress = sParam.treeScanner.progress;
              na_progress += 0.5 + progress * 0.5;
              break;
            }
            
            case UnMHTSave.STATUS_DOC_ORI_NA: {
              sParam.status = UnMHTSave.STATUS_DOC_ORI_FAVICON_START;
              break;
            }
            case UnMHTSave.STATUS_DOC_ORI_FAVICON_START: {
              sParam.status = UnMHTSave.STATUS_DOC_ORI_FAVICON_WAIT;
              this._startGetFavicon(sParam);
              break;
            }
            case UnMHTSave.STATUS_DOC_ORI_FAVICON_WAIT: {
              break;
            }
            case UnMHTSave.STATUS_DOC_ORI_GATHER_START: {
              sParam.status = UnMHTSave.STATUS_DOC_ORI_GATHER_WAIT;
              yield this._startGatherDocument(sParam);
              break;
            }
            case UnMHTSave.STATUS_DOC_ORI_GATHER_WAIT: {
              let progress = sParam.treeScanner.progress;
              na_progress += progress * 0.5;
              break;
            }

            case UnMHTSave.STATUS_MAIL_CUR_MAIL_START: {
              sParam.status = UnMHTSave.STATUS_MAIL_CUR_MAIL_WAIT;
              this._startGetMail(sParam);
              break;
            }
            case UnMHTSave.STATUS_MAIL_CUR_MAIL_WAIT: {
              break;
            }

            case UnMHTSave.STATUS_MAIL_ORI_MAIL_START: {
              sParam.status = UnMHTSave.STATUS_MAIL_ORI_MAIL_WAIT;
              this._startGetMail(sParam);
              break;
            }
            case UnMHTSave.STATUS_MAIL_ORI_MAIL_WAIT: {
              break;
            }
            case UnMHTSave.STATUS_MAIL_ORI_NA: {
              sParam.status = UnMHTSave.STATUS_MAIL_ORI_GATHER_START;
              break;
            }
            case UnMHTSave.STATUS_MAIL_ORI_GATHER_START: {
              sParam.status = UnMHTSave.STATUS_MAIL_ORI_GATHER_WAIT;
              yield this._startGatherDocument(sParam);
              break;
            }
            case UnMHTSave.STATUS_MAIL_ORI_GATHER_WAIT: {
              let progress = sParam.treeScanner.progress;
              na_progress += progress;
              break;
            }
            
            case UnMHTSave.STATUS_REMOTE_NA: {
              sParam.status = UnMHTSave.STATUS_REMOTE_GATHER_START;
              break;
            }
            case UnMHTSave.STATUS_REMOTE_GATHER_START: {
              sParam.status = UnMHTSave.STATUS_REMOTE_GATHER_WAIT;
              yield this._startGatherDocument(sParam);
              break;
            }
            case UnMHTSave.STATUS_REMOTE_GATHER_WAIT: {
              let progress = sParam.treeScanner.progress;
              na_progress += progress;
              break;
            }
            
            case UnMHTSave.STATUS_OK: {
              ok ++;
              break;
            }
            case UnMHTSave.STATUS_NG: {
              ng ++;
              break;
            }
            case UnMHTSave.STATUS_IGNORE: {
              ok ++;
              break;
            }
          }
        }
        
        for (let i = 0, len = this._saveParams.length; i < len; i ++) {
          let sParam = this._saveParams[i];
          
          switch (sParam.status) {
            case UnMHTSave.STATUS_FILE_NA: {
              if (!sParam.host) {
                try {
                  let uri = Services.io.newURI(sParam.currentLocation, null, null);
                  sParam.host = uri.host;
                } catch (e) {
                  sParam.host = "";
                }
                if (!sParam.host) {
                  sParam.host = "unknown";
                }
              }
              
              if (conns >= this._maxConns) {
                continue;
              }
              if (connsPerServer.has(sParam.host) &&
                  connsPerServer.get(sParam.host) >= this._maxConnsPerServer) {
                continue;
              }
              
              if (connsPerServer.has(sParam.host)) {
                connsPerServer.set(sParam.host,
                                   connsPerServer.get(sParam.host) + 1);
              } else {
                connsPerServer.set(sParam.host, 1);
              }
              conns ++;
              
              this._startFileLoad(sParam);
              break;
            }
          }
        }
        
        let na = this._saveParams.length - (ok + ng);
        if (na > 0) {
          this.progress = (ok + na_progress) / this._saveParams.length;
        } else {
          this.progress = 1;
        }
        if (this._listener) {
          this._listener.onProgress(this, currentFile, currentSize, ok, na, ng);
        }
        
        if (na == 0) {
          break;
        }
        
        yield arTimer.wait(100);
      };
      
      this._storeToFile();
    }.bind(this)).then(null, function(e) {
      Cu.reportError(e + "\n" + e.stack);
    });
  },
  
  /**
   * mht で保存 - ファイルの保存
   */
  _storeToFile: function() {
    if (this._isCanceled) {
      this._isStopped = true;
      if (this._listener) {
        this._listener.onCancel(this);
      }
      this._deferred.reject(UnMHTSave.RESULT_CANCEL);
      this.destruct();
      return;
    }
    
    if (this._saveParams
        .some(function(sParam)
              sParam.isRequired &&
              sParam.status == UnMHTSave.STATUS_NG)) {
      this._isStopped = true;
      if (this._listener) {
        this._listener.onError(this, UnMHTSave.ERROR_REQUIRED_FAILED, null);
      }
      this._deferred.reject(UnMHTSave.RESULT_ERROR_REQUIRED_FAILED);
      this.destruct();
      return;
    }
    
    let setPartData = function(part) {
      if (part.isMultipart) {
        part.parts = part.parts.filter(function(p) setPartData(p));
        return true;
      }
      
      let sParam = part.sParam;
      if (!sParam) {
        return true;
      }
      
      if (sParam.status == UnMHTSave.STATUS_NG ||
          sParam.status == UnMHTSave.STATUS_IGNORE) {
        part.destruct();
        return false;
      }
      
      part.contentTransferEncoding
        = sParam.isText ? "quoted-printable" : "base64";
      part.charset = sParam.charset;
      part.mimetype = sParam.mimetype;
      
      let location = sParam.location;
      if (sParam.cid) {
        part.contentID = sParam.cid;
      }
      
      let storeContentID = sParam.storeContentID;
      if (UnMHTMailPrivacy.isMailURL(location)) {
        if (this._mailPart) {
          let re = location.match(/^.+\?part=([^&]+).*/);
          if (re) {
            let partID = re[1];
            let sourcePart
              = partID.split(/\./)
              .map(function(i) parseInt(i, 10))
              .slice(1)
              .reduce(function(p, i) {
                if (!p) {
                  return null;
                }
                if (!p.parts) {
                  return null;
                }
                
                if (i - 1 >= p.parts.length) {
                  return null;
                }
                return p.parts[i - 1];
              }, this._mailPart);
            if (sourcePart &&
                sourcePart.contentID) {
              storeContentID = true;
              part.contentID = sourcePart.contentID;
            }
          }
        }
        
        location = UnMHTMailPrivacy.hideMailURL(location, this._location);
      }
      part.contentLocation = location;
      
      part.body = sParam.content;
      
      let param = new arMIMEParam(part.mimetype);
      if (part.charset) {
        param.params.set("charset", part.charset);
      }
      part.setContentType(param);
      part.addField("Content-Transfer-Encoding",
                    part.contentTransferEncoding);
      
      if (!part.contentLocation ||
          storeContentID) {
        part.addField("Content-ID",
                      "<" + part.contentID + ">");
      }
      
      if (part.contentLocation) {
        part.addField("Content-Location",
                      part.contentLocation);
      }
      
      return true;
    }.bind(this);
    setPartData(this._topPart);
    
    if (this._startPart &&
        (this._startPart.mimetype ||
         (this._startPart.sParam &&
          this._startPart.sParam.mimetype))) {
      if (this._startPart.mimetype) {
        this._topPart.startMimetype = this._startPart.mimetype;
      } else {
        this._topPart.startMimetype = this._startPart.sParam.mimetype;
      }
      
      let param = new arMIMEParam();
      param.type = "multipart";
      param.subtype = "related";
      param.params.set("boundary", this._topPart.boundary);
      param.params.set("type", this._topPart.startMimetype);
      this._topPart.setContentType(param);
    } else if (this._isMixed) {
      this._topPart.startMimetype = "";
      
      let param = new arMIMEParam();
      param.type = "multipart";
      param.subtype = "mixed";
      param.params.set("boundary", this._topPart.boundary);
      this._topPart.setContentType(param);
    } else if (this._isAlternative) {
      this._topPart.startMimetype = "";
      
      let param = new arMIMEParam();
      param.type = "multipart";
      param.subtype = "alternative";
      param.params.set("boundary", this._topPart.boundary);
      this._topPart.setContentType(param);
    } else {
      /* **** FIXME: この分岐は本当は来てはいけない **** */
      
      if (this._startPart) {
        Cu.reportError("FIXME: UnMHT: MIME-Type for startPart is not found for multipart/related.");
      } else {
        Cu.reportError("FIXME: UnMHT: startPart is not found for multipart/related.");
      }
      
      let param = new arMIMEParam();
      param.type = "multipart";
      param.subtype = "related";
      param.params.set("boundary", this._topPart.boundary);
      this._topPart.setContentType(param);
    }
    this._topPart.removeField("X-UnMHT-tabs");
    if (this._isTabs) {
      this._topPart.addField("X-UnMHT-tabs", "true");
    }
    this._topPart.removeField("X-UnMHT-Tabs-List");
    if (this._tabs) {
      this._topPart.addField("X-UnMHT-Tabs-List", this._tabs.map(function(cid) ("<" + cid + ">")).join(","));
    }
    this._topPart.removeField("X-UnMHT-Save-State");
    switch (this._saveState) {
      case UnMHTSave.SAVESTATE_CURRENT: {
        this._topPart.addField("X-UnMHT-Save-State", "Current-State");
        break;
      }
      case UnMHTSave.SAVESTATE_CURRENT_NOJS: {
        this._topPart.addField("X-UnMHT-Save-State", "Current-State-Without-Script");
        break;
      }
      case UnMHTSave.SAVESTATE_ORIGINAL: {
        this._topPart.addField("X-UnMHT-Save-State", "Original-File");
        break;
      }
    }
    
    let fstream = arFileUtil.openOutputStream(this._targetFile);
    if (!fstream) {
      this._isStopped = true;
      if (this._listener) {
        this._listener.onError(this, UnMHTSave.ERROR_WRITE_FILE, null);
      }
      this._deferred.reject(UnMHTSave.RESULT_ERROR_WRITE_FILE);
      this.destruct();
      return;
    }
    
    arMIMEEncoder.encodeMessage(this._topPart, fstream);
    
    fstream.close();
    
    this._addDownloadHistory();
    
    this._isStopped = true;
    
    let failedList
      = this._saveParams
      .filter(function(sParam) sParam.status == UnMHTSave.STATUS_NG)
      .map(function(sParam) sParam.currentLocation);
    
    this.progress = 1;
    if (this._listener) {
      this._listener.onSave(this, failedList);
    }
    this._deferred.resolve([this._targetFile, failedList]);
    
    this.destruct();
  },
  
  /**
   * フィールドを追加する
   *
   * @param   {Array} fieldList
   *          ヘッダ
   *            [[string フィールド名, string フィールド値], ...]
   * @param   {boolean} storeAll
   *          UnMHT が使用するフィールドも無視しない
   */
  _addFieldList: function(fieldList, storeAll) {
    fieldList.forEach(function([name, value]) {
      this.addField(name, value, storeAll);
    }.bind(this));
  },
  
  /**
   * ダウンロード履歴に登録する
   */
  _addDownloadHistory: function() {
    if (this._hasDownload) {
      return;
    }
    
    try {
      let source = Services.io.newURI(this._location, null, null);
      let target = Services.io.newFileURI(this._targetFile);
      let startTime = null;
      
      Cc["@mozilla.org/browser/download-history;1"]
        .getService(Ci.nsIDownloadHistory)
        .addDownload(source, null, startTime, target);
    } catch (e) {
      if (arAppInfo.isFennec) {
        /* ==== COMPAT: Fennec では nsIDownloadManager が deprecated でない？ ==== */
        try {
          let source = Services.io.newURI(this._location, null, null);
          let target = Services.io.newFileURI(this._targetFile);
          let startTime = null;
          
          let type = Ci.nsIDownloadManager.DOWNLOAD_TYPE_DOWNLOAD;
          let displayName = this._targetFile.leafName;
          let mimeInfo
            = Cc["@mozilla.org/mime;1"].getService(Ci.nsIMIMEService)
            .getFromTypeAndExtension("message/rfc822", "mht");
          let tempFile = null;
          let cancelable = {
            QueryInterface: XPCOMUtils.generateQI([Ci.nsICancelable]),
            cancel: function(reason) {
            }
          };
          let isPrivate = false;
          try {
            let context = {};
            Cu.import("resource://gre/modules/PrivateBrowsingUtils.jsm", context);
            isPrivate = context.PrivateBrowsingUtils.isWindowPrivate(this._window);
          } catch (e) {
          }

          let request = null;
          /* 保存した直後にファイルが消える事もありうるので
           * 消えている場合を想定する */
          let fileSize
            = this._targetFile.exists()
            ? this._targetFile.fileSize
            : 0;
          let maxFileSize = fileSize;
          
          let download
            = Services.downloads.addDownload(type,
                                             source, target,
                                             displayName, mimeInfo,
                                             startTime, tempFile, cancelable,
                                             isPrivate);
          
          download.onProgressChange(null, request, fileSize, maxFileSize,
                                    fileSize, maxFileSize);

          download.onStateChange(null, request,
                                 Ci.nsIWebProgressListener.STATE_STOP
                                 | Ci.nsIWebProgressListener.STATE_IS_NETWORK,
                                 Cr.NS_OK);
        } catch (e) {
          Cu.reportError(e + "\n" + e.stack);
        }
      } else {
        Cu.reportError(e + "\n" + e.stack);
      }
    }
  },
  
  /**
   * トップレベルのパートを作成する
   *
   * @param   {string} subject
   *          ファイルの subject
   */
  _prepareTopPart: function(subject) {
    let part = new arMIMEPart();
    part.isMultipart = true;
    part.boundary = arMIMEUtil.createUniqueBoundary();
    part.addField("From",
                  "<Saved by UnMHT Ver." + UnMHTVersion + ">");
    part.addField("Subject", subject);
    part.addField("Date",
                  arMIMEEncoder.encodeDate(this._saveDate));
    part.addField("MIME-Version", "1.0");
    part.parts = [];
    
    this._topPart = part;
  },
  
  /**
   * ドキュメントをシリアライズする
   *
   * @param   {UnMHTSaveParam} sParam
   *          対象の保存情報
   */
  _startSerializeDocument: function(sParam) {
    Task.spawn(function() {
      if (sParam.targetDocument.doctype) {
        sParam.content
          += "<!DOCTYPE "
          + sParam.targetDocument.doctype.name
          + (sParam.targetDocument.doctype.publicId
             ? (" PUBLIC \""
                + UnMHTHTMLUtil.escapeEntity(sParam.targetDocument.doctype.publicId)
                + "\"")
             : "")
          + (sParam.targetDocument.doctype.systemId
             ? (" \""
                + UnMHTHTMLUtil.escapeEntity(sParam.targetDocument.doctype.systemId)
                + "\"")
             : "")
          + ">\n";
      }
      
      let tmpNodes = [];
      let ignoreNodes = [];
      
      let nodes = sParam.targetDocument.getElementsByTagName("head");
      if (nodes.length > 0) {
        let head = nodes[0];
        
        if (sParam.faviconData &&
            !Array.some(head.getElementsByTagName("link"),
                        function(node)
                        (node.hasAttribute("rel") &&
                         /icon/i.test(node.getAttribute("rel"))))) {
          let node = sParam.targetDocument.createElement("link");
          node.setAttribute("rel", "shortcut icon");
          node.setAttribute("href", sParam.faviconData);
          if (sParam.faviconType) {
            node.setAttribute("type", sParam.faviconType);
          }
          head.insertBefore(node, head.firstChild);
          tmpNodes.push(node);
        }
        
        if (sParam.targetDocument.characterSet) {
          let nodes = head.getElementsByTagName("meta");
          
          let oldMetas
            = Array.slice(nodes)
            .filter(function(node)
                    (node.hasAttribute("http-equiv") &&
                     /Content-Type/i.test(node.getAttribute("http-equiv")) &&
                     node.hasAttribute("content") &&
                     /charset=/i.test(node.getAttribute("content"))));
          
          let newMetas
            = Array.slice(nodes)
            .filter(function(node) node.hasAttribute("charset"));
          
          let charset = sParam.targetDocument.characterSet;
          
          let node = sParam.targetDocument.createElement("meta");
          if (newMetas.length) {
            node.setAttribute("charset", charset);
          } else {
            node.setAttribute("http-equiv", "Content-Type");
            node.setAttribute("content", "text/html; charset=" + charset);
          }
          head.insertBefore(node, head.firstChild);
          tmpNodes.push(node);
          ignoreNodes = ignoreNodes.concat(oldMetas, newMetas);
        }
        
        if (!sParam.hasBaseNode) {
          let node = sParam.targetDocument.createElement("base");
          node.setAttribute("href", sParam.baseDir);
          head.appendChild(node);
          tmpNodes.push(node);
        }
      }
      
      let markerID = "marker." + arPseudoID.gen() + "@unmht.org";
      let params = {
        noScript: sParam.saveState == UnMHTSave.SAVESTATE_CURRENT_NOJS,
        isMail: sParam.isMail,
        saveCanvas: sParam.saveState == UnMHTSave.SAVESTATE_CURRENT_NOJS,
        ignoreNodes: ignoreNodes,
        frameIDs: sParam.frameIDs,
        markerID: markerID
      };
      sParam.content
        += UnMHTDOMConverter.toPlainText(this._window,
                                         sParam.location,
                                         sParam.targetDocument.documentElement,
                                         sParam.targetSelection, params);
      
      tmpNodes.forEach(function(node) {
        node.parentNode.removeChild(node);
      });
      
      if (sParam.mailHeaderDisplay) {
        let divStyles = [];
        divStyles.push("font-size: 0.8em");
        divStyles.push("background-color: white");
        divStyles.push("color: black");
        divStyles.push("padding-bottom: 1em");
        divStyles.push("text-align: left");

        let box
          = "<div style=\"" + divStyles.join(";") + ";\">"
          + sParam.mailHeaderDisplay.map(function([name, value]) {
            return "<b>" + UnMHTHTMLUtil.escapeEntity(name) + ": </b>"
              + UnMHTHTMLUtil.escapeEntity(value);
          }).join("<br>")
          + "</div>";
        
          sParam.content
            = UnMHTSave.modifyBody(sParam.content,
                                   box + "<div style=\"position: relative\">",
                                   "</div>",
                                   markerID);
      }
      
      if (sParam.attachments) {
        let box
          = "<div>"
          + "<hr>"
          + "<ul>"
          + sParam.attachments.map(function([name, url]) {
            url = UnMHTMailPrivacy.hideMailURL(url, sParam.location);
            
            return "<li>"
              + "<a href=\"" + UnMHTHTMLUtil.escapeEntity(url) + "\">"
              + UnMHTHTMLUtil.escapeEntity(name)
              + "</a>";
          }).join("")
          + "</ul>"
          + "</div>";
        
        sParam.content
          = UnMHTSave.modifyBody(sParam.content,
                                 "",
                                 box,
                                 markerID);
      }
      
      sParam.content
        = arUconv.fromUnicodeWithEntity(sParam.content, sParam.charset);
      
      if (sParam.eFileInfo) {
        sParam.content
          = UnMHTExtractor.unmodifyAttribute(sParam.eFileInfo, sParam.content);
        sParam.content
          = UnMHTExtractor.unmodifyCSS(sParam.eFileInfo, sParam.content);
      }
      
      UnMHTSave.modifyContent(sParam, sParam.savedFrom, this._saveDate, markerID);
      
      sParam.content
        = UnMHTDOMConverter.removeMarker(sParam.content, markerID);
    }.bind(this)).then(function() {
      sParam.status = UnMHTSave.STATUS_DOC_CUR_GATHER_START;
    }.bind(this), function(e) {
      if (UnMHTDocumentCategory.isAlive(sParam.targetDocument)) {
        Cu.reportError(e + "\n" + e.stack);
      }
      sParam.status = UnMHTSave.STATUS_NG;
    });
  },
  
  /**
   * ファイルを収集する
   *
   * @param   {UnMHTSaveParam} sParam
   *          対象の保存情報
   * @returns {Promise<boolean>}
   *          resolve: ダイアログ表示完了までの Promise
   */
  _startGatherDocument: function(sParam) {
    let deferred = Promise.defer();
    
    sParam.treeScanner
      = new UnMHTTreeScanner(this._window,
                             sParam.targetDocument.documentElement);
    
    Task.spawn(function() {
      let saveVideo = false, saveEmbed = false;
      
      if (["video", "audio", "source"].some(function(name) (sParam.targetDocument.getElementsByTagName(name).length > 0))) {
        saveVideo = yield this._checkSaveContents("video");
      }
      if (["embed", "object"].some(function(name) (sParam.targetDocument.getElementsByTagName(name).length > 0))) {
        saveEmbed = yield this._checkSaveContents("embed");
      }
      
      deferred.resolve(true);
      
      let domUtil
        = Cc["@mozilla.org/inspector/dom-utils;1"].getService(Ci.inIDOMUtils);
      let i = 0;
      let startTime = Date.now();
      
      let range = sParam.targetDocument.createRange();
      for (let node of sParam.treeScanner.gen) {
        i ++;
        if (i == 10) {
          i = 0;
          let now = Date.now();
          if (now > startTime + 100) {
            /* UI に処理を返すために 100ms 以上実行したら 10ms 待つ */
            yield arTimer.wait(10);
            startTime = now;
            
            if (this._isCanceled) {
              throw UnMHTSave.RESULT_CANCEL;
            }
          }
        }
        
        if (node instanceof Ci.nsIDOMElement) {
          this._gatherFilesFromNode(
            sParam.targetDocument,
            sParam.location, sParam.savedFrom, sParam.saveState, node,
            sParam.baseDir, sParam.depth,
            sParam.treeScanner.isHead ? null: sParam.targetSelection,
            sParam.referrer, sParam.isMail,
            sParam.status == UnMHTSave.STATUS_REMOTE_GATHER_WAIT,
            sParam.frameIDs, saveVideo, saveEmbed);
        } else if (node instanceof Ci.nsIDOMText) {
          range.selectNode(node);
          let fontList = domUtil.getUsedFontFaces(range);
          for (let j = 0, length = fontList.length; j < length; j ++) {
            let font = fontList.item(j);
            if (!font.URI) {
              continue;
            }
            this._checkAndPush(font.URI, Ci.nsIContentPolicy.TYPE_FONT,
                               sParam.targetDocument, 0, sParam.referrer,
                               null,
                               sParam.status == UnMHTSave.STATUS_REMOTE_GATHER_WAIT);
          }
        }
      }

      switch (sParam.status) {
        case UnMHTSave.STATUS_DOC_CUR_GATHER_WAIT: {
          sParam.status = UnMHTSave.STATUS_OK;
          break;
        }
        case UnMHTSave.STATUS_DOC_ORI_GATHER_WAIT: {
          sParam.status = UnMHTSave.STATUS_FILE_NA;
          break;
        }
        case UnMHTSave.STATUS_MAIL_ORI_GATHER_WAIT: {
          /* 完了したらこのパートは不要 */
          sParam.status = UnMHTSave.STATUS_IGNORE;
          break;
        }
        case UnMHTSave.STATUS_REMOTE_GATHER_WAIT: {
          /* 完了したらこのパートは不要 */
          sParam.status = UnMHTSave.STATUS_IGNORE;
          break;
        }
      }
    }.bind(this)).then(null, function(e) {
      if (e != UnMHTSave.RESULT_CANCEL &&
          UnMHTDocumentCategory.isAlive(sParam.targetDocument)) {
        Cu.reportError(e + "\n" + e.stack);
      }
      sParam.status = UnMHTSave.STATUS_NG;
    }.bind(this));
    
    return deferred.promise;
  },
  
  /**
   * favicon の取得を開始する
   *
   * @param   {UnMHTSaveParam} sParam
   *          対象の保存情報
   */
  _startGetFavicon: function(sParam) {
    try {
      let faviconService
        = Cc["@mozilla.org/browser/favicon-service;1"]
        .getService(Ci.nsIFaviconService);
      
      let pageURI = Services.io.newURI(sParam.savedFrom, null, null);
      
      let listener = {
        QueryInterface: XPCOMUtils.generateQI([Ci.nsIFaviconDataCallback]),
        
        /**
         * favicon のデータが到着したイベント
         *
         * @param   {nsIURI} uri
         *          favicon の URI
         * @param   {number} dataLen
         *          データの長さ
         * @param   {string} data
         *          データ
         * @param   {string} mimeType
         *          MIME-Type
         */
        onComplete: function(uri, dataLen, data, mimeType) {
          if (!uri) {
          } else if (uri.scheme == "data") {
            sParam.faviconData = uri.spec;
            if (mimeType) {
              sParam.faviconType = mimeType;
            } else {
              let re = sParam.faviconData.match(/^data:(image\/[^;,]+)/);
              if (re) {
                sParam.faviconType = re[1];
              }
            }
          } else if (dataLen && mimeType) {
            let bindata
              = Array.map(data, function(c) String.fromCharCode(c))
              .join("");
            sParam.faviconData
              = "data:" + mimeType + ";base64,"
              + arMIMEEncoder.encodeBase64(bindata, false);
            sParam.faviconType = mimeType;
          }
          
          if (sParam.status == UnMHTSave.STATUS_DOC_CUR_FAVICON_WAIT) {
            sParam.status = UnMHTSave.STATUS_DOC_CUR_SERIAL_START;
          } else if (sParam.status == UnMHTSave.STATUS_DOC_ORI_FAVICON_WAIT) {
            sParam.status = UnMHTSave.STATUS_DOC_ORI_GATHER_START;
          }
        }
      };
      
      faviconService.getFaviconDataForPage(pageURI, listener);
    } catch (e) {
      if (sParam.status == UnMHTSave.STATUS_DOC_CUR_FAVICON_WAIT) {
        sParam.status = UnMHTSave.STATUS_DOC_CUR_SERIAL_START;
      } else if (sParam.status == UnMHTSave.STATUS_DOC_ORI_FAVICON_WAIT) {
        sParam.status = UnMHTSave.STATUS_DOC_ORI_GATHER_START;
      }
    }
  },
  
  /**
   * ファイルの取得を開始する
   *
   * @param   {UnMHTSaveParam} sParam
   *          対象の保存情報
   * @param   {number} now
   *          現在時刻
   */
  _startFileLoad: function(sParam, now) {
    sParam.startTime = now;
    sParam.status = UnMHTSave.STATUS_FILE_NET;
    
    let uri = null;
    let referrer = null;
    
    try {
      uri = Services.io.newURI(sParam.currentLocation, null, null);
    } catch (e) {
      sParam.status = UnMHTSave.STATUS_NG;
      return;
    }
    if (sParam.referrer) {
      try {
        referrer = Services.io.newURI(sParam.referrer, null, null);
      } catch (e) {
      }
    }
    sParam.loadContext
      = arNetFileLoader.createContext(uri, referrer, true, sParam);
    sParam.loadContext.start();
  },

  /**
   * メールの取得を開始する
   *
   * @param   {UnMHTSaveParam} sParam
   *          対象の保存情報
   */
  _startGetMail: function(sParam) {
    Task.spawn(function() {
      let result = yield arNetFileLoader.loadData(sParam.messageURI, null, true);
      
      let mailPart = null;
      if (result.status != arNetFileLoader.STATUS_NG) {
        mailPart
          = arMIMEDecoder.decodeMessage(result.data.replace(/\r\n|\r|\n/g, "\r\n"),
                                        true);
      }
      
      if (mailPart &&
          UnMHTPref.getPref("save.header.mail.original_field")) {
        this._addFieldList(mailPart.fieldList, true);
      }
      
      if (sParam.status == UnMHTSave.STATUS_MAIL_CUR_MAIL_WAIT) {
        sParam.status = UnMHTSave.STATUS_DOC_CUR_NA;
        this._mailPart = mailPart;
      } else if (sParam.status == UnMHTSave.STATUS_MAIL_ORI_MAIL_WAIT) {
        if (mailPart) {
          mailPart.fieldList
            = mailPart.fieldList
            .filter(function([name, value]) {
              let n = name.toLowerCase();
              return (n == "content-type" ||
                      n == "content-transfer-encoding");
            });
          mailPart.createFieldsFromList();
          
          this._topPart.parts.push(mailPart);
          
          sParam.status = UnMHTSave.STATUS_MAIL_ORI_NA;
        } else {
          sParam.status = UnMHTSave.STATUS_NG;
        }
      }
    }.bind(this));
  }
};

/**
 * タブを MHT で保存
 */
let UnMHTSave = {
  /**
   * ファイルの取得状況
   * @type {number}
   */
  /* 元のファイル (通常のファイル) */
  STATUS_FILE_NA:               0x0001, /* 未取得 */
  STATUS_FILE_NET:              0x0011, /* ネットワーク取得中 */
  
  /* 現在の状態 (ドキュメント) */
  STATUS_DOC_CUR_NA:            0x0101, /* 未取得 */
  STATUS_DOC_CUR_FAVICON_START: 0x0111, /* favicon 取得開始 */
  STATUS_DOC_CUR_FAVICON_WAIT:  0x0112, /* favicon 取得中 */
  STATUS_DOC_CUR_SERIAL_START:  0x0121, /* シリアライズ開始 */
  STATUS_DOC_CUR_SERIAL_WAIT:   0x0122, /* シリアライズ中 */
  STATUS_DOC_CUR_GATHER_START:  0x0131, /* ファイル回収開始 */
  STATUS_DOC_CUR_GATHER_WAIT:   0x0132, /* ファイル回収中 */
  
  /* 元のファイル (ドキュメント) */
  STATUS_DOC_ORI_NA:            0x0201, /* 未取得 */
  STATUS_DOC_ORI_FAVICON_START: 0x0211, /* favicon 取得開始 */
  STATUS_DOC_ORI_FAVICON_WAIT:  0x0212, /* favicon 取得中 */
  STATUS_DOC_ORI_GATHER_START:  0x0221, /* ファイル回収開始 */
  STATUS_DOC_ORI_GATHER_WAIT:   0x0222, /* ファイル回収中
                                         * 次は STATUS_FILE_NA */
  
  /* メールデータ取得中 (現在の状態) */
  STATUS_MAIL_CUR_MAIL_START:   0x0301, /* 現在の状態 - メール取得開始 */
  STATUS_MAIL_CUR_MAIL_WAIT:    0x0302, /* 現在の状態 - メール取得中
                                         * 次は STATUS_DOC_CUR_NA */

  /* メールデータ取得中 (元のファイル) */
  STATUS_MAIL_ORI_MAIL_START:   0x0401, /* 元のファイル - メール取得開始 */
  STATUS_MAIL_ORI_MAIL_WAIT:    0x0402, /* 元のファイル - メール取得中 */
  STATUS_MAIL_ORI_NA:           0x0411, /* 元のファイル - ファイル未回収 */
  STATUS_MAIL_ORI_GATHER_START: 0x0421, /* 元のファイル - ファイル回収開始 */
  STATUS_MAIL_ORI_GATHER_WAIT:  0x0422, /* 元のファイル - ファイル回収中
                                         * 次は STATUS_IGNORE */

  /* リモートファイル取得中 */
  STATUS_REMOTE_NA:             0x0501, /* ファイル未回収 */
  STATUS_REMOTE_GATHER_START:   0x0511, /* ファイル回収開始 */
  STATUS_REMOTE_GATHER_WAIT:    0x0512, /* ファイル回収中
                                         * 次は STATUS_IGNORE */
  
  STATUS_OK:                    0x1001, /* 取得完了 */
  STATUS_NG:                    0x1002, /* 取得失敗 */
  STATUS_IGNORE:                0x1003, /* このパートを無視する */
  
  /**
   * エラー定数
   * @type {number}
   */
  ERROR_WRITE_FILE:          0x0001, /* ファイル書き込み失敗 */
  ERROR_REQUIRED_FAILED:     0x0002, /* 必須ファイル取得失敗 */
  
  /**
   * 保存結果定数
   * @type {number}
   */
  RESULT_OK:                    0x00000000, /* 成功 */
  RESULT_FAIL:                  0x10000001, /* 一般エラー */
  RESULT_CANCEL:                0x10000002, /* ユーザによるキャンセル */
  RESULT_ERROR_ADD_DOCUMENT:    0x10000101, /* ファイル書き込み失敗 */
  RESULT_ERROR_WRITE_FILE:      0x10000102, /* ドキュメント追加失敗 */
  RESULT_ERROR_REQUIRED_FAILED: 0x10000103, /* 必須ファイル取得失敗 */
  
  /**
   * 保存状態のマスク
   * @type {number}
   */
  SAVESTATE_MASK_ALL:       0xffff, /* 全て */
  SAVESTATE_MASK_ORIGINAL:  0x000f, /* 元のファイル */
  SAVESTATE_MASK_CURRENT:   0x00f0, /* 現在の状態 */

  /**
   * 保存状態
   * @type {number}
   */
  SAVESTATE_ORIGINAL:       0x0001, /* 元のファイル */
  SAVESTATE_CURRENT:        0x0010, /* 現在の状態 */
  SAVESTATE_CURRENT_NOJS:   0x0020, /* 現在の状態 (スクリプト削除) */
  SAVESTATE_UNKNOWN:        0x0000, /* 不明 */

  saveStateList: [],   /** 保存状態の数値表現と文字列表現のリスト
                        *  @type {Array}
                        *   [[number 数値表現, string 文字列表現], ...] */
  
  MAX_IMPORT_DEPTH: 10, /** 最大の import の深さ
                         *  @type {number} */
  
  CACHE_TIMEOUT: 5,     /** キャッシュ取得のタイムアウト
                         *  @type {number} */
  RECEIVE_TIMEOUT: 15,  /** ネットワーク取得のタイムアウト
                         *  @type {number} */
  
  /**
   * 初期化処理
   */
  init: function() {
    this._contentPolicy
      = Cc["@mozilla.org/layout/content-policy;1"]
      .getService(Ci.nsIContentPolicy);
    
    this.saveStateList = [
      [this.SAVESTATE_CURRENT,      "current"],
      [this.SAVESTATE_CURRENT_NOJS, "curnojs"],
      [this.SAVESTATE_ORIGINAL,     "original"]
    ];
  },
  
  /**
   * 保存コンテキストを作成する
   *
   * @param   {nsIDOMWindow} window
   *          実行中のウィンドウ
   * @param   {nsIFile} targetFile
   *          対象のファイル,
   * @param   {string} location
   *          メールを保存する際のルートドキュメントの URI
   * @param   {string} subject
   *          ファイルの subject
   * @param   {nsIDownloadProgressListener} downloadListener
   *          ダウンロードのイベントリスナ
   * @param   {number} saveState
   *          保存状態の数値表現
   * @returns {UnMHTSaveContext}
   *          保存コンテキスト
   */
  createContext: function(window, targetFile, location, subject,
                          downloadListener, saveState) {
    return new UnMHTSaveContext(window, targetFile, location, subject,
                                downloadListener, saveState);
  },
  
  /**
   * URI がロード可能かどうか調べる
   *
   * @param   {number} type
   *          nsIContentPolicy の対象の種類
   * @param   {string} location
   *          対象 URL
   * @param   {nsIDOMElement} context
   *          読み込み元の要素
   *          null の場合は type は Ci.nsIContentPolicy.TYPE_DOCUMENT 固定
   * @returns {boolean}
   *          ロード可能か
   */
  checkPolicy: function(type, location, context) {
    if (!context) {
      type = Ci.nsIContentPolicy.TYPE_DOCUMENT;
    }
    
    let uri;
    try {
      uri = Services.io.newURI(location, null, null);
    } catch (e) {
      return false;
    }
    let originURI = null;
    if (context) {
      let doc
        = (context instanceof Ci.nsIDOMDocument)
        ? context
        : context.ownerDocument;
      if (doc) {
        try {
          originURI = Services.io.newURI(doc.URL, null, null);
        } catch (e) {
        }
      }
    }
    
    try {
      let result
        = this._contentPolicy.shouldLoad(type, uri, originURI, context,
                                         null, null);
      return (result == Ci.nsIContentPolicy.ACCEPT);
    } catch (e) {
    }
    
    return true;
  },
  
  /**
   * URL をコメントとして挿入する
   *
   * @param   {string} content
   *          ファイルの内容
   * @param   {string} location
   *          ドキュメントの URL
   * @returns {string}
   *          ファイルの内容
   * @param   {string} markerID
   *          マーカーの ID
   */
  insertComment: function(content, location, markerID) {
    if (UnMHTScheme.isUnMHTURI(location)) {
      location = UnMHTScheme.deUnMHTURI(location);
    }
    
    let comment = "<!-- Saved from " + arUconv.fromUnicodeWithEntity(location, "ascii") + " -->";
    
    if (markerID) {
      let result;
      result = UnMHTDOMConverter.appendToMarker(content, markerID,
                                                "html", "first",
                                                comment);
      if (result.ok) {
        return result.text;
      }
      
      result = UnMHTDOMConverter.appendToMarker(content, markerID,
                                                "head", "first",
                                                comment);
      if (result.ok) {
        return  result.text;
      }
      
      result = UnMHTDOMConverter.appendToMarker(content, markerID,
                                                "body", "first",
                                                comment);
      if (result.ok) {
        return result.text;
      }
      
      return comment + content;
    }
    else if (/<html[^>]*>/i.test(content)) {
      return content
        .replace(/<html[^>]*>/i,
                 function(matched) {
                   return matched + comment;
                 });
    } else if (/<head[^>]*>/i.test(content)) {
      return content
        .replace(/<head[^>]*>/i,
                 function(matched) {
                   return matched + comment;
                 });
    } else if (/<body[^>]*>/i.test(content)) {
      return content
        .replace(/<body[^>]*>/i,
                 function(matched) {
                   return matched + comment;
                 });
    }
    
    return comment + content;
  },

  /**
   * URL をリンクとして挿入する
   *
   * @param   {string} content
   *          ファイルの内容
   * @param   {string} location
   *          ドキュメントの URL
   * @param   {Date} saveDate
   *          保存日時
   * @param   {string} markerID
   *          マーカーの ID
   * @returns {string}
   *          ファイルの内容
   */
  insertLink: function(content, location, saveDate, markerID) {
    if (UnMHTScheme.isUnMHTURI(location)) {
      location = UnMHTScheme.deUnMHTURI(location);
    }
    
    let pos = UnMHTPref.getPref("save.insert.link.pos");
    
    let divStyles = [];
    if (pos == "bottom") {
      divStyles.push("clear: both");
      divStyles.push("margin-top: 2em");
    }
    divStyles.push("font-size: 0.8em");
    divStyles.push("border: 1px solid black");
    divStyles.push("background-color: white");
    divStyles.push("color: black");
    divStyles.push("padding: 0.5em");
    divStyles.push("text-align: center");
    
    let aStyles = [];
    aStyles.push("color: blue");
    aStyles.push("background-color: inherit");
    aStyles.push("text-decoration: underline");
    
    let link
      = "<div style=\"" + divStyles.join(";") + ";\">"
      + "Saved from <a class=\"unmht_link_to_original\" style=\""
      + aStyles.join(";") + "\" href=\""
      + arUconv.fromUnicodeWithEntity(UnMHTHTMLUtil.escapeEntity(location), "ascii") + "\">"
      + arUconv.fromUnicodeWithEntity(UnMHTHTMLUtil.escapeEntity(location), "ascii")
      + "</a>"
      + (UnMHTPref.getPref("save.insert.link.date")
         ? ("<br />" + arUconv.fromUnicodeWithEntity(UnMHTHTMLUtil.escapeEntity(saveDate.toString()), "ascii"))
         : "")
      + (UnMHTPref.getPref("save.insert.link.version")
         ? ("<br />" + "Saved by UnMHT Ver." + UnMHTVersion)
         : "")
      + "</div>";
    
    if (pos == "bottom") {
      content
        = UnMHTSave.modifyBody(content,
                               "",
                               link,
                               markerID);
    } else {
      content
        = UnMHTSave.modifyBody(content,
                               link + "<div style=\"position: relative\">",
                               "</div>",
                               markerID);
    }
    
    return content;
  },
  
  /**
   * ファイルを修正する
   *
   * @param   {UnMHTSaveParam} param
   *          保存情報
   * @param   {string} location
   *          対象のファイルの URI
   * @param   {Date} saveDate
   *          保存日時
   * @param   {string} markerID
   *          マーカーの ID
   */
  modifyContent: function(sParam, location, saveDate, markerID) {
    if (sParam.isHTML && !sParam.isIFrame) {
      if (UnMHTPref.getPref("save.insert.url")) {
        sParam.content = this.insertComment(sParam.content, location, markerID);
      }
      if (UnMHTPref.getPref("save.insert.link")) {
        if (!sParam.isLocal ||
            !UnMHTPref.getPref("save.insert.link.nolocal")) {
          sParam.content
            = this.insertLink(sParam.content, location, saveDate, markerID);
        }
      }
    }
  },
  
  /**
   * body の最初と最後にテキストを挿入する
   *
   * @param   {string} content
   *          対象のソース
   * @param   {string} firstText
   *          body の最初に挿入するテキスト
   * @param   {string} lastText
   *          body の最後に挿入するテキスト
   * @param   {string} markerID
   *          マーカーの ID
   */
  modifyBody: function(content, firstText, lastText, markerID) {
    if (markerID) {
      let result;
      result = UnMHTDOMConverter.appendToMarker(content, markerID, ["body:first"], firstText);
      if (result.ok) {
        content = result.text;
      }
      result = UnMHTDOMConverter.appendToMarker(content, markerID, ["body:last"], lastText);
      if (result.ok) {
        content = result.text;
      }
    }
    
    if (/<body[^>]*/i.test(content)) {
      content
        = content
        .replace(/<body[^>]*>/i,
                 function(matched) {
                   return matched + firstText;
                 })
        .replace(/<\/body>/i,
                 function(matched) {
                   return lastText + matched;
                 });
    }
    
    return content;
  },
  
  /**
   * CSS の階層が深い場合の警告を表示する
   *
   * @param   {nsIDOMWindow} window
   *          実行中のウィンドウ
   * @returns {Promise<object>}
   *          resolve: 今回の動作
   */
  showDeepCSSConfirm: function(window) {
    return Task.spawn(function() {
      let confirmURL = "chrome://unmht/content/confirm.xul";
      let features = "chrome,dialog,centerscreen";
      let arg = {
        title:    UnMHTStringBundle.get("save.import.limit.exceeded.title"),
        message1: UnMHTStringBundle.get("save.import.limit.exceeded.message"),
        message2: UnMHTStringBundle.get("save.import.limit.exceeded.message2"),
        message3: UnMHTStringBundle.get("save.import.limit.exceeded.message3"),
        accept:   UnMHTStringBundle.get("save.import.limit.exceeded.save"),
        cancel:   UnMHTStringBundle.get("save.import.limit.exceeded.ignore"),
        toall:    UnMHTStringBundle.get("save.import.limit.exceeded.do_not_ask_again"),
        focus:    "no"
      };
      let result = yield arWindowUtil.openDialog(window, confirmURL, "", features, arg);
      
      let action;
      
      if (result.action == "yes") {
        action = "save";
      } else if (result.action == "no") {
        action = "ignore";
      }
      
      if (result.applyToAll) {
        UnMHTPref.setPref("save.import.limit.exceeded.action", action);
      }
      
      throw new Task.Result(action);
    }.bind(this));
  },

  /**
   * 内容を保存するかどうかの警告を表示する
   *
   * @param   {nsIDOMWindow} window
   *          実行中のウィンドウ
   * @param   {number} type
   *          内容の種類
   *            UnMHTSave.SAVE_CONTENTS_*
   * @returns {Promise<object>}
   *          resolve: 今回の動作
   */
  showSaveContentsConfirm: function(window, type) {
    return Task.spawn(function() {
      let confirmURL = "chrome://unmht/content/confirm.xul";
      let features = "chrome,dialog,centerscreen";
      let arg = {
        title:    UnMHTStringBundle.get("save.contents." + type + ".title"),
        message1: UnMHTStringBundle.get("save.contents." + type + ".message"),
        message2: UnMHTStringBundle.get("save.contents." + type + ".message2"),
        accept:   UnMHTStringBundle.get("save.contents.save"),
        cancel:   UnMHTStringBundle.get("save.contents.ignore"),
        toall:    UnMHTStringBundle.get("save.contents.do_not_ask_again"),
        focus:    "yes"
      };
      let result = yield arWindowUtil.openDialog(window, confirmURL, "", features, arg);
      
      let action;
      
      if (result.action == "yes") {
        action = "save";
      } else if (result.action == "no") {
        action = "ignore";
      }
      
      if (result.applyToAll) {
        UnMHTPref.setPref("save.contents." + type, action);
      }
      
      throw new Task.Result(action);
    }.bind(this));
  },
  
  /**
   * 同名のファイルが存在した場合の警告を表示する
   *
   * @param   {nsIDOMWindow} window
   *          実行中のウィンドウ
   * @param   {nsIFile} targetFile
   *          同名のファイル
   * @param   {string} filenameBase
   *          ファイル名の拡張子以外
   * @param   {boolean} showApplyToAll
   *          全てに適用チェックを表示するか
   * @param   {boolean} showSkip
   *          スキップを表示するか
   * @returns {Promise<object>}
   *          resolve: 今回の動作
   */
  showOverwriteConfirm: function(window, targetFile, filenameBase,
                                 showApplyToAll, showSkip) {
    let baseDir = targetFile.parent.clone();
    let lastModifiedDate = new Date(targetFile.lastModifiedTime);
    
    let confirmURL = "chrome://unmht/content/confirm.xul";
    let features = "chrome,dialog,centerscreen";
    let arg = {
      title:    UnMHTStringBundle.get("save.overwrite.title"),
      message1: UnMHTStringBundle.get("save.overwrite.message",
                                      [targetFile.leafName]),
      message2: (UnMHTStringBundle.get("save.overwrite.lastaccess")
                 + " "
                 + UnMHTFormatter.getReadableDate(lastModifiedDate)),
      message3: UnMHTStringBundle.get("save.overwrite.message2"),
      accept:   UnMHTStringBundle.get("replace"),
      extra2:   UnMHTStringBundle.get("abort"),
      rename:   UnMHTStringBundle.get("rename"),
      toall:    null,
      file:     this.getUniqueFile(baseDir, filenameBase),
      exist:    function(name) {
        return UnMHTStringBundle.get("save.overwrite.message", [name]);
      },
      focus:    "yes"
    };
    if (showSkip) {
      arg.extra1 = UnMHTStringBundle.get("skip");
    }
    if (showApplyToAll) {
      arg.toall = UnMHTStringBundle.get("save.overwrite.all");
    }
    return arWindowUtil.openDialog(window, confirmURL, "", features, arg);
  },
  
  /**
   * ユニークなファイルを返す
   *
   * @param   {nsIFile} baseDir
   *          親ディレクトリ
   * @param   {string} filenameBase
   *          ファイル名
   * @returns {nsIFile}
   *          ユニークなファイル
   */
  getUniqueFile: function(baseDir, filenameBase) {
    let dupInit = UnMHTPref.getPref("save.filename.dup.rename.init");
    let dupDigits = UnMHTPref.getPref("save.filename.dup.rename.digits");
    
    let padDup = function(n, dupDigits) {
      let s = n + "";
      while (s.length < dupDigits) {
        s = "0" + s;
      }
      return s;
    };
    
    let file
      = arFileUtil.checkAndAppend(baseDir, filenameBase,
                                  "", UnMHTFilename.defaultExtension);
    
    let i = dupInit;
    while (file.exists()) {
      file
        = arFileUtil.checkAndAppend(baseDir, filenameBase,
                                    "-" + padDup(i, dupDigits),
                                    UnMHTFilename.defaultExtension);
      i ++;
    }
    
    return file;
  },
  
  /**
   * 保存する際のファイル名を取得する
   *
   * @param   {nsIDOMDocument} targetDocument
   *          対象のドキュメント
   * @param   {object} scrapBookData
   *          ScrapBook のデータ
   * @param   {string} filenameType
   *          ファイル名の種類
   *            title/leafname/custom
   * @param   {Date} date
   *          日時
   *          カスタムファイル名に使用する
   * @returns {string}
   *          ファイル名 (拡張子以外)
   */
  getFilenameBase: function(targetDocument, scrapBookData, filenameType,
                            date) {
    let [uri, title]
      = this.getLocationAndTitle(targetDocument, scrapBookData);
    
    let decodedURI = uri;
    try {
      decodedURI = decodeURI(uri);
    } catch (e) {
    }
    
    let path = uri;
    let decodedPath = decodedURI;
    let re = uri.match(/^([^?]+)(\?.+)?(#.+)?$/, "");
    if (re) {
      path = re[1];
      
      try {
        decodedPath = decodeURI(path);
      } catch (e) {
      }
    }
    
    let localFile = { value: null };
    if (UnMHTPref.getPref("save.local.same_dir") &&
        UnMHTDocumentCategory.isLocalFile(decodedURI, localFile)) {
      return localFile.value.leafName.replace(/\.[^\.]+$/, "")
        + UnMHTPref.getPref("save.local.same_dir.append");
    }
    
    let filename = "";
    
    re = decodedPath.match(/\/([^\/]+)$/);
    if (re) {
      filename = re[1].replace(/\.[^\.]+$/, "");
    } else {
      filename = "index";
    }
    title = title || filename;
    
    if (!filenameType) {
      filenameType = UnMHTPref.getPref("save.filename");
    }
    if (UnMHTMailPrivacy.isMailURL(decodedURI) &&
        filenameType == "filename") {
      filenameType = "title";
    }
    
    switch (filenameType) {
      case "title": {
        filename = UnMHTFilename.validateFileName(title) || filename;
        break;
      }
      case "custom": {
        filename
          = UnMHTFilename.validateFileName(UnMHTCustomFilename.format(title, filename, uri, date));
        break;
      }
      default: {
        filename = UnMHTFilename.validateFileName(filename);
      }
    }
    
    return filename;
  },
  
  /**
   * ファイル名中の使用不可能な文字の置換、
   * 長すぎるファイル名の短縮、拡張子の追加をする
   *
   * @param   {nsIFile} file
   *          対象のファイル
   * @returns {Promise<nsIFile>}
   *          resolve: 保存可能なファイル
   *                   中断した場合は null
   */
  getValidFile: function(window, origFile) {
    return Task.spawn(function() {
      let leafName = UnMHTFilename.validateFileName(origFile.leafName);
      let file = origFile.parent.clone();
      file.append(leafName);
      
      if (UnMHTFilename.removeExtension(file.leafName) == file.leafName) {
        /* 拡張子 が無いと UnMHT でハンドルできなくなるので追加する */
        let filenameBase = file.leafName;
        file
          = arFileUtil.checkAndAppend(file.parent, filenameBase,
                                      "", UnMHTFilename.defaultExtension);
        if (file.exists()) {
          /* .mht を追加した事でファイル名が被った */
          let result = yield UnMHTSave.showOverwriteConfirm(window, file, filenameBase,
                                                            false, false);
          
          if (result.action == "abort" || result.action == "no") {
            file = null;
          } else if (result.action == "rename") {
            file = result.file;
          }
        }
      }
      
      throw new Task.Result(file);
    }.bind(this));
  },
  
  /**
   * ドキュメントのアドレスとタイトルを取得する
   *
   * @param   {nsIDOMDocument} targetDocument
   *          対象のドキュメント
   * @param   {object} scrapBookData
   *          ScrapBook のデータ
   * @returns Array
   *            [stirng アドレス, string タイトル]
   */
  getLocationAndTitle: function(targetDocument, scrapBookData) {
    if (scrapBookData) {
      return [scrapBookData["source"], scrapBookData["title"]];
    } else {
      return [targetDocument.location.href, targetDocument.title];
    }
  },
  
  /**
   * 保存状態を文字列表現から数値表現に変換する
   *
   * @param   {string} _str
   *          文字列表現
   * @returns {number}
   *          数値表現
   */
  stringToSaveState: function(_str) {
    return (arArrayUtil
            .find(this.saveStateList,
                  function([state, str]) str == _str) ||
            [this.SAVESTATE_UNKNOWN, "unknown"])[0];
  },
  
  /**
   * 保存状態を数値表現から文字列表現に変換する
   *
   * @param   {number} _state
   *          数値表現
   * @returns {string}
   *          文字列表現
   */
  saveStateToString: function(_state) {
    return (arArrayUtil
            .find(this.saveStateList,
                  function([state, str]) state == _state) ||
            [this.SAVESTATE_UNKNOWN, "unknown"])[1];
  },
  
  /**
   * マスクでフィルタした保存状態の数値表現のリストを返す
   *
   * @param   {number} mask
   *          保存状態のマスク
   * @returns {Array}
   *          保存状態の数値表現のリスト
   */
  getSaveStateList: function(mask) {
    return this.saveStateList
      .filter(function([state, str]) state & mask)
      .map(function([state, str]) state);
  },
  
  /**
   * 保存状態の数値表現と追加ファイルの有無から
   * ファイルタイプの文字列を作成する
   *
   * @param   {number} saveState
   *          保存状態の数値表現
   * @param   {boolean} useAdd 
   *          追加ファイルの有無
   * @returns {string}
   *          ファイルタイプ
   */
  makeFiletype: function(saveState, useAdd) {
    let filetype = UnMHTStringBundle.get("save.filetype");
    
    switch (saveState) {
      case UnMHTSave.SAVESTATE_CURRENT: {
        filetype += " " + UnMHTStringBundle.get("save.state.current");
        break;
      }
      case UnMHTSave.SAVESTATE_CURRENT_NOJS: {
        filetype += " " + UnMHTStringBundle.get("save.state.current")
          + " (" + UnMHTStringBundle.get("save.nojs") + ")";
        break;
      }
      case UnMHTSave.SAVESTATE_ORIGINAL: {
        filetype += " " + UnMHTStringBundle.get("save.state.original");
        break;
      }
    }
    
    if (useAdd) {
      filetype += " + " + UnMHTStringBundle.get("save.add");
    }
    
    return filetype;
  },
  
  /**
   * 追加 URI を解析する
   *
   * @param   {string} text
   *          行ごとに URI を記述したテキスト
   * @returns {Array.<string>}
   *          URI リスト
   */
  parseAdditionalURIs: function(text) {
    return text.split(/[\r\n]/)
      .map(function(line) line.replace(/^\s*|\s*$/g, ""))
      .filter(function(uri) uri);
  },
  
  /**
   * エラーの理由から結果定数に変換する
   *
   * @param   {number} reason
   *          エラーの理由
   * @returns {number}
   *          結果定数
   */
  errorReasonToResult: function(reason) {
    let result = UnMHTSave.RESULT_FAIL;
    switch (reason) {
      case UnMHTSave.ERROR_WRITE_FILE: {
        result = UnMHTSave.RESULT_ERROR_WRITE_FILE;
        break;
      }
      case UnMHTSave.ERROR_REQUIRED_FAILED: {
        result = UnMHTSave.RESULT_ERROR_REQUIRED_FAILED;
        break;
      }
    }
    return result;
  },

  /**
   * エラーの理由からメッセージに変換する
   *
   * @param   {number} reason
   *          エラーの理由
   * @returns {string}
   *          メッセージ
   */
  errorReasonToMessage: function(reason) {
    let message = "";
    switch (reason) {
      case UnMHTSave.ERROR_WRITE_FILE: {
        message = UnMHTStringBundle.get("save.error.open");
        break;
      }
      case UnMHTSave.ERROR_REQUIRED_FAILED: {
        message = UnMHTStringBundle.get("save.error.required");
        break;
      }
    }
    return message;
  },
  
  /* ==== private ==== */
  
  _contentPolicy: null /** ブロックチェック用
                        *  @type {nsIContentPolicy} */
};
UnMHTSave.init();
