"use strict";

let { classes: Cc, interfaces: Ci, results: Cr, utils: Cu } = Components;
Cu.import("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative(this, "base/arDialog.jsm");
XPCOMUtils.importRelative(this, "base/arPrototype.jsm");
XPCOMUtils.importRelative(this, "base/arTimer.jsm");
XPCOMUtils.importRelative(this, "UnMHTPref.jsm");
XPCOMUtils.importRelative(this, "UnMHTSave.jsm");
XPCOMUtils.importRelative(this, "UnMHTSaveProgress.jsm");
XPCOMUtils.importRelative(this, "UnMHTStringBundle.jsm");

let EXPORTED_SYMBOLS = ["UnMHTSaveDialogClass"];

/**
 * 保存ダイアログ
 *
 * ウィンドウの引数 {object}
 *   {
 *     saveState:       {number}
 *                      保存状態 (必須)
 *     saveContext:     {UnMHTSaveContext}
 *                      対象の保存コンテキスト (必須)
 *   }
 *
 * 結果 {Promise<IUnMHTSaveListener>}
 *   resolve: イベントリスナ
 *
 * @param   {nsIDOMWindow} window
 *          対象のウィンドウ
 * @extends    arDialog
 * @implements arIDialog
 * @implements IUnMHTSaveProgressOwner
 */
function UnMHTSaveDialogClass(window) {
  arDialog.call(this, window);
}
UnMHTSaveDialogClass.prototype = {
  /* ==== arIDialog インターフェース ==== */
  
  /**
   * 破棄
   */
  destruct: function() {
    this._saveContext = null;
    arDialog.prototype.destruct.call(this);
  },
  
  /**
   * ロードイベント
   */
  onLoad: function() {
    let arg = this._window.arguments[0];
    
    let title = UnMHTStringBundle.get("save.title");
    switch (arg.saveState) {
      case UnMHTSave.SAVESTATE_CURRENT: {
        title += " " + UnMHTStringBundle.get("save.state.current");
        break;
      }
      case UnMHTSave.SAVESTATE_CURRENT_NOJS: {
        title += " " + UnMHTStringBundle.get("save.state.current")
          + " (" + UnMHTStringBundle.get("save.nojs") + ")";
        break;
      }
      case UnMHTSave.SAVESTATE_ORIGINAL: {
        title += " " + UnMHTStringBundle.get("save.state.original");
        break;
      }
    }

    let dialog = this.el("unmht_save_dialog");
    dialog.setAttribute("title", title);
    
    dialog.getButton("accept").disabled = true;
    
    this.result.resolve(new UnMHTSaveProgress(this._window, this));
    
    this._saveContext = arg.saveContext;
  },
  
  /**
   * ダイアログをキャンセルしたイベント
   *
   * @returns {boolean}
   *          ダイアログを閉じるか
   */
  onDialogCancel: function(context) {
    if (!this._saveContext) {
      return true;
    }
    if (this._saveContext) {
      this._saveContext.cancel();
      this._saveContext = null;
      
      return false;
    }
    
    return true;
  },
  
  /* ==== IUnMHTSaveProgressOwner インターフェース ===== */
  
  /**
   * キャンセルが完了したイベント
   *
   * @param   {UnMHTSaveProgress} progress
   *          進捗状況表示オブジェクト
   * @param   {UnMHTSaveContext} context
   *          対象の保存コンテキスト
   */
  onCancel: function(progress, context) {
    if (this._window) {
      this._window.close();
    }
  },
  
  /**
   * 保存失敗イベント
   *
   * @param   {UnMHTSaveProgress} progress
   *          進捗状況表示オブジェクト
   * @param   {UnMHTSaveContext} context
   *          対象の保存コンテキスト
   * @param   {number} reason
   *          エラーの理由
   * @param   {Exception} e
   *          例外
   */
  onError: function(progress, context, reason, e) {
  },
  
  /**
   * 保存完了
   *
   * @param   {UnMHTSaveProgress} progress
   *          進捗状況表示オブジェクト
   * @param   {UnMHTSaveContext} context
   *          対象の保存コンテキスト
   * @param   {Array.<string>} failedList
   *          失敗したファイル名
   */
  onSave: function(progress, context, failedList) {
    let dialog = this.el("unmht_save_dialog");
    
    dialog.getButton("accept").disabled = false;
    dialog.getButton("accept").focus();
    dialog.getButton("cancel").disabled = true;
    
    if (UnMHTPref.getPref("save.show_errors") && failedList.length > 0) {
      progress.showFail(failedList);
    } else {
      let window = this._window;
      arTimer.setTimeout(function() {
        window.close();
      }, 1000);
    }
  },
  
  /* ==== private ==== */
  
  _saveContext: null /** 保存コンテキスト
                      *  @type {UnMHTSaveContext} */
};
arPrototype.extend(UnMHTSaveDialogClass, arDialog);
