"use strict";

let { classes: Cc, interfaces: Ci, results: Cr, utils: Cu } = Components;
Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative(this, "Promise.jsm");
XPCOMUtils.importRelative(this, "Task.jsm");

let EXPORTED_SYMBOLS = ["arFilePicker", "arFilePickerClass"];

/**
 * ファイル選択ダイアログ
 */
function arFilePickerClass() {
}
arFilePickerClass.prototype = {
  /**
   * ファイル選択ダイアログを非同期で開く
   *
   * @param   {object} params
   *          パラメータ
   * @throws  {Error}
   *          params.window が無い場合
   */
  asyncOpen: function(params) {
    if (!("window" in params
          && params.window)) {
      throw new Error("params.window must not be null");
    }
    let window = params.window;
    
    let mode = ("mode" in params) ? params.mode : Ci.nsIFilePicker.modeOpen;
    let callback = ("callback" in params) ? params.callback : null;
    
    Task.spawn(function() {
      let result, filePicker;
      
      switch (mode) {
        case Ci.nsIFilePicker.modeOpen: {
          [result, filePicker] = yield this.getOpenFile(window, params);
          break;
        }
        case Ci.nsIFilePicker.modeSave: {
          [result, filePicker] = yield this.getSaveFile(window, params);
          break;
        }
        case Ci.nsIFilePicker.modeGetFolder: {
          [result, filePicker] = yield this.getFolder(window, params);
          break;
        }
        case Ci.nsIFilePicker.modeOpenMultiple: {
          [result, filePicker] = yield this.getOpenFiles(window, params);
          break;
        }
      }
      
      if (callback) {
        callback(result, Ci.nsIFilePicker.returnOK, filePicker);
      }
    }.bind(this)).then(null, function(e) {
      if (callback) {
        callback(null, e, null);
      }
    });
  },
  
  /**
   * 開くファイルを取得する
   *
   * @param   {nsIDOMWindow} window
   *          親ウィンドウ
   * @param   {object} params
   *          パラメータ
   * @returns {Promise<[nsIFile,nsIFilePicker]>}
   *          resolve: 開くファイルとファイルピッカー
   *                   キャンセルの場合はファイルが null
   */
  getOpenFile: function(window, params) {
    let getResult = function(filePicker, uri) {
      let file = filePicker.file;
      
      this.setLastOpenDir(window, file.parent);
      
      return file;
    }.bind(this);
    return this._getFileHelper(window, params, Ci.nsIFilePicker.modeOpen,
                               getResult);
  },
  
  /**
   * 保存するファイルを取得する
   *
   * @param   {nsIDOMWindow} window
   *          親ウィンドウ
   * @param   {object} params
   *          パラメータ
   * @returns {Promise<[nsIFile,nsIFilePicker]>}
   *          resolve: 保存するファイルとファイルピッカー
   *                   キャンセルの場合はファイルが null
   */
  getSaveFile: function(window, params) {
    let getResult = function(filePicker, uri) {
      let file = filePicker.file;
      this.setLastSaveDir(window, uri, file.parent);
      
      return file;
    }.bind(this);
    return this._getFileHelper(window, params, Ci.nsIFilePicker.modeSave,
                               getResult);
  },

  /**
   * フォルダを取得する
   *
   * @param   {nsIDOMWindow} window
   *          親ウィンドウ
   * @param   {object} params
   *          パラメータ
   * @returns {Promise<[nsIFile,nsIFilePicker]>}
   *          resolve: フォルダとファイルピッカー
   *                   キャンセルの場合はフォルダが null
   */
  getFolder: function(window, params) {
    let getResult = function(filePicker, uri) {
      let dir = filePicker.file;
      
      this.setLastOpenDir(window, dir);
      
      return dir;
    }.bind(this);
    return this._getFileHelper(window, params, Ci.nsIFilePicker.modeGetFolder,
                               getResult);
  },
  
  /**
   * 複数の開くファイルを取得する
   *
   * @param   {nsIDOMWindow} window
   *          親ウィンドウ
   * @param   {object} params
   *          パラメータ
   * @returns {Promise<[Array.<nsIFile>,nsIFilePicker]>}
   *          resolve: 開くファイルの配列とファイルピッカー
   *                   キャンセルの場合はファイルの配列が null
   */
  getOpenFiles: function(window, params) {
    let getResult = function(filePicker, uri) {
      let files
        = [file for (file of XPCOMUtils.IterSimpleEnumerator(
          filePicker.files, Ci.nsIFile))];
      if (files.length) {
        this.setLastOpenDir(window, files[0].parent);
      }
      
      return files;
    }.bind(this);
    return this._getFileHelper(window, params, Ci.nsIFilePicker.modeOpenMultiple,
                               getResult);
  },
  
  /**
   * 開くのに最後に使用したディレクトリを取得する
   *
   * @param   {nsIDOMWindow} window
   *          親ウィンドウ
   * @returns {nsIFile}
   *          最後に使用したディレクトリ
   *          無い場合は null
   */
  getLastOpenDir: function(window) {
    let name = "browser.open.lastDir";
    if (Services.prefs.getPrefType(name) != 0) {
      return Services.prefs.getComplexValue(name, Ci.nsILocalFile);
    }
    
    return null;
  },
  
  /**
   * 開くのに最後に使用したディレクトリを設定する
   *
   * @param   {nsIDOMWindow} window
   *          親ウィンドウ
   * @param   {nsIFile} dir
   *          最後に使用したディレクトリ
   */
  setLastOpenDir: function(window, dir) {
    Services.prefs.setComplexValue("browser.open.lastDir",
                                   Ci.nsILocalFile, dir);
  },
  
  /**
   * 保存するのに最後に使用したディレクトリを取得する
   *
   * @param   {nsIDOMWindow} window
   *          親ウィンドウ
   * @param   {string} uri
   *          保存対象の URI
   *          不要ならば null
   * @returns {nsIFile}
   *          最後に使用したディレクトリ
   *          無い場合は null
   */
  getLastSaveDir: function(window, uri) {
    let context = {};
    Cu.import("resource://gre/modules/DownloadLastDir.jsm", context);
    
    if ("DownloadLastDir" in context) {
      return (new context.DownloadLastDir(window)).getFile(uri);
    } else if ("gDownloadLastDir" in context) {
      /* COMPAT: Firefox 18 以前 */
      return context.gDownloadLastDir.getFile(uri);
    }
    
    return null;
  },
  
  /**
   * 保存するのに最後に使用したディレクトリを設定する
   *
   * @param   {nsIDOMWindow} window
   *          親ウィンドウ
   * @param   {string} uri
   *          保存対象の URI
   *          不要ならば null
   * @param   {nsIFile} dir
   *          最後に使用したディレクトリ
   */
  setLastSaveDir: function(window, uri, dir) {
    let context = {};
    Cu.import("resource://gre/modules/DownloadLastDir.jsm", context);
    
    if ("DownloadLastDir" in context) {
      new context.DownloadLastDir(window).setFile(uri, dir);
    } else if ("gDownloadLastDir" in context) {
      /* COMPAT: Firefox 18 以前 */
      context.gDownloadLastDir.setFile(uri, dir);
    }
  },
  
  /* ==== private ==== */

  /**
   * ファイル選択ダイアログを初期化する
   *
   * @param   {nsIDOMWindow} window
   *          親ウィンドウ
   * @param   {object} params
   *          パラメータ
   *            {
   *              title: {string}
   *                     ダイアログのタイトル (オプショナル),
   *              useLastDir: {boolean}
   *                      最後に使用したディレクトリを使用/保存する
   *                      (オプショナル)
   *                      デフォルトは false,
   *              displayDirectory: {nsIFile}
   *                      useLastDir が false の際に表示するディレクトリ
   *                      (オプショナル)
   *                      ファイルを指している場合は親ディレクトリになる,
   *              uri: {string}
   *                   保存対象の URI (オプショナル)
   *                   サイトごとに記憶する際に使用する
   *                   デフォルトは null,
   *              defaultString: {string}
   *                             デフォルトファイル名 (オプショナル),
   *              defaultextension: {string}
   *                                デフォルト拡張子 (オプショナル),
   *              filter: {Array.<([string,string]|number)>}
   *                      ファイルタイプ (オプショナル)
   *                        [type, ...]
   *                          type: [string 名前, string 拡張子]
   *                                または
   *                                number filterMask
   *              filterIndex: {number}
   *                           デフォルトのファイルタイプの
   *                           インデックス (オプショナル),
   *            }
   * @param   {number} mode
   *          nsIFilePicker のモード
   *            Ci.nsIFilePicker.mode*
   * @param   {nsIURI} uri
   *          保存対象の URI (保存の場合のみ)
   */
  _initFilePicker: function(window, params, mode, uri) {
    let getParam = function(name, defValue) {
      return (name in params) ? params[name] : defValue;
    };
    
    let title            = getParam("title", "");
    let useLastDir       = getParam("useLastDir", false);
    let displayDirectory = getParam("displayDirectory", null);
    let defaultString    = getParam("defaultString", null);
    let defaultExtension = getParam("defaultExtension", null);
    let filter           = getParam("filter", null);
    let filterIndex      = getParam("filterIndex", -1);
    
    let filePicker
      = Cc["@mozilla.org/filepicker;1"].createInstance(Ci.nsIFilePicker);
    
    filePicker.init(window, title, mode);
    
    let dir = null;
    if (useLastDir) {
      if (mode == Ci.nsIFilePicker.modeSave) {
        dir = this.getLastSaveDir(window, uri);
      } else {
        dir = this.getLastOpenDir(window);
      }
    } else if (displayDirectory) {
      dir = displayDirectory.clone();
      if (dir && dir.exists()) {
        if (!dir.isDirectory()) {
          dir = dir.parent;
        }
      } else {
        dir = null;
      }
    }
    if (dir) {
      filePicker.displayDirectory = dir;
    }
    
    if (filter) {
      filter.forEach(function(type) {
        if (type.length == 1) {
          let [mask] = type;
          filePicker.appendFilters(mask);
        } else {
          let [name, ext] = type;
          filePicker.appendFilter(name, ext);
        }
      });
    }
    if (filterIndex >= 0 && filterIndex < filter.length) {
      filePicker.filterIndex = filterIndex;
    }
    
    if (defaultString) {
      filePicker.defaultString = defaultString;
    }
    if (defaultExtension) {
      filePicker.defaultExtension = defaultExtension;
    }
    
    return filePicker;
  },
  
  /**
   * ファイル選択ダイアログを初期化する
   *
   * @param   {nsIDOMWindow} window
   *          親ウィンドウ
   * @param   {object} params
   *          パラメータ
   * @param   {number} mode
   *          nsIFilePicker のモード
   *            Ci.nsIFilePicker.mode*
   * @param   {Function} getResult
   *          結果を返す関数
   *            function(filePicker, uri) 
   *              *param   {nsIFilePicker} filePicker
   *                       使用したファイル選択ダイアログ
   *              *param   {nsIURI} uri
   *                       保存対象の URI (保存の場合のみ)
   * @returns {Promise<[ANY,nsIFilePicker]>}
   *          resolve: 結果とファイルピッカー
   *                   キャンセルの場合は結果が null
   */
  _getFileHelper: function(window, params, mode, getResult) {
    let deferred = Promise.defer();
    
    let uri = null;
    if ("uri" in params) {
      try {
        uri = Services.io.newURI(params.uri, null, null);
      } catch (e) {
      }
    }
    
    let filePicker = this._initFilePicker(window, params, mode, uri);
    
    let callback = {
      QueryInterface: XPCOMUtils.generateQI([Ci.nsIFilePickerShownCallback]),
      
      done: function(result) {
        if (!(result == Ci.nsIFilePicker.returnOK ||
              result == Ci.nsIFilePicker.returnReplace)) {
          deferred.resolve([null, filePicker]);
          return;
        }
        
        deferred.resolve([getResult(filePicker, uri), filePicker]);
      }.bind(this)
    };
    filePicker.open(callback);
    
    return deferred.promise;
  }
};

let arFilePicker = new arFilePickerClass();
