"use strict";

let { classes: Cc, interfaces: Ci, results: Cr, utils: Cu } = Components;
Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative(this, "Promise.jsm");
XPCOMUtils.importRelative(this, "arAppInfo.jsm");
XPCOMUtils.importRelative(this, "arTimer.jsm");
XPCOMUtils.defineLazyGetter(this, "arArrayUtil", function() {
  XPCOMUtils.importRelative(this, "arArrayUtil.jsm");
  return arArrayUtil;
});

let EXPORTED_SYMBOLS = ["arWindowUtil"];

/**
 * Window 操作ユーティリティ
 */
let arWindowUtil = {
  /**
   * ブラウザを取得する
   *
   * @param   {nsIDOMWindow} window
   *          実行中のウィンドウ
   * @returns {(nsIDOMXULElement|BrowserApp)}
   *          ブラウザ (tabbrowser もしくは browser もしくは BrowserApp)
   *          見付からない場合は null
   */
  getBrowser: function(window) {
    if (arAppInfo.isFennec) {
      return window.BrowserApp;
    }
    
    if (this.isMailWindow(window)) {
      return window.document.getElementById("messagepane");
    }
    
    return window.document.getElementById("content");
  },
  
  /**
   * 現在開いているドキュメントを取得する
   *
   * @param   {nsIDOMWindow} window
   *          実行中のウィンドウ
   * @returns {nsIDOMDocument}
   *          現在開いているドキュメント
   *          見付からない場合は null
   */
  getCurrentDocument: function(window) {
    let browser = this.getBrowser(window);
    if (!browser) {
      return null;
    }
    if (arAppInfo.isFennec) {
      return browser.selectedTab.browser.contentDocument;
    } else {
      return browser.contentDocument;
    }
  },
  
  /**
   * ウィンドウのタブの配列を取得する
   *
   * @param   {nsIDOMWindow} window
   *          対象のウィンドウ
   * @returns {Array.<nsIDOMXULElement>}
   *          タブの配列
   */
  getTabs: function(window) {
    let browser = this.getBrowser(window);
    if (!browser) {
      return [];
    }
    
    return ("visibleTabs" in browser)
      ? browser.visibleTabs
      : browser.tabs;
  },

  /**
   * ウィンドウのブラウザの配列を取得する
   *
   * @param   {nsIDOMWindow} window
   *          対象のウィンドウ
   * @returns {Array.<nsIDOMXULElement>}
   *          ブラウザの配列
   */
  getBrowsers: function(window) {
    let browser = this.getBrowser(window);
    if (!browser) {
      return [];
    }
    
    if (arAppInfo.isFennec) {
      return Array
        .map(this.getTabs(window),
             function(tab) tab.browser);
    } else {
      return Array
        .map(this.getTabs(window),
             function(tab) tab.linkedBrowser);
    }
  },

  /**
   * ウィンドウのドキュメントの配列を取得する
   *
   * @param   {nsIDOMWindow} window
   *          対象のウィンドウ
   * @returns {Array.<nsIDOMDocument>}
   *          ドキュメントの配列
   */
  getDocuments: function(window) {
    return Array
      .map(this.getBrowsers(window),
           function(browser) browser.contentDocument);
  },
  
  /**
   * ドキュメントに対応するタブを返す
   *
   * @param   {nsIDOMWindow} window
   *          対象のウィンドウ
   * @param   {nsIDOMDocument} targetDocument
   *          対象のドキュメント
   * @returns {nsIDOMXULElement}
   *          対応するタブ
   *          見付からなかったら null
   */
  getTabForDocument: function(window, targetDocument) {
    let browser = this.getBrowser(window);
    if (!browser) {
      return null;
    }
    
    if (arAppInfo.isFennec) {
      return arArrayUtil
        .find(browser.tabs,
              function(tab) (tab.browser &&
                             tab.browser.contentDocument == targetDocument)) || null;
    } else {
      return arArrayUtil
        .find(browser.tabs,
              function(tab) (tab.linkedBrowser &&
                             tab.linkedBrowser.contentDocument == targetDocument)) || null;
    }
  },
  
  /**
   * ドキュメントに対応するタブを閉じる
   *
   * @param   {nsIDOMWindow} window
   *          対象のウィンドウ
   * @param   {nsIDOMDocument} targetDocument
   *          対象のドキュメント
   */
  closeTabForDocument: function(window, targetDocument) {
    let tab = arWindowUtil.getTabForDocument(window, targetDocument);
    if (tab) {
      let browser = arWindowUtil.getBrowser(window);
      if (arAppInfo.isFennec) {
        browser.closeTab(tab);
      } else {
        browser.removeTab(tab);
      }
    }
  },
  
  /**
   * メッセージパネルが閉じているかどうか調べる
   *
   * @param   {nsIDOMWindow} window
   *          対象のウィンドウ
   * @returns {boolean}
   *          メッセージパネルが閉じているか
   *          メッセージパネルが無い場合は false
   */
  isMessagePaneCollapsed: function(window) {
    if (!this.isMailWindow(window)) {
      return false;
    }
    
    let node;
    node = window.document.getElementById("messagepanebox");
    if (node && node.collapsed) {
      return true;
    }
    node = window.document.getElementById("messagepaneboxwrapper");
    if (node && node.collapsed) {
      return true;
    }
    
    return false;
  },
  
  /**
   * メールのウィンドウかどうかを返す
   *
   * @param   {nsIDOMWindow} window
   *          対象のウィンドウ
   * @returns {boolean}
   *          メールのウィンドウかどうか
   */
  isMailWindow: function(window) {
    return (window.document.documentURI
            == "chrome://messenger/content/messenger.xul");
  },
  
  /**
   * ページのロードイベントのリスナを追加する
   *
   * @param   {nsIDOMWindow} window
   *          対象のウィンドウ
   * @param   {Function} func
   *          イベントリスナ
   */
  addPageLoadListener: function(window, func) {
    if (arAppInfo.isFennec) {
      let BrowserApp = window.BrowserApp;
      if (BrowserApp) {
        BrowserApp.deck.addEventListener("load", func, true);
      }
    } else {
      let appcontent = window.document.getElementById("appcontent");
      if (appcontent) {
        appcontent.addEventListener("DOMContentLoaded", func, false);
      }
    }
  },
  
  /**
   * ページのロードイベントのリスナを削除する
   *
   * @param   {nsIDOMWindow} window
   *          対象のウィンドウ
   * @param   {Function} func
   *          イベントリスナ
   */
  removePageLoadListener: function(window, func) {
    if (arAppInfo.isFennec) {
      let BrowserApp = window.BrowserApp;
      if (BrowserApp) {
        BrowserApp.deck.removeEventListener("load", func, true);
      }
    } else {
      let appcontent = window.document.getElementById("appcontent");
      if (appcontent) {
        appcontent.removeEventListener("DOMContentLoaded", func, false);
      }
    }
  },
  
  /**
   * URL で指定されたウィンドウを探す
   *
   * @param   {string} url
   *          探すウィンドウの URL
   * @returns {nsIDOMWindow}
   *          URL に一致するウィンドウ
   */
  find: function(url) {
    for (let window of XPCOMUtils.IterSimpleEnumerator
         (Services.wm.getEnumerator(null), Ci.nsIDOMWindow)) {
      if (window.document.documentURI == url) {
        return window;
      }
    }
    
    return null;
  },
  
  /**
   * URL で指定されたウィンドウにフォーカスする
   *
   * @param   {string} url
   *          フォーカスするウィンドウの URL
   * @returns {boolean}
   *          見付かったか
   */
  focus: function(url) {
    let window = this.find(url);
    if (window) {
      window.focus();
      return true;
    }
    
    return false;
  },

  /**
   * ダイアログを開く
   *
   * @param    {nsIDOMWindow} window
   *           実行中のウィンドウ
   * @param    {string} url
   *           ダイアログの URL
   * @param    {string} name
   *           ダイアログの名前
   * @param    {string} options
   *           ダイアログのオプション
   * @param    {nsISupport} arg
   *           ダイアログの引数
   * @returns  {Promise<nsISupport>}
   *           resolve: ダイアログの結果
   *           reject: {ANY}
   */
  openDialog: function(window, url, name, features, arg) {
    let deferred = Promise.defer();
    
    window.openDialog(url, name, features, arg, deferred);
    
    return deferred.promise;
  }
};
